/*
 * Copyright 1988 by the Radio Amateur Telecommunications Society
 * and Thomas A. Moulton, W2VY
 *
 * This software may only be modified, copied, distributed or
 * executed for non-profit purposes by individuals operating
 * systems in the Amateur Radio Service.  Credit to the
 * author(s) and to the Radio Amateur Telecommunications Society
 * must be made in modules where RATS provided software is used,
 * and in any announcements and documentation.  
 *
 * As a non-profit, research and development organization,  the
 * Radio Amateur Telecommunications Society distributes software
 * in both executable and source forms.  This policy is in place
 * to encourage the development and distribution of OSI-based,
 * networking tools.  In order to protect the interests of the
 * Society and the authors, we have placed some conditions
 * of use on the software.  Other groups are encouraged
 * to place the same or similar guidelines on
 * software they produce.
 *
 * The Radio Amateur Telecommunications Society reserves the right
 * to specify and alter the terms under which software provided by
 * the Society may be used.  This policy is consistent with the 
 * objective of uniform and consistent "Open Systems Interconnections."
 * 
 * All acceptable Amateur Radio related uses of this software
 * will be outlined in the "ROSE Implementer's Guide".  Individuals
 * or organizations wishing to add to, or modify the provisions of
 * the guide to accommodate local or evolutionary requirements
 * should document the proposed change(s) and forward them to the
 * Society.  If accepted, written notification will be provided by
 * the Society to the submitting organization or individual(s).
 * The Society will then issue a "ROSE Implementer's Guide Change
 * Notice".  Periodically, the Society will re-issue the "ROSE 
 * Implementer's Guide" and incorporate the text of the change 
 * notices.  This procedure has been put in to place to ensure
 * compatibility between systems and to ensure their "Openness"
 * and interoperability.
 *
 * No part of this software may be used in other packages 
 * without prior authorization from the author or the Society.  
 * Software incorporating this module, all or in part, must be 
 * provided to the Society prior to distribution or use by
 * anyone not directly involved in testing of the revised  
 * environment.  Current releases of the combined software must
 * be provided to the Society in both source and executable
 * forms.  Adequate documention to produce an executable module 
 * from the provided source must also be included.
 *
 * Non-Amateur Radio non-profit uses may be authorized on a case
 * by case basis.  Inquiries for such use may be made in writing
 * to the Society. Non-commercial uses consistent with the
 * general principles of Open Systems Interconnection Reference
 * Model will be generally considered with favor.
 *
 * Commercial licensing of the software is also available based
 * on normal commercial terms.  Licensing inquiries should be
 * directed to the Society.  Commercial licensing of the standard
 * software will be done in situations which materially benefit
 * the Amateur Radio Packet Network.  Additional licensing is
 * reserved by the individual authors.
 * 
 * The Radio Amateur Telecommunications Society provides this software
 * on an "as is" basis.  The Society assumes no liability for
 * loss incurred through the use of this software.  Amateur Radio
 * use of this software implies non-commercial and voluntary 
 * development, deployment and use of this software in a "Amateur",
 * non-commercial service.  Commercial users are encouraged to
 * inspect their copies of the source code.  Source code modification
 * licenses are available if a combined Object and Source Code
 * license was not originally established.
 * 
 * The Society may be contacted by writing or calling at:
 * 
 * The Radio Amateur Telecommunications Society 
 * 206 North Vivyen Street.
 * Bergenfield, New Jersey 07621
 *
 * Telephone: 201-387-8896
 *
 */
#include "data.h"
#include "buffer.h"
#include "iface.h"
#include "timer.h"
#include "ax25.h"
#include "ax25l2.h"
#include "l3struc.h"
#include "x25cause.h"
#include "l3calls.h"
#include "tx.h"
#include "config.h"
#include "upfcn.h"

int NULLFCN(), clr_aftcpy(), con_aftcpy(), recv_boot();
int send_aftcpy(), rst_boot(), unl_aftcpy(), init_aftcpy();

struct upfcn aftcpy={
	{{0x82, 0x8c, 0xa8, 0x86, 0xa0, 0xb2}, {0x00}}, /* AFTCPY-0 */
	rst_boot, clr_aftcpy, con_aftcpy, recv_boot,
	send_aftcpy, init_aftcpy, unl_aftcpy,
	"AFTCPY - ROSE Switch AFT to HDLC Copy Program\r"
};

extern struct interface *ifaces[4];
extern struct axcb *links;

struct interface *p0, *p1, Port0, Port1;

int
con_aftcpy(vc)
register struct VCS *vc;
{
	vc=vc->peer;
	if (!vc) return;
	set_p(vc,P6,Number_Busy);
}

int
clr_aftcpy(vc,c)
struct VCS *vc;
int c;
{
	set_p(vc, P1, 0);
}

int
send_aftcpy(vc, pkt)
register struct VCS *vc;
struct datastr *pkt;
{
	vc_queue_data(vc,pkt);
	free_queue(&vc->tx_queue);
	set_p(vc, P6, DTE_Orig);
	return 256;	/* We are never busy! */
}

struct datastr *
rx0(ifx, pkt)
struct interface *ifx;
struct datastr *pkt;
{
	static struct datastr *bp, *bp1;
	static int ch;

	bp = new_buffer(buflen(pkt));
	bp1 = dup_pkt(pkt);
	while ((ch=bgetch(&bp1)) != EOF) bappch(bp, ch);
	pkt = (*p0->recv)(ifx, pkt);
	if (pkt) /* We did not use the frame, pass it to the other port */ {
		free_pkt(pkt);
		(*ifaces[1]->send)(ifaces[1], bp);
	} else free_pkt(bp);
	return NULL;
}

struct datastr *
rx1(ifx, pkt)
struct interface *ifx;
struct datastr *pkt;
{
	static struct datastr *bp, *bp1;
	static int ch;

	bp = new_buffer(buflen(pkt));
	bp1 = dup_pkt(pkt);
	while ((ch=bgetch(&bp1)) != EOF) bappch(bp, ch);
	pkt = (*p1->recv)(ifx, pkt);
	if (pkt) /* We did not use the frame, pass it to the other port */ {
		free_pkt(pkt);
		(*ifaces[0]->send)(ifaces[0], bp);
	} else free_pkt(bp);
	return NULL;
}

cpy_iface(ifout, ifin)
struct interface *ifout, *ifin;
{
	ifout->dev = ifin->dev;
	ifout->name = ifin->name;
	ifout->send = ifin->send;
	ifout->recv = ifin->recv;
	ifout->poll = ifin->poll;
	ifout->open = ifin->open;
	ifout->close = ifin->close;
}

init_aftcpy()
{
	static struct axcb *axcb;

	axcb = links;
	while (axcb) /* Fix all iface's of all links */ {
		if (axcb->iface == ifaces[0]) axcb->iface = &Port0;
		if (axcb->iface == ifaces[1]) axcb->iface = &Port1;
		axcb=axcb->next;
	}

	cpy_iface(&Port0, ifaces[0]);
	Port0.recv = rx0;
	p0 = ifaces[0];
	ifaces[0] = &Port0;
	cpy_iface(&Port1, ifaces[1]);
	Port1.recv = rx1;
	p1 = ifaces[1];
	ifaces[1] = &Port1;
}

unl_aftcpy()
{
	static struct axcb *axcb;

	axcb = links;
	while (axcb) /* Restore any links to the correct interface */ {
		if (axcb->iface == &Port0) axcb->iface = p0;
		if (axcb->iface == &Port1) axcb->iface = p1;
		axcb = axcb->next;
	}
	ifaces[0] = p0;
	ifaces[1] = p1;
	return 1;
}
