/*
 * Copyright 1988 by the Radio Amateur Telecommunications Society
 * and Thomas A. Moulton, W2VY
 *
 * This software may only be modified, copied, distributed or
 * executed for non-profit purposes by individuals operating
 * systems in the Amateur Radio Service.  Credit to the
 * author(s) and to the Radio Amateur Telecommunications Society
 * must be made in modules where RATS provided software is used,
 * and in any announcements and documentation.  
 *
 * As a non-profit, research and development organization,  the
 * Radio Amateur Telecommunications Society distributes software
 * in both executable and source forms.  This policy is in place
 * to encourage the development and distribution of OSI-based,
 * networking tools.  In order to protect the interests of the
 * Society and the authors, we have placed some conditions
 * of use on the software.  Other groups are encouraged
 * to place the same or similar guidelines on
 * software they produce.
 *
 * The Radio Amateur Telecommunications Society reserves the right
 * to specify and alter the terms under which software provided by
 * the Society may be used.  This policy is consistent with the 
 * objective of uniform and consistent "Open Systems Interconnections."
 * 
 * All acceptable Amateur Radio related uses of this software
 * will be outlined in the "ROSE Implementer's Guide".  Individuals
 * or organizations wishing to add to, or modify the provisions of
 * the guide to accommodate local or evolutionary requirements
 * should document the proposed change(s) and forward them to the
 * Society.  If accepted, written notification will be provided by
 * the Society to the submitting organization or individual(s).
 * The Society will then issue a "ROSE Implementer's Guide Change
 * Notice".  Periodically, the Society will re-issue the "ROSE 
 * Implementer's Guide" and incorporate the text of the change 
 * notices.  This procedure has been put in to place to ensure
 * compatibility between systems and to ensure their "Openness"
 * and interoperability.
 *
 * No part of this software may be used in other packages 
 * without prior authorization from the author or the Society.  
 * Software incorporating this module, all or in part, must be 
 * provided to the Society prior to distribution or use by
 * anyone not directly involved in testing of the revised  
 * environment.  Current releases of the combined software must
 * be provided to the Society in both source and executable
 * forms.  Adequate documention to produce an executable module 
 * from the provided source must also be included.
 *
 * Non-Amateur Radio non-profit uses may be authorized on a case
 * by case basis.  Inquiries for such use may be made in writing
 * to the Society. Non-commercial uses consistent with the
 * general principles of Open Systems Interconnection Reference
 * Model will be generally considered with favor.
 *
 * Commercial licensing of the software is also available based
 * on normal commercial terms.  Licensing inquiries should be
 * directed to the Society.  Commercial licensing of the standard
 * software will be done in situations which materially benefit
 * the Amateur Radio Packet Network.  Additional licensing is
 * reserved by the individual authors.
 * 
 * The Radio Amateur Telecommunications Society provides this software
 * on an "as is" basis.  The Society assumes no liability for
 * loss incurred through the use of this software.  Amateur Radio
 * use of this software implies non-commercial and voluntary 
 * development, deployment and use of this software in a "Amateur",
 * non-commercial service.  Commercial users are encouraged to
 * inspect their copies of the source code.  Source code modification
 * licenses are available if a combined Object and Source Code
 * license was not originally established.
 * 
 * The Society may be contacted by writing or calling at:
 * 
 * The Radio Amateur Telecommunications Society 
 * 206 North Vivyen Street.
 * Bergenfield, New Jersey 07621
 *
 * Telephone: 201-387-8896
 *
 */
#include "data.h"
#include "buffer.h"
#include "iface.h"
#include "timer.h"
#include "ax25.h"
#include "ax25l2.h"
#include "l3struc.h"
#include "x25cause.h"
#include "l3calls.h"
#include "tx.h"
#include "config.h"
#include "upfcn.h"

int NULLFCN(), clr_ckmem(), con_ckmem(), recv_boot();
int send_ckmem(), rst_boot(), init_ckmem(), unload_ok();

struct upfcn ckmem={
	{{0x86, 0x90, 0x96, 0x9a, 0x8a, 0x9a}, {0x00}}, /* ckmem-0 */
	rst_boot, clr_ckmem, con_ckmem, recv_boot,
	send_ckmem, NULLFCN, unload_ok,
	"CHKMEM - ROSE Switch Memory Check program, ver 1.0\r"
};

struct VCS *ckmem_llcn;
int ckmem_busy;
struct datastr *mkpkt(), *bappstr(), *info_pkt();
struct datastr *ckmem_in;

int
con_ckmem(vc)
register struct VCS *vc;
{
	vc=vc->peer;
	if (!vc) return;
	(*vc->SEND)(vc,info_pkt());
	if (ckmem_busy) {
		(*vc->SEND)(vc,mkpkt("Busy\r"));
		set_p(vc,P6,Number_Busy);
	}
	else {
		ckmem_busy=TRUE;
	}
}

int
clr_ckmem(vc,c)
struct VCS *vc;
int c;
{
	ckmem_busy = 0;
	set_p(vc, P1, 0);
}

putint(p,val)
register struct datastr *p;
int val;
{
	putxch(p,(val>>8));
	putxch(p,val);		/* HHLL */
}

int
send_ckmem(vc, pkt)
struct VCS *vc;
struct datastr *pkt;
{
	extern int lostlen;
	extern unsigned int LOSTLIST[];
	static int i;

	vc_queue_data(vc,pkt);
	if (vc->tx_queue) /* There is stuff waiting */ {
		while ((ckmem_in = vc->tx_queue)) {
			vc->tx_queue = ckmem_in->next;
			free_buffer(ckmem_in);
		}
		/* Do it here */
		do_memchk();
		ckmem_in = new_buffer(1024);
		for (i=0;i<lostlen;i=i+2) {
			putint(ckmem_in, LOSTLIST[i]);
			bappch(ckmem_in,'/');
			putint(ckmem_in, LOSTLIST[i+1]);
			bappch(ckmem_in, 0x0d);
		}
		if (lostlen) (*vc->peer->SEND)(vc->peer, ckmem_in);
		else free_pkt(ckmem_in);
		(*vc->peer->SEND)(vc->peer, mkpkt("Done\r"));
		ckmem_in = NULL;
	}
	return 256;	/* We are never busy! */
}

