/*
 * Copyright 1988 by the Radio Amateur Telecommunications Society
 * and Thomas A. Moulton, W2VY
 *
 * This software may only be modified, copied, distributed or
 * executed for non-profit purposes by individuals operating
 * systems in the Amateur Radio Service.  Credit to the
 * author(s) and to the Radio Amateur Telecommunications Society
 * must be made in modules where RATS provided software is used,
 * and in any announcements and documentation.  
 *
 * As a non-profit, research and development organization,  the
 * Radio Amateur Telecommunications Society distributes software
 * in both executable and source forms.  This policy is in place
 * to encourage the development and distribution of OSI-based,
 * networking tools.  In order to protect the interests of the
 * Society and the authors, we have placed some conditions
 * of use on the software.  Other groups are encouraged
 * to place the same or similar guidelines on
 * software they produce.
 *
 * The Radio Amateur Telecommunications Society reserves the right
 * to specify and alter the terms under which software provided by
 * the Society may be used.  This policy is consistent with the 
 * objective of uniform and consistent "Open Systems Interconnections."
 * 
 * All acceptable Amateur Radio related uses of this software
 * will be outlined in the "ROSE Implementer's Guide".  Individuals
 * or organizations wishing to add to, or modify the provisions of
 * the guide to accommodate local or evolutionary requirements
 * should document the proposed change(s) and forward them to the
 * Society.  If accepted, written notification will be provided by
 * the Society to the submitting organization or individual(s).
 * The Society will then issue a "ROSE Implementer's Guide Change
 * Notice".  Periodically, the Society will re-issue the "ROSE 
 * Implementer's Guide" and incorporate the text of the change 
 * notices.  This procedure has been put in to place to ensure
 * compatibility between systems and to ensure their "Openness"
 * and interoperability.
 *
 * No part of this software may be used in other packages 
 * without prior authorization from the author or the Society.  
 * Software incorporating this module, all or in part, must be 
 * provided to the Society prior to distribution or use by
 * anyone not directly involved in testing of the revised  
 * environment.  Current releases of the combined software must
 * be provided to the Society in both source and executable
 * forms.  Adequate documention to produce an executable module 
 * from the provided source must also be included.
 *
 * Non-Amateur Radio non-profit uses may be authorized on a case
 * by case basis.  Inquiries for such use may be made in writing
 * to the Society. Non-commercial uses consistent with the
 * general principles of Open Systems Interconnection Reference
 * Model will be generally considered with favor.
 *
 * Commercial licensing of the software is also available based
 * on normal commercial terms.  Licensing inquiries should be
 * directed to the Society.  Commercial licensing of the standard
 * software will be done in situations which materially benefit
 * the Amateur Radio Packet Network.  Additional licensing is
 * reserved by the individual authors.
 * 
 * The Radio Amateur Telecommunications Society provides this software
 * on an "as is" basis.  The Society assumes no liability for
 * loss incurred through the use of this software.  Amateur Radio
 * use of this software implies non-commercial and voluntary 
 * development, deployment and use of this software in a "Amateur",
 * non-commercial service.  Commercial users are encouraged to
 * inspect their copies of the source code.  Source code modification
 * licenses are available if a combined Object and Source Code
 * license was not originally established.
 * 
 * The Society may be contacted by writing or calling at:
 * 
 * The Radio Amateur Telecommunications Society 
 * 206 North Vivyen Street.
 * Bergenfield, New Jersey 07621
 *
 * Telephone: 201-387-8896
 *
 */
#include "data.h"
#include "buffer.h"
#include "iface.h"
#include "timer.h"
#include "ax25.h"
#include "ax25l2.h"
#include "l3struc.h"
#include "x25cause.h"
#include "l3calls.h"
#include "tx.h"
#include "config.h"
#include "upfcn.h"

int NULLFCN(), clr_ckrout(), con_ckrout(), recv_boot();
int send_ckrout(), rst_boot(), unload_ok();

struct upfcn ckrout={
	{{0x86, 0x96, 0xa4, 0x9e, 0xaa, 0xa8}, {0x00}}, /* CKROUT-0 */
	rst_boot, clr_ckrout, con_ckrout, recv_boot,
	send_ckrout, NULLFCN, unload_ok,
	"CKROUT - ROSE X.25 Packet Switch Routing Test Program Ver 1.1\r"
};

extern unsigned char node_addr[];

extern unsigned char NPAroute[][16];
extern unsigned char RTable[];

struct datastr *mkpkt(), *info_pkt();

int call_str(), x121_str(), find_route();

struct tmrblk *ckr_timer;
char ckr_busy, outstr[128];
struct datastr *ckr_in;
char xadr[16], xlen, xpos, d1, d2, d3, d4, a1, a2, a3, e1, e2, e3;
char addr[16], *ptr, xdone;

int
x121_adr(str,x121)
char str[];
char x121[16];
{
	int i,len,ch,hi;
	char x[8];

	x121[0]=0;
	for (len=0;len<16;len++) /* Insure valid numbers */ {
		ch=str[len];
		if (ch == 0 || ch == 0x0d) break; /* End of address */
		if ((ch < 0x30) || (ch > 0x39)) return 0; /* Non-digit */
	}

	x[0]=len;
	ch=0;
	hi=0;
	for (i=0;i<len;i++) {
		hi = !hi;
		if (hi) ch = (str[i] - 0x30) << 4;
		else {
			ch = ch | (str[i] - 0x30);
			x[ (i>>1) + 1 ]= ch;
		}
	}
	if (hi) x[ (i>>1) + 1 ]= ch;
	adr_copy(0,x,1,x121);
	x121[0]=((x121[0] & 0x0f) + ((x121[0] & 0x0f0) >> 4));
	return len;
}

int
con_ckrout(vc)
register struct VCS *vc;
{
	static int (*fn)();

	vc=vc->peer;
	if (!vc) return;
	fn=vc->SEND;
	puthex2("con",vc,"at",fn);
	(*fn)(vc,info_pkt());
	if (ckr_busy) {
		(*fn)(vc,mkpkt("Busy\r"));
		set_p(vc,P6,Number_Busy);
	}
	else {
		ckr_busy++;
		xdone = 1;
		ckr_timer=NULL;
	}
}

int
clr_ckrout(vc,c)
struct VCS *vc;
int c;
{
	ckr_busy--;
	if (ckr_busy == 0) kill(ckr_timer);
	set_p(vc, P1, 0);
}

int
send_ckrout(vc, pkt)
register struct VCS *vc;
struct datastr *pkt;
{
	int ckitout();
	static char *ch;
	static int i;
	static char buff[80];

	vc_queue_data(vc,pkt);
	if (!vc->tx_queue) return 256;
	while ((ckr_in = vc->tx_queue)) /* There is stuff waiting */ {
		vc->tx_queue=ckr_in->next;
		ckr_in->next=NULL;
		ch=buff;
		while ((i=bgetch(&ckr_in)) != EOF) *ch++ = (i&0xff);
		*ch = 0;
	}
	vc=vc->peer;
	if (!xdone) /* Tell them where we are... */ {
		ch = buff;
		*ch++ = 'W';
		*ch++ = 'o';
		*ch++ = 'r';
		*ch++ = 'k';
		*ch++ = 'i';
		*ch++ = 'n';
		*ch++ = 'g';
		*ch++ = ' ';
		*ch++ = 'o';
		*ch++ = 'n';
		*ch++ = ' ';
		ch += x121_str(xadr, ch);
		*ch++ = 0x0d;
		*ch++ = 0;
		(*vc->SEND)(vc, mkpkt(buff));
	} else /* Not running, restart */ {
		(*vc->SEND)(vc, mkpkt(buff));
		(*vc->SEND)(vc, mkpkt("\rStarting Now.\r"));
		xadr[0] = 10;
		xadr[1] = node_addr[1];
		xadr[2] = node_addr[2];
		xadr[3] = 0;
		xadr[4] = 0;
		xadr[5] = 0;
		i = x121_adr(buff, xadr);
		ch = buff;
		ch += x121_str(xadr, ch);
		*ch++ = 0x0d;
		*ch = 0;
		(*vc->SEND)(vc, mkpkt(buff));
		xlen = xpos = xadr[0];
		for (i=xlen;i>0;i--) /* Find last non-zero digit */ {
			if (i&1) /* High Order digit */ {
				if (xadr[(i+1)>>1]>>4) break;
			} else /* Low Order digit */ {
				if (xadr[(i+1)>>1]&0x0f) break;
			}
		}
		xpos = i;
		xdone = 0;
		queue(&ckr_timer,ckitout, 5, vc);
	}
	return 256;
}

int
ckitout(vc)
struct VCS *vc;
{
	static int r, loop, siz;
	static struct datastr *bp, *bp1;
	static char i, j;

	if (xdone) /* Done */ {
		(*vc->SEND)(vc, mkpkt("Done\07!!!!\r"));
		return;
	}
	loop =siz = (*vc->SEND)(vc, NULL);	/* Are we flow controled? */
	bp = bp1 = NULL;
	while (loop--)  /* First do the current item */ {
		adr_copy(0, xadr, 0, addr);
		r = find_route(&ptr, addr);
		outstr[0]=0;
		if (r == 1) {
			route_ok(ptr, addr);
		} else if (r == 0) {
			route_ng(ptr, addr);
		} else if (r == -1) {
			route_dir(ptr, addr);
		}
		if (outstr[0]) /* Have reply */ {
			if (!bp) bp = bp1 = mkpkt(outstr);
			else {
				bp1->nexts = mkpkt(outstr);
				bp1 = bp1->nexts;
			}
			if (buflen(bp1) >= siz) {
				loop = siz = (*vc->SEND)(vc, bp);
				bp=bp1=NULL;
			}
		}
		for (i=xlen;i>xpos;i--) /* Now Increment the address */ {
			j = (i+1)>>1;
			if (i&1) /* High Order Address Digit */ {
				xadr[j] += 0x10;
				if ((xadr[j]&0xf0) != 0xa0) break;
				xadr[j] = xadr[j]&0x0f;
			} else /* Low Order Address Digit */ {
				xadr[j]++;
				if ((xadr[j]&0x0f) != 0x0a) break;
				xadr[j] = xadr[j]&0xf0;
			}
		}
		if (i == xpos) /* We are all done... */ {
			xdone = 1;
			break;
		}
	}
	if (bp) (*vc->SEND)(vc, bp);
	queue(&ckr_timer, ckitout, 3, vc); /* Queue ourselves for next time */
}

int
route_ok(ptr, adrx)
char *ptr, *adrx;
{
	static char *ch, i, l;

	ch = &outstr[x121_str(adrx, outstr)];
	*ch++ = ' ';
	*ch++ = '-';
	*ch = '>';
	l = (*ptr>>4)&0x0f;
	for (i=1;i<=l;i++) /* Print Each Callsign in ALT list */ {
		ch++;
		*ch++ = ' ';
		ch += call_str(&L3CALL[ptr[i]], ch);
		*ch = ',';
	}
	*ch++ = 0x0d;
	*ch++ = 0;
}

int
route_ng(ptr, adrx)
{}

int
route_dir(ptr, adrx)
char *ptr, *adrx;
{
	static char *ch;

	ch = &outstr[x121_str(adrx, outstr)];
	*ch++ = ' ';
	*ch++ = '=';
	*ch++ = '=';
	*ch++ = '>';
	*ch++ = ' ';
	ch += x121_str(ptr, ch);
	*ch++ = 0x0d;
	*ch++ = 0;
}
