/*
 * Copyright 1988 by the Radio Amateur Telecommunications Society
 * and Thomas A. Moulton, W2VY
 *
 * This software may only be modified, copied, distributed or
 * executed for non-profit purposes by individuals operating
 * systems in the Amateur Radio Service.  Credit to the
 * author(s) and to the Radio Amateur Telecommunications Society
 * must be made in modules where RATS provided software is used,
 * and in any announcements and documentation.  
 *
 * As a non-profit, research and development organization,  the
 * Radio Amateur Telecommunications Society distributes software
 * in both executable and source forms.  This policy is in place
 * to encourage the development and distribution of OSI-based,
 * networking tools.  In order to protect the interests of the
 * Society and the authors, we have placed some conditions
 * of use on the software.  Other groups are encouraged
 * to place the same or similar guidelines on
 * software they produce.
 *
 * The Radio Amateur Telecommunications Society reserves the right
 * to specify and alter the terms under which software provided by
 * the Society may be used.  This policy is consistent with the 
 * objective of uniform and consistent "Open Systems Interconnections."
 * 
 * All acceptable Amateur Radio related uses of this software
 * will be outlined in the "ROSE Implementer's Guide".  Individuals
 * or organizations wishing to add to, or modify the provisions of
 * the guide to accommodate local or evolutionary requirements
 * should document the proposed change(s) and forward them to the
 * Society.  If accepted, written notification will be provided by
 * the Society to the submitting organization or individual(s).
 * The Society will then issue a "ROSE Implementer's Guide Change
 * Notice".  Periodically, the Society will re-issue the "ROSE 
 * Implementer's Guide" and incorporate the text of the change 
 * notices.  This procedure has been put in to place to ensure
 * compatibility between systems and to ensure their "Openness"
 * and interoperability.
 *
 * No part of this software may be used in other packages 
 * without prior authorization from the author or the Society.  
 * Software incorporating this module, all or in part, must be 
 * provided to the Society prior to distribution or use by
 * anyone not directly involved in testing of the revised  
 * environment.  Current releases of the combined software must
 * be provided to the Society in both source and executable
 * forms.  Adequate documention to produce an executable module 
 * from the provided source must also be included.
 *
 * Non-Amateur Radio non-profit uses may be authorized on a case
 * by case basis.  Inquiries for such use may be made in writing
 * to the Society. Non-commercial uses consistent with the
 * general principles of Open Systems Interconnection Reference
 * Model will be generally considered with favor.
 *
 * Commercial licensing of the software is also available based
 * on normal commercial terms.  Licensing inquiries should be
 * directed to the Society.  Commercial licensing of the standard
 * software will be done in situations which materially benefit
 * the Amateur Radio Packet Network.  Additional licensing is
 * reserved by the individual authors.
 * 
 * The Radio Amateur Telecommunications Society provides this software
 * on an "as is" basis.  The Society assumes no liability for
 * loss incurred through the use of this software.  Amateur Radio
 * use of this software implies non-commercial and voluntary 
 * development, deployment and use of this software in a "Amateur",
 * non-commercial service.  Commercial users are encouraged to
 * inspect their copies of the source code.  Source code modification
 * licenses are available if a combined Object and Source Code
 * license was not originally established.
 * 
 * The Society may be contacted by writing or calling at:
 * 
 * The Radio Amateur Telecommunications Society 
 * 206 North Vivyen Street.
 * Bergenfield, New Jersey 07621
 *
 * Telephone: 201-387-8896
 *
 */
#include "data.h"
#include "buffer.h"
#include "iface.h"
#include "timer.h"
#include "ax25.h"
#include "ax25l2.h"
#include "l3struc.h"
#include "x25cause.h"
#include "l3calls.h"
#include "tx.h"
#include "config.h"
#include "upfcn.h"

int NULLFCN(), clr_info(), con_info(), recv_boot();
int send_info(), rst_boot(), init_info(), unload_info();

struct upfcn info={
	{{0x92, 0x9c, 0x8c, 0x9e, 0x40, 0x40}, {0x00}}, /* INFO-0 */
	rst_boot, clr_info, con_info, recv_boot,
	send_info, init_info, unload_info,
	"INFO   - ROSE Switch Information Text Display, Version 1.1\r"
};

extern struct datastr *l2_user_info;
extern struct axcb *links;
extern struct VCS *pending_calls;
extern int (*L2PARS[5])();

struct VCS *info_llcn;
struct datastr *mkpkt(), *bappstr();
struct datastr *info_in;
int (*l2clr)(), (*l2reset)();
int l2nreset(), l2nclear();

struct errmsg {
	unsigned int value;
	char *msg;
};

#define ENGLISH 1
#ifdef SPANISH
#undef ENGLISH
char pse_disc[]="\rPor Favor Desconecte ahora\r";
char rpe[]="Error de Procedimiento Remoto";
char lpe[]="Error de Procedimiento Local";
char netcong[]="Congestionamiento de la Red";
char incompat[]="Destino Incompatible";

struct errmsg reset_msg[] = {
	{ DTE_Orig, "Reiniciacion Remota Efectuada Por Otra Estacion"},
	{ Net_Congest, netcong},
	{ 0, NULL}
};

struct errmsg clear_msg[] = {
	{ DTE_Orig, "La Estacion Remota Termino la Conexion"},
	{ Number_Busy, "Estacion Remota Ocupada"},
	{ Number_Busy+72, "Colision de la LLamada, Estacion Remota esta conectando con usted"},
	{ Invalid_Fac, "Facilidad no Soportada en el X.25 "},
	{ Congestion, netcong},
	{ Congestion+146, "Congestion, Cantidad de Reintentos excedida"},
	{ Congestion+179, "Problema de Configuracion, PID erroneo del dispositivo"},
	{ Congestion+229, "Reiniciacion sobre conexion confiable"},
	{ Out_of_Order, "Fuera de Servicio"},
	{ Access_Barred, "Acceso No Permitido"},
	{ Access_Barred+245, "No se permite conectar un Conmutador desde otro Conmutador"},
	{ Not_Obtainable, "La Ruta No esta Definida para la direccion especificada"},
	{ Remote_Proc, rpe},
	{ Local_Proc, lpe},
	{ Ship_Absent, "Estacion Remota no contesta"},
	{ 0, NULL}
};
#endif

#ifdef GERMAN
#undef ENGLISH

char pse_disc[]="\rbitte jetzt unterbrechen\r";
char rpe[]="fern prozedur fehler";
char lpe[]="ortliche prozedur fehler";
char netcong[]="Netzwerk \201berf\201llt";
char incompat[]="unvereinbares ziel";

struct errmsg reset_msg[] = {
	{ DTE_Orig, "ferngeste\201rte reset von die andere Station"},
	{ Net_Congest, netcong},
	{ 0, NULL}
};

struct errmsg clear_msg[] = {
	{ DTE_Orig, "ferngeste\201rte Station getrennt die verbindung"},
	{ Number_Busy, "ferngeste\201rte Station ist BESETZT"},
	{ Number_Busy+72, "\232bertragungsfehler, ferngeste\201rte Station verbindet sich"},
	{ Invalid_Fac, "ung\201ltig X.25 Ausr\201stung gesucht"},
	{ Congestion, netcong},
	{ Congestion+146, "Netzwerk \201berf\201llt, Zahl der Wiederholungsversuche \201berschritten"},
	{ Congestion+179, "Problem bei der Grundeinstellung, falsche PID"},
	{ Congestion+229, "reset, aber Verbindungslinie ist i.O."},
	{ Out_of_Order, "ausser betrieb"},
	{ Access_Barred, "Zugang nicht erlaubt"},
	{ Access_Barred+245, "verbindung mit Schalter nicht erlaubt"},
	{ Not_Obtainable, "keine Strecke festgelegt f\201r diese addresse"},
	{ Remote_Proc, rpe},
	{ Local_Proc, lpe},
	{ Ship_Absent, "ferngeste\201rte Station reagiert nicht"},
	{ 0, NULL}
};
#endif

#ifdef FRENCH
#undef ENGLISH
#endif

#ifdef ENGLISH

char pse_disc[]="\rPlease Disconnect now\r";
char rpe[]="Remote Procedure Error";
char lpe[]="Local Procedure Error";
char netcong[]="Network Congestion";
char incompat[]="Incompatible Destination";

struct errmsg reset_msg[] = {
	{ DTE_Orig, "Remote Reset from other station"},
	{ Net_Congest, netcong},
	{ 0, NULL}
};

struct errmsg clear_msg[] = {
	{ DTE_Orig, "Remote Station cleared connection"},
	{ Number_Busy, "Remote Station is BUSY"},
	{ Number_Busy+72, "Call Collision, Remote station is connecting to you"},
	{ Invalid_Fac, "Invalid X.25 Facility Requested"},
	{ Congestion, netcong},
	{ Congestion+146, "Congestion, Retry Count Exceeded"},
	{ Congestion+179, "Configuration Problem, wrong PID from device"},
	{ Congestion+229, "Reset on reliable connection"},
	{ Out_of_Order, "Out of Order"},
	{ Access_Barred, "Access Barred"},
	{ Access_Barred+245, "Not allowed to connect to a switch from a switch"},
	{ Not_Obtainable, "No Route Defined for address specified"},
	{ Remote_Proc, rpe},
	{ Local_Proc, lpe},
	{ Ship_Absent, "Remote Station not responding"},
	{ 0, NULL}
};
#endif

int
init_info()
{
	l2reset = L2PARS[0];
	l2clr = L2PARS[1];
	L2PARS[0] = l2nreset;
	L2PARS[1] = l2nclear;
	info_llcn = NULL;
}

int
unload_info()
{
	register struct VCS *vc;
	static struct axcb *axcb;

	L2PARS[0] = l2reset;
	L2PARS[1] = l2clr;
	while (info_llcn) /* Clear all active calls */ {
		vc = info_llcn;
		info_llcn = vc->next;
		set_p(vc, P6, DTE_Orig+122);
	}
	axcb = links;
	while (axcb) /* Find and reset all references to these routines */ {
		fix_pointer(axcb->llcn);
		axcb = axcb->next;
	}
	fix_pointer(pending_calls);
	return 1;
}

fix_pointer(vc)
register struct VCS *vc;
{
	while (vc) /* Check each VC */ {
		if (vc->RESET == l2nreset) vc->RESET = l2reset;
		if (vc->CLEAR == l2nclear) vc->CLEAR = l2clr;
		vc = vc->next;
	}
}

int
con_info(vc)
register struct VCS *vc;
{
	vc->next = info_llcn;	/* This works even when info_llcn is NULL */
	info_llcn = vc;

	vc=vc->peer;
	if (!vc) return;
	puthex2("con",vc,"at",fn);
	(*vc->SEND)(vc,dup_pkt(l2_user_info));
	(*vc->SEND)(vc,mkpkt(pse_disc));
}

int
clr_info(vc,c)
struct VCS *vc;
int c;
{
	register struct VCS *vcp;

	if (info_llcn == vc) info_llcn = vc->next; /* First item */
	else {
		vcp = info_llcn;
		while (vcp->next && vcp->next != vc) vcp = vcp->next;
		if (vcp->next == vc) vcp->next = vc->next;
	}
	vc->next = NULL;
	set_p(vc, P1, 0);
}

int
send_info(vc, pkt)
struct VCS *vc;
struct datastr *pkt;
{
	vc_queue_data(vc,pkt);
	if (vc->tx_queue) /* There is stuff waiting */ {
		while ((info_in = vc->tx_queue)) {
			vc->tx_queue = info_in->next;
			free_buffer(info_in);
		}
		(*vc->peer->SEND)(vc->peer,mkpkt(pse_disc));
	}
	return 256;	/* We are never busy! */
}

char *
msg(tab, val)
struct errmsg tab[];
unsigned int val;
{
	static int i;

	i=0;
	while (tab[i].msg != NULL) /* Find the right value */ {
		if (val == tab[i].value) return tab[i].msg;
		i++;
	}
	return NULL;
}

struct datastr *
l_message2(str, v, str2)
char *str;
int v;
char *str2;
{
	register struct datastr *p;

	if ((p=mkpkt(str)) == NULL) return NULL;
	putxch(p,(v>>8));
	putxch(p,(v&0xff));
	bappch(p, 0x20);
	bappstr(p, str2);
	bappch(p,0x0d);
	return p;
}

int
l2nreset(vc,diag) 
struct VCS *vc;
unsigned int diag;
{
	static struct axcb *lnk;
	static char *str;

	set_d(vc, D1, 0);
	if ((lnk=vc->parent)) {
		axconnect(lnk);
		str = NULL;
		str = msg(reset_msg, diag);
		if (!str) str = msg(reset_msg, diag&0xff00);
		send_tnc(lnk,l_message2("\r*** Reset *** ", diag, str));
	}
}

int
l2nclear(vc,c)
struct VCS *vc;
unsigned int c;
{
	register struct axcb *lnk;
	static char *str;

	lnk=vc->parent;
	set_p(vc,P1,0);
	if (!lnk) return;	/* No link exists */
	lnk_ign_fcn(lnk);

	if (lnk->state < 3) close_ax25(lnk);
	else {
		str = NULL;
		str = msg(clear_msg, c);
		if (!str) str = msg(clear_msg, c&0xff00);
		send_tnc(lnk,l_message2("\r*** Disconnect *** ", c, str));
	}
}
