/* Copyright 1989 Xanthus Inc and Thomas A. Moulton, W2VY
   ALL RIGHTS RESERVED!
*/
/* These are the routines used to parse command lines */

#include <string.h>
#include "ini.h"
#include "rtable.h"
#include "dnic.h"

#define TRUE 1
#define FALSE 0
#define NULL 0
#define EOF (-1)

int isupper(ch)
char ch;
{
	ch = ch&0x7f;
	return (ch>0x40 && ch<0x5b);
}

int
find_sym(sym, len, tab)	/* Find the string SYM in the TABle */
char *sym;
int len;
struct menu tab[];
{
	static int f, i, j, l;

	i=0;
	while (l=strlen(tab[i].cmd)) {
		if (l >= len) {
			f=TRUE;
			for (j=0;j<len;j++)
				if (toupper(sym[j]) != toupper(tab[i].cmd[j]))
					f=FALSE;
			if (f) {
				if (l == len) return i; /* Exact Match */
				if (!isupper(tab[i].cmd[j])) return i;
			}
		}
		i++;
	}
	i++;	/* Incase First is open */
	return -i;
}

#if 0
int
find_sym(sym, len, tab)	/* Find the string SYM in the TABle */
char *sym;
int len;
struct menu tab[];
{
	int f, i, j, l;

	i=0;
	while (l=strlen(tab[i].cmd)) {
		if (l == len) {
			f=TRUE;
			for (j=0;j<len;j++)
					if (sym[j] != tab[i].cmd[j]) f=FALSE;
			if (f) return i;
		}
		i++;
	}
	i++;	/* Incase First is open */
	return -i;
}
#endif

int
add_sym(sym, tab, maxsym)
char *sym;
struct menu tab[];
int maxsym;
{
	int j,i,k;

	k=strlen(sym);
	if (k <= 0) return EOF;
	if ( (j=find_sym(sym, tab)) > 0 ) return j;
	j = (-j);
	if (--j >= maxsym) return EOF;
	for (i=0;i<k;i++) tab[j].cmd[i] = sym[i];
	return j;
}

int
x121_check(ptr,adr)
register char adr[], ptr[];
{
	char i, j, alen;

	alen=ptr[0] & 0x0f;
	if ( (adr[0] & 0x0f < alen) || (alen == 0) ) return -1;
	j=alen>>1;

	for (i=1; i<=j; i++)
		if (adr[i] != ptr[i]) return -1;

	if (alen & 1 && /* Length is odd, check last nibble */
		 ( ptr[i] &0xf0 != (adr[i] & 0x0f0) )) return -1;
	if (alen == adr[0]&0x0f) return 0;
	return 1;
}

x121_adr(str,x121)
char str[];
char x121[16];
{
	int i,len,ch,hi;
	char x[8];
	extern struct dnicstr *DNIC;	/* Current active DNIC to use */

	x121[0]=len=0;
	len=word_len(str);
	for (i=0;i<len;i++) /* Insure valid numbers */ {
		ch=str[i];
		if (ch == 0) break; /* End of address */
		if ((ch < 0x30) || (ch > 0x39)) return 0; /* Non-digit */
	}
	if (DNIC) adr_copy(0,DNIC->adr,0,x121);

	x[0]=len;
	ch=0;
	hi=0;
	for (i=0;i<len;i++) {
		hi = !hi;
		if (hi) ch = (str[i] - 0x30) << 4;
		else {
			ch = ch | (str[i] - 0x30);
			x[ (i>>1) + 1 ]= ch;
		}
	}
	if (hi) x[ (i>>1) + 1 ]= ch;
	adr_copy(0,x,1,x121);
	x121[0]=((x121[0] & 0x0f) + ((x121[0] & 0x0f0) >> 4));
	return len;
}

x121_str(x121, str)
char x121[], *str;
{
	int l, i, ch;

	l = x121[0] & 0x0f;	/* Only Supports low order address */
	for (i=0; i<l; i++) /* Copy and convert address */ {
		ch = x121[ 1+(i>>1) ];
		if (i & 1) *str++ = 0x30 + (ch & 0x0f); /* Low digit */
		else *str++ = 0x30 + ( (ch>>4) & 0x0f); /* High Digit */
	}
	*str = NULL;
	return;
}


adr_copy(src,istr,des,ostr)
int src;
char istr[16];
int des;
char ostr[16];

{
	int ipos, opos, ilow, olow, len, i, acc;

	if(src) /* Copy High order call (calling address) */ {
		ilow=istr[0]&1;
		ipos= ( (istr[0] & 0x0f) + ilow) >> 1;
		len=(istr[0] & 0x0f0)>>4;
		if (!ilow) ipos++;
	}
	else /* Copy low order call (called address) */ {
		ipos=1;
		ilow=0;
		len=istr[0] & 0x0f;
	}
	if (des) /* Copy to high order call (calling) */ {
		olow=ostr[0] & 1;
		opos=( (ostr[0] & 0x0f) + olow) >> 1;
		ostr[0]=ostr[0] & 0x0f | len<<4;
		if (!olow) opos++;
	}
	else /* Copy to low order call (called) */ {
		olow=0;
		opos=1;
		ostr[0]=ostr[0] & 0xf0 | len;
	}
	acc=0;
	if (olow)
		acc=ostr[opos] & 0xf0; /* Init acc */
	for (i=1;i<=len;i++) {
		if (ilow) acc += istr[ipos++] & 0x0f;
		else acc += (istr[ipos] & 0x0f0) >> 4;
		ilow = !ilow;
		if (olow) /* Output a byte */ {
			ostr[opos++]=acc;
			acc=0;
		}
		else acc=acc<<4;
		olow = !olow;
	}
	if (olow) ostr[opos]=acc;
}

get_call(adr, len, stn)
char adr[], stn[];	/* stn is really struct ax25_adr * */
int len;
{
	int i,j;

	j = 0; /* Source pos */

	for (i=0;i<6;i++) {
		adr[j]=toupper(adr[j]);
		if ((j==len) || (adr[j] == 0x2d))
			stn[i]=0x40;
		else /* Normal ch */
			stn[i]=(adr[j++]<<1);
	}
	stn[i]=0;
	if (adr[j++] == 0x2d && j <= len && adr[j] >= 0x30 && adr[j] <= 0x39) {
		stn[i]=(adr[j++]-0x30)<<1;
		if (j <= len && adr[j] >= 0x30 && adr[j] <= 0x39) {
			stn[i] = stn[i]*10 + ((adr[j] - 0x30)<<1);
		}
		if (stn[i] > 30) stn[i]=0;
	}
	stn[i] |= 0x60;		/* Set Reserved bits */
	return 0;
}

call_str(stn, str)
unsigned char stn[], *str;
{
	int i;

	*str = NULL;
	for (i=0; i<6; i++) /* Copy callsign to str */ {
		if ( (stn[i] == 0x40) || (stn[i] == 0) ) break;
		*str++ = (stn[i]) >> 1;
	}
	i = (stn[6]>>1) & 0x0f;
	if (i) /* SSID present */ {
		*str++ = 0x2d;	/* Dash */
		if (i > 9) {
			*str++ = 0x31;
			i -= 10;
		}
		*str++ = i+0x30;
	}
	*str = NULL;
}
