/* Copyright 1989 Xanthus Inc and Thomas A. Moulton, W2VY
   ALL RIGHTS RESERVED!
*/
/* ROSE Configuration File Generator */

/* This file contains the data structures supplied to the command parser
   for the Interactive Network Interface */

#include "ini.h"
#include "rtable.h"
#include "dnic.h"
#include "node.h"

#include <string.h>
#include <stdio.h>

#define TRUE 1
#define FALSE 0
#define EOF (-1)

extern int S_TOP;
extern struct menu *State[16];
extern int F_TOP;
extern struct menu *(*Fcn[16])();

int THIS;
int Verify;
int DEF_L3W, DEF_PORT, DEF_MAXVC, DEF_TIMEOUT;
char DEF_INCL[80];

int ndnics=0;
struct dnicstr DNICS[16];
struct dnicstr *DNIC=0;

int numrt;	/* Number of routing records currently in use */
struct rt_tab *RTable[MaxRT];

#define MAXNU 24
int nnodes=0;
int nusers=0;
struct nodestr Nodes[MAXNU];
struct nodestr *NODE=0;

int userport=0;
int textlen=0;
char textdata[2048];

extern char x_dnicfmt[];
char x_dnicnam[]="*** DNIC Specified does not exist, must specify name when adding a new DNIC ***";

char x_pwd_long[]="Specified Password is too long - 255 Chars max";
char x_pwd_fbad[]="Specified Password File does not exist";

int passlen;
unsigned char passwd[257];

static struct menu retstate;
static char altadr[16];

roseinit()
{
	int i, j;

	NODE=NULL;
	DNIC=NULL;
	textlen=0;
	textdata[0]=0;
	DEF_INCL[0]=0;
	for (i=0;i<MAXNU;i++) {
		DNICS[i].name[0]=0;
		DNICS[i].adr[0]=0;

		Nodes[i].name[0]=
		Nodes[i].adr[0]=
		Nodes[i].call[0]=
		Nodes[i].digi[0]=0;
		Nodes[i].port=
		Nodes[i].maxvc=
		Nodes[i].l3w=
		Nodes[i].timeout=0;
	}
	numrt=0;
	for (j=0;j<MaxRT;j++) RTable[j] = NULL;
	passlen=0;
	passwd[0]=0;
}

roselini()
{
	THIS=FALSE;
}

int
decimal(ch, val)
char *ch;
int *val;
{
	int i, l, v;

	l=v=0;

	while ( (*ch < 0x3a) && (*ch > 0x2f)) {
		v *= 10;
		v += *ch - 0x30;
		ch++;
		l++;
	}
	if (l) *val = v;	/* Only update return value only */
	return l;		/* if we processed some data */
}

struct menu *
thisflag(tok, err, state)
char **tok;
char **err;
struct menu state[];
{
	THIS=TRUE;
	return NULL;
}

struct menu *
verifyon(tok, err, state)
char **tok;
char **err;
struct menu state[];
{
	Verify=TRUE;
	return NULL;
}

struct menu *
verifyoff(tok, err, state)
char **tok;
char **err;
struct menu state[];
{
	Verify=FALSE;
	return NULL;
}

int
find_dnic(x121)
char *x121;
{
	int i, j;

	for (i=0;i<=ndnics;i++)
	     if (x121_check(DNICS[i].adr, x121) == 0) return i; /* Found It! */

	return EOF;
}

int
find_node(name,l)
char *name;
int l;
{
	int i, j;

	for (i=0;i<=nnodes;i++) if (strlen(Nodes[i].name) == l) {
		for (j=0; j<l; j++) if (Nodes[i].name[j] != name[j]) j=l+1; /* Signal Break and no match if on last char */
		if (j == l) return i;	/* Found it! */
	}
	return EOF;
}

struct menu *
do_dnic(tok, err, state)
char **tok;
char **err;
struct menu state[];
{
	char adr[8], *ch;
	int i, j, l;

	word_skip(tok);	/* Skip Blanks */
	if (word_len(*tok) > 0) DNIC=NULL;
	l=x121_adr(*tok, adr);
	if (l == 1 && adr[1] == 0) /* Set no DNIC (for routing) */ {
		*tok += l;
		DNIC = NULL;
		return NULL;
	}
	if (l == 4) /* Address Specified */ {
		if (THIS) /* This Nodes' DNIC */ {
			adr_copy(0,adr,0,DNICS[0].adr);
			DNICS[0].name[0]=NULL;
			j=0;
		} else j=find_dnic(adr);

		*tok += l;
		word_skip(tok);
		if (**tok) /* Name specified */ {
			if (j == EOF) /* New Entry */ {
				j = ++ndnics;
				DNICS[j].name[0] = NULL;
			}
			adr_copy(0,adr,0,DNICS[j].adr);
			ch = DNICS[j].name;
			if (*ch) printf("* Warning * old name over written %s\n",ch);
			while (**tok) /* copy rest of line */ {
				*ch++ = **tok;
				(*tok)++;
			}
			*ch = NULL;
		} else if (j == EOF) *err = x_dnicnam;	/* DNIC Does not exist */
	} else *err = x_dnicfmt;	/* DNIC must have 4 digits */
	if (j != EOF) DNIC = &DNICS[j];
	return NULL;
}

struct menu *
quit(tok, err, state)
char **tok;
char **err;
struct menu state[];
{
	exit(0);
	return NULL;
}

struct menu *
passfile(tok, err, state)
char **tok;
char **err;
struct menu state[];
{
	FILE *pwd;
	FILE *get_file();
	int inlen, plen, len;
	char instr[81], pwdstr[256];

	inlen = 0;
	len = 0;
	plen = 0;
	pwdstr[0] = 0;
	word_skip(tok);
	pwd = get_file(*tok);
	*tok += word_len(*tok);	/* Eat the token in any case */
	if (pwd) /* Got a valid file that exists */ {
		while (inlen != EOF) /* Read all lines */ {
			inlen = fgetlin(instr, 80, pwd);
			if (inlen>0 && instr[0] == '#') inlen = 0;
			if (inlen>0) /* Copy the data, upto 255 chars */ {
				if (plen == 0) sscanf(instr, "%d", &plen);
				else {
					if (len+inlen < 255) {
						len += inlen;
						strcat(pwdstr, instr);
					} else {
						plen = 0;
						*err = x_pwd_long;
						break;
					}
				}
			}
		}
		if (plen && len) /* All looks OK */ {
			passlen = plen;
			passwd[0] = len;
			strcpy(&passwd[1], pwdstr);
		}
	} else *err = x_pwd_fbad;
	return NULL;
}

#define not_imp(fc) struct menu *fc() {printf("fc\n");return 0;}

struct menu *nodeinit(), *userinit(), *do_dnic(), *writefn();
struct menu donode[], douser[], dodef[], doroute[], doshow[];

struct menu level0[]={
	{"This",level0,thisflag},
	{"Node",donode,nodeinit},
	{"User",douser,userinit},
	{"DNIC",level0,do_dnic},
	{"Default",dodef,NULL},
	{"Route",doroute,NULL},
	{"WRITE",level0,writefn},
	{"Show",doshow,NULL},
	{"PASSword",level0,passfile},
	{"VERify",level0,verifyon},
	{"NOVERify",level0,verifyoff},
	{"QUIT",NULL,quit},
	{"",NULL,NULL}
};
