DEFDBL A-Z

REM DECLARE FUNCTION GetSimulationParams# ()
DECLARE SUB GetSatelliteParams ()
DECLARE SUB vgadata (a%, b%, c%, d%)
DECLARE FUNCTION Kepler# (MeanAnomaly#, TempEcc#)
DECLARE FUNCTION asin# (x)
DECLARE FUNCTION acos# (x)
DECLARE FUNCTION GetDay# (year#, month#, day#)

CONST PI = 3.1415926353#
CONST PI2 = (PI * 2)
CONST MinutePerDay = (24 * 60!)
CONST SeconderDay = (60 * MinutePerDay)
CONST HalfSecond = (.5 / SeconderDay)
CONST EarthRadius = 6378.16
CONST c = 299792.5
CONST DegRad = (180 / PI)
CONST RadDeg = (PI / 180)
CONST EarthFlat = (1 / 298.25)
CONST SiderealSolar = 1.0027379093#
CONST RefYear = 91
CONST RefSidereal = .27544157#
CONST MaxSats = 600
CONST xmax = 600
CONST ymax = 360
CONST DataLen = (640 / 8 * 361) - 1
CONST Addr = &H3CE
CONST Vga = &H3CF
CONST DispLines = 23

REM ********** Written by Robert W. Beyer - WA6JJY *******************
REM ********** Based on ORBIT23 by Robert W Berger *******************
REM ***** Version 1.1 - Changed bottom display               *********
REM ***** Version 1.2 - 'End' no longer required             *********
REM ***** Version 1.3 - Added 'v' and 'm' command            *********
REM *****               Fixed advance orbit so it is smooth  *********
REM *****               Now handles 205 Satellites           *********
REM ***** Version 1.4 - Now supports 600 satelites           *********
REM ***** Version 1.41- Added Fast and slow orbit advances   *********
REM ***** Version 1.50- So file name fits on compuserve      *********

REM  Keplerian Elements and misc. data for the satellite
REM  EpochDay:           time of epoch
REM  EpochMeanAnomaly:   Mean Anomaly at epoch
REM  EpochOrbNum!:       Integer orbit   of epoch
REM  EpochRAAN:          RAAN at epoch
REM  EpochMeanMotion:    Revolutions/day
REM  Decay:              Revolutions/day^2
REM  EpochArgPer:        argument of perigee at epoch
REM  Ecc:                Eccentricity
REM  Incl:               Orbital Inclination
REM  Satname$:           Satellite Name
REM  ElementSet:         Element Set
REM  BeaconFreq:         Mhz, used for doppler calc
REM  Height:             Height of Satellite (km)
REM  Lat:                Current Latitude of Sat
REM  Longit:             Current Longitude of Sat
REM  Az:                 Current Azimuth
REM  El:                 Current Elevation
REM  Dist:               Current Distance
REM  PrevTime:           Time of last position
REM  PrevRange:          Distance of last position
REM  Freq:               Frequency (with doppler)
REM  OrbitalPeriod:      Orbital period (revs/day)

DIM MData1%((DataLen / 2) + 1):    REM Display plane 1 data
DIM MData2%((DataLen / 2) + 1):    REM Display plane 2 data
DIM SHARED MonthDays(12):                 REM Days in Month table

DIM SatName$(MaxSats)
DIM SatIndex(MaxSats)

CLS : dataline = 24: GOSUB InitSimTime
GOSUB GetSiteParams: GOSUB GetSatelliteParams: CLS : Sat = 0
GOSUB ReadSatelliteParam
Restart:
WIDTH , 25: GOSUB PrintMenu
SCREEN 12: GOSUB LoadMap
GOSUB DrawMap
GOSUB PrintHeader

REM ******************** Main Loop ********************************
lp:
i$ = INKEY$
if len(i$) = 2 and mid$(i$,2,1) = ";" THEN GOTO Restart
IF i$ = "q" OR i$ = "Q" THEN CLS : END
IF i$ = CHR$(27) THEN END
IF i$ = "o" THEN GOSUB DispOrbit
IF i$ = "s" OR i$ = "S" THEN GOSUB SelectSat
IF i$ = "v" OR i$ = "V" THEN GOSUB AdvanceVisible
IF i$ = "m" OR i$ = "M" THEN GOSUB AdvanceMax
IF i$ = "a" OR i$ = "A" THEN GOSUB AdvanceOrbit
IF i$ = "c" OR i$ = "C" THEN GOSUB AdvanceSlowOrbit
IF i$ = "f" OR i$ = "F" THEN GOSUB AdvanceFastOrbit
IF i$ = "e" OR i$ = "E" THEN GOSUB DrawMap: GOSUB PrintHeader
IF i$ = "h" OR i$ = "H" THEN GOTO Restart
REM CurTime = GetSimulationParams: GOSUB GetLocation: GOSUB PrintSatData
GOSUB GetSimulationParams: GOSUB GetLocation: GOSUB PrintSatData
GOTO lp

REM ******************** Select a Satellite ********************************
SelectSat:
SCREEN 0: TempSat = Sat: WIDTH , 25
FOR Sat = SatDispStart TO SatDispEnd: GOSUB UnHiliteSat: NEXT Sat
PRINT : COLOR 12: PRINT "Press Return to select satellite";
Sat = TempSat: GOSUB HiliteSat: i$ = ""
WHILE i$ <> CHR$(13)
	i$ = "": WHILE i$ = "": i$ = INKEY$: WEND
	IF LEN(i$) = 2 THEN i = ASC(RIGHT$(i$, 1)) ELSE i = 0
	IF i = 77 OR i = 75 OR i = 72 OR i = 80 THEN GOSUB UnHiliteSat
	IF i = 77 THEN Sat = Sat + 1
	IF i = 75 THEN Sat = Sat - 1
	IF i = 72 THEN Sat = Sat - 5
	IF i = 80 THEN Sat = Sat + 5
	IF i = 81 THEN GOSUB SatPageUp
	IF i = 73 THEN GOSUB SatPageDown
	IF Sat < SatDispStart THEN Sat = SatDispEnd 
	rem IF Sat < SatDispStart THEN Sat = SatDispEnd + Sat + 1
	IF Sat > SatDispEnd THEN Sat = SatDispStart 
	rem IF Sat > SatDispEnd THEN Sat = SatDispStart + (Sat - SatDispEnd - 1)
	GOSUB HiliteSat
	GOSUB ReadSatelliteParam
WEND
OrbitFlag = 0: WIDTH , 25: SCREEN 12: GOSUB DrawMap: GOSUB PrintHeader: RETURN

SatPageUp:
	IF SatDispEnd >= NumSats THEN RETURN
	SatDispStart = SatDispStart + DispLines * 5
	SatDispEnd = SatDispEnd + DispLines * 5
	IF SatDispEnd > NumSats THEN SatDispEnd = NumSats 
	cls
	FOR Sat = SatDispStart TO SatDispEnd: GOSUB UnHiliteSat: NEXT Sat
	PRINT : COLOR 12: PRINT "Press Return to select satellite";
	Sat = SatDispStart: RETURN

SatPageDown:
	IF SatDispStart = 0 THEN RETURN
	SatDispStart = SatDispStart - DispLines * 5
	SatDispEnd = SatDispEnd - DispLines * 5
	IF SatDispStart < 0 THEN SatDispStart = 0 : SatDispEnd = DispLines*5
	IF SatDispEnd - SatDispStart < DispLines*5 THEN SatDispEnd = SatDispStart + DispLines*5 - 1
	cls
	FOR Sat = SatDispStart TO SatDispEnd: GOSUB UnHiliteSat: NEXT Sat
	PRINT : COLOR 12: PRINT "Press Return to select satellite";
	Sat = SatDispStart: RETURN

UnHiliteSat:
SatPos = Sat MOD DispLines*5
x = (SatPos MOD 5) * 15 + 1: y = INT(SatPos / 5) + 1
COLOR 10, 0: LOCATE y, x: PRINT SatName$(Sat); "    ";
RETURN

HiliteSat:
SatPos = Sat MOD DispLines*5
x = (SatPos MOD 5) * 15 + 1: y = INT(SatPos / 5) + 1
COLOR 12, 1: LOCATE y, x: PRINT SatName$(Sat); "    "; : COLOR 10, 0
RETURN

REM ********************* Read Site Parameters ***************************
GetSiteParams:
OPEN "disp.cfg" FOR INPUT AS 1
LINE INPUT #1, a$: satfile$ = a$
LINE INPUT #1, a$: qth$ = a$
LINE INPUT #1, a$
SiteLat = VAL(a$) * RadDeg
LINE INPUT #1, a$
SiteLong = VAL(a$) * RadDeg
LINE INPUT #1, a$
SiteAltitude = VAL(a$) / 1000: REM convert to km
LINE INPUT #1, a$
SiteMinElev = VAL(a$) * RadDeg
LINE INPUT #1, a$
UTC = VAL(a$)
CLOSE #1: RETURN

REM ****************** Get the location of a Satellite **********************
GetLocation:
SemiMajorAxis = 331.25 * EXP(2 * LOG(MinutePerDay / EpochMeanMotion) / 3)
GOSUB GetPrecession
ReferenceOrbit = EpochMeanAnomaly / PI2 + EpochOrbNum!
AverageMotion = EpochMeanMotion + (CurTime - EpochDay) * Decay / 2
CurrentMotion = EpochMeanMotion + (CurTime - EpochDay) * Decay
CurrentOrbit = ReferenceOrbit + (CurTime - EpochDay) * AverageMotion
MeanAnomaly = (CurrentOrbit - INT(CurrentOrbit)) * PI2
Phase = MeanAnomaly / PI2 * 360
TrueAnomaly = Kepler(MeanAnomaly, Ecc)
Radius = SemiMajorAxis * (1 - (Ecc * Ecc)) / (1 + Ecc * COS(TrueAnomaly))
GOSUB GetSubSatPoint: GOSUB GetBearings

IF CurTime - PrevTime < .00001 THEN GOTO SkipDopler
Velocity = -(Range - PrevRange) / ((CurTime - PrevTime) * SeconderDay)
PrevTime = CurTime
PrevRange = Range
Freq = BeaconFreq * 1000000: REM Convert to Hz
Doppler = Freq * Velocity / c
Freq = Freq + Doppler
SkipDopler:

Az = Azimuth * DegRad
El = Elevation * DegRad
Dist = Range
Height = Radius - EarthRadius
Lat = SatLat * DegRad
Longit = SatLong * DegRad
RETURN
 
REM **************** Get Satellite Sub Point ****************************
GetSubSatPoint:
ArgPerigee = EpochArgPer + (CurTime - EpochDay) * PerigeePrecession
SatLat = asin(SIN(Incl) * SIN(ArgPerigee + TrueAnomaly))
RAAN = EpochRAAN - (CurTime - EpochDay) * RAANPrecession
SatLong = -acos(COS(TrueAnomaly + ArgPerigee) / COS(SatLat))
IF ((Incl > PI / 2 AND SatLat < 0) OR (Incl < PI / 2 AND SatLat > 0)) THEN SatLong = -SatLong
SatLong = SatLong + RAAN
REM Convert from celestial to terrestrial coordinates
SatLong = SatLong - (PI2 * (CurTime * SiderealSolar + RefSidereal))
SatLong = -SatLong: REM Make West be positive
i = INT(SatLong / PI2)
IF (i < 0) THEN i = i - 1
SatLong = SatLong - i * PI2
RETURN

REM **************** Calculate Perigee Precession **********************
GetPrecession:
TempEcc = Ecc * Ecc: TempIncl = COS(Incl)
TempEcc1 = (1 - TempEcc) * (1 - TempEcc)
TempPre = (EarthRadius / SemiMajorAxis) ^ 3.5
RAANPrecession = 9.95 * TempPre * TempIncl / (TempEcc1) * RadDeg
PerigeePrecession = 4.97 * TempPre * (5 * (TempIncl * TempIncl) - 1) / (TempEcc1) * RadDeg
RETURN

REM ************** Calculate Bearings *********************************
GetBearings:
SinSiteLat = SIN(SiteLat): sinSiteLong = SIN(SiteLong)
CosSiteLat = COS(SiteLat): cosSiteLong = COS(SiteLong)
SinSatLat = SIN(SatLat): SinSatLong = SIN(SatLong)
CosSatLat = COS(SatLat): CosSatLong = COS(SatLong)
LongDiff = SiteLong - SatLong
CosBeta = SinSiteLat * SinSatLat + CosSiteLat * CosSatLat * COS(LongDiff)
SinBeta = SQR(1 - CosBeta * CosBeta)
Azimuth = acos((SinSatLat - SinSiteLat * CosBeta) / CosSiteLat / SinBeta)
 
IF (LongDiff < -PI) THEN LongDiff = LongDiff + PI2
IF (LongDiff > PI) THEN LongDiff = LongDiff - PI2
IF (LongDiff < 0) THEN Azimuth = PI2 - Azimuth
 
REM    /* Convert to geocentric radius */
SitAlt = SiteAltitude + EarthRadius * (1 - EarthFlat / 2 + EarthFlat / 2 * COS(2 * SiteLat))
Elevation = ATN((Radius * CosBeta - (SitAlt)) / (Radius * SinBeta))
Range = SQR(Radius * Radius + SitAlt * SitAlt - 2 * Radius * SitAlt * CosBeta)
RETURN

REM ************************ Print Satelite Data *************************
PrintSatData:
GOSUB PlotSat
GOSUB PrintCurData
GOSUB PrintElemData
GOSUB PrintSimTime
GOSUB PrintTime
TmpTime = CurTime - (UTC / 24): GOSUB PrTime
RETURN

REM ************ Print Current time, Latitude, and Long *******************
PrintTime:
LOCATE dataline + 3, 1
COLOR 12
PRINT "Current    Time   ";
COLOR 10
PRINT DATE$; "  "; TIME$;
COLOR 11: PRINT "  ";
GOSUB PrintUTC
COLOR 9
PRINT "  Lat  = "; : PRINT USING "##.###"; SiteLat / RadDeg;
RETURN

REM ************************ Print the current time **********************
PrTime:
TYear = RefYear
WHILE TmpTime > 365
	IF TmpTime > 365 THEN TmpTime = TmpTime - 365: TYear = TYear + 1
WEND
TMonth = 1
WHILE TmpTime > 31
	IF TmpTime > 31 THEN TmpTime = TmpTime - MonthDays(TMonth): TMonth = TMonth + 1
WEND
TYear$ = "19" + LTRIM$(STR$(TYear)): TMonth$ = LTRIM$(STR$(TMonth))
TDay$ = LTRIM$(STR$(INT(TmpTime))): TmpTime = TmpTime - INT(TmpTime)
TmpTime = TmpTime * 24: THour$ = LTRIM$(STR$(INT(TmpTime))): TmpTime = TmpTime - INT(TmpTime)
TmpTime = TmpTime * 60: TMin$ = LTRIM$(STR$(INT(TmpTime))): TmpTime = TmpTime - INT(TmpTime)
TmpTime = TmpTime * 60: TSec$ = LTRIM$(STR$(INT(TmpTime))): TmpTime = TmpTime - INT(TmpTime)
IF LEN(TMonth$) = 1 THEN TMonth$ = "0" + TMonth$
IF LEN(TDay$) = 1 THEN TDay$ = "0" + TDay$
IF LEN(THour$) = 1 THEN THour$ = "0" + THour$
IF LEN(TMin$) = 1 THEN TMin$ = "0" + TMin$
IF LEN(TSec$) = 1 THEN TSec$ = "0" + TSec$
LOCATE dataline + 4, 1
COLOR 12
PRINT "Simulation Time   ";
COLOR 10
PRINT TMonth$; "-"; TDay$; "-"; TYear$; "  "; THour$; ":"; TMin$; ":"; TSec$;
COLOR 11: PRINT "  ";
TmpTime = CurTime: GOSUB PrTime1: REM kludge to print out UTC
RETURN

PrTime1:
TYear = RefYear
WHILE TmpTime > 365
	IF TmpTime > 365 THEN TmpTime = TmpTime - 365: TYear = TYear + 1
WEND
TMonth = 1
WHILE TmpTime > 31
	IF TmpTime > 31 THEN TmpTime = TmpTime - MonthDays(TMonth): TMonth = TMonth + 1
WEND
TYear$ = "19" + LTRIM$(STR$(TYear)): TMonth$ = LTRIM$(STR$(TMonth))
TDay$ = LTRIM$(STR$(INT(TmpTime))): TmpTime = TmpTime - INT(TmpTime)
TmpTime = TmpTime * 24: THour$ = LTRIM$(STR$(INT(TmpTime))): TmpTime = TmpTime - INT(TmpTime)
TmpTime = TmpTime * 60: TMin$ = LTRIM$(STR$(INT(TmpTime))): TmpTime = TmpTime - INT(TmpTime)
TmpTime = TmpTime * 60: TSec$ = LTRIM$(STR$(INT(TmpTime))): TmpTime = TmpTime - INT(TmpTime)
IF LEN(TMonth$) = 1 THEN TMonth$ = "0" + TMonth$
IF LEN(TDay$) = 1 THEN TDay$ = "0" + TDay$
IF LEN(THour$) = 1 THEN THour$ = "0" + THour$
IF LEN(TMin$) = 1 THEN TMin$ = "0" + TMin$
IF LEN(TSec$) = 1 THEN TSec$ = "0" + TSec$
PRINT TMonth$; "-"; TDay$; "-"; TYear$; "  "; THour$; ":"; TMin$; ":"; TSec$;
COLOR 9
PRINT "  Long = "; : PRINT USING "###.###"; SiteLong / RadDeg;
RETURN

REM ************** Initialize the simulated (look ahead) time *****************
InitSimTime:
SYear = VAL(MID$(DATE$, 9, 2)): SMonth = VAL(MID$(DATE$, 1, 2))
SDay = VAL(MID$(DATE$, 4, 2)): STime = GetDay(SYear, SMonth, SDay)
Shour = VAL(MID$(TIME$, 1, 2)): SMinute = VAL(MID$(TIME$, 4, 2))
SSecond = VAL(MID$(TIME$, 7, 2))
STime = STime + SMinute / 1440 + SSecond / 86400 + Shour / 24 + UTC / 24
RETURN

REM **************** Get the Simulation time **************************
GetTime:
STime = GetDay(SYear, SMonth, SDay)
STime = STime + SMinute / 1440 + SSecond / 86400 + Shour / 24 + UTC / 24
RETURN

REM *************** Increment the Simulation Time by TimeIncr ***************
IncTime:
SMinute = SMinute + TimeIncr
IncT1:
IF SMinute >= 60 THEN SMinute = SMinute - 60: Shour = Shour + 1: GOTO IncT1
IF Shour < 24 THEN RETURN
Shour = 0: SDay = SDay + 1: RETURN

REM **************** Advance simulation 1 orbit ***************************
DispOrbit:
IF OrbitFlag = 0 THEN GOSUB LoadMap
IF OrbitFlag = 0 THEN GOSUB InitSimTime: OrbitFlag = 1
OrbitTime = OrbitalPeriod / (24 * 60)
FOR SimTime = STime TO (STime + OrbitTime) STEP (OrbitTime / 360)
	CurTime = SimTime: GOSUB GetLocation
	GOSUB PlotOrbit
	IF INKEY$ <> "" THEN RETURN
NEXT SimTime
STime = SimTime - (OrbitTime / 100): PlotFlag = 0
RETURN

REM ********** Advance Orbit Until a Key is Hit **************************
AdvanceOrbit:
REM IF OrbitFlag = 0 THEN CurTime = GetSimulationParams: GOSUB GetLocation
IF OrbitFlag = 0 THEN GOSUB GetSimulationParams: GOSUB GetLocation
IF OrbitFlag = 0 THEN GOSUB DrawMap: GOSUB PrintHeader: GOSUB PrintSatData
IF OrbitFlag = 0 THEN GOSUB InitSimTime: OrbitFlag = 1
OrbitTime = OrbitalPeriod / (24 * 60)
TimeIncr = OrbitTime / 90
WHILE INKEY$ = ""
	CurTime = STime: GOSUB GetLocation
	GOSUB PlotOrbit: GOSUB PrintSimData
	TmpTime = CurTime - (UTC / 24): GOSUB PrTime
	IF Ecc > .45 THEN
		IF Phase > 335 OR Phase < 25 THEN TimeIncr = OrbitTime / 450 ELSE TimeIncr = OrbitTime / 90
	END IF
	STime = STime + TimeIncr
WEND
STime = STime - TimeIncr: PlotFlag = 0
RETURN

REM ********** Advance Orbit Slow Until a Key is Hit **************************
AdvanceSlowOrbit:
REM IF OrbitFlag = 0 THEN CurTime = GetSimulationParams: GOSUB GetLocation
IF OrbitFlag = 0 THEN GOSUB GetSimulationParams: GOSUB GetLocation
IF OrbitFlag = 0 THEN GOSUB DrawMap: GOSUB PrintHeader: GOSUB PrintSatData
IF OrbitFlag = 0 THEN GOSUB InitSimTime: OrbitFlag = 1
OrbitTime = OrbitalPeriod / (24 * 60)
TimeIncr = OrbitTime / (90*5)
WHILE INKEY$ = ""
	CurTime = STime: GOSUB GetLocation
	GOSUB PlotOrbit: GOSUB PrintSimData
	TmpTime = CurTime - (UTC / 24): GOSUB PrTime
	IF Ecc > .45 THEN
		IF Phase > 335 OR Phase < 25 THEN TimeIncr = OrbitTime / 450 ELSE TimeIncr = OrbitTime / (90*5)
	END IF
	STime = STime + TimeIncr
WEND
STime = STime - TimeIncr: PlotFlag = 0
RETURN

REM ********** Advance Orbit Fast Until a Key is Hit **************************
AdvanceFastOrbit:
REM IF OrbitFlag = 0 THEN CurTime = GetSimulationParams: GOSUB GetLocation
IF OrbitFlag = 0 THEN GOSUB GetSimulationParams: GOSUB GetLocation
IF OrbitFlag = 0 THEN GOSUB DrawMap: GOSUB PrintHeader: GOSUB PrintSatData
IF OrbitFlag = 0 THEN GOSUB InitSimTime: OrbitFlag = 1
OrbitTime = OrbitalPeriod / (24 * 60)
TimeIncr = OrbitTime / 90*5
WHILE INKEY$ = ""
	CurTime = STime: GOSUB GetLocation
	GOSUB PlotOrbit: GOSUB PrintSimData
	TmpTime = CurTime - (UTC / 24): GOSUB PrTime
	IF Ecc > .45 THEN
		IF Phase > 335 OR Phase < 25 THEN TimeIncr = OrbitTime / 450 ELSE TimeIncr = OrbitTime / 90*5
	END IF
	STime = STime + TimeIncr
WEND
STime = STime - TimeIncr: PlotFlag = 0
RETURN

REM ********** Advance Orbit Until visible **************************
AdvanceVisible:
REM IF OrbitFlag = 0 THEN CurTime = GetSimulationParams: GOSUB GetLocation
IF OrbitFlag = 0 THEN GOSUB GetSimulationParams: GOSUB GetLocation
IF OrbitFlag = 0 THEN GOSUB DrawMap: GOSUB PrintHeader: GOSUB PrintSatData
IF OrbitFlag = 0 THEN GOSUB InitSimTime: OrbitFlag = 1
OrbitTime = OrbitalPeriod / (24 * 60)
TimeIncr = OrbitTime / 180
WHILE El < 0
	CurTime = STime: GOSUB GetLocation
	GOSUB PlotOrbit: GOSUB PrintSimData
	TmpTime = CurTime - (UTC / 24): GOSUB PrTime
	IF El > -5 THEN TimeIncr = OrbitTime / 360 ELSE TimeIncr = OrbitTime / 90
	STime = STime + TimeIncr
	IF INKEY$ <> "" THEN PlotFlag = 0: STime = STime - TimeIncr: RETURN
WEND
STime = STime - TimeIncr: PlotFlag = 0
RETURN

REM ********** Advance Orbit Until Max overhead **************************
AdvanceMax:
OldEl = -180
REM IF OrbitFlag = 0 THEN CurTime = GetSimulationParams: GOSUB GetLocation
IF OrbitFlag = 0 THEN GOSUB GetSimulationParams: GOSUB GetLocation
IF OrbitFlag = 0 THEN GOSUB DrawMap: GOSUB PrintHeader: GOSUB PrintSatData
IF OrbitFlag = 0 THEN GOSUB InitSimTime: OrbitFlag = 1
OrbitTime = OrbitalPeriod / (24 * 60)
TimeIncr = OrbitTime / 90
WHILE INKEY$ = ""
	CurTime = STime: GOSUB GetLocation
	IF El > 0 THEN
		IF OldEl > El THEN
			PlotFlag = 0: STime = STime - TimeIncr: RETURN
		END IF
	END IF
	GOSUB PlotOrbit: GOSUB PrintSimData
	TmpTime = CurTime - (UTC / 24): GOSUB PrTime
	IF El > -5 THEN TimeIncr = OrbitTime / 360 ELSE TimeIncr = OrbitTime / 90
	STime = STime + TimeIncr
	OldEl = El
WEND
STime = STime - TimeIncr: PlotFlag = 0
RETURN

REM ************* Show current position of Satellite ***********************
PlotSat:
IF El > 0 THEN SatColor% = 14 ELSE SatColor% = 13
yp = (90 - Lat) * ymax / 180
xp = (360 - Longit) * xmax / 360
PSET (oldxp, oldyp), PrevColor1%
PSET (oldxp + 1, oldyp), PrevColor2%
PSET (oldxp + 2, oldyp), PrevColor3%
PSET (oldxp - 1, oldyp), PrevColor4%
PSET (oldxp - 2, oldyp), PrevColor5%
PSET (oldxp, oldyp + 1), PrevColor6%
PSET (oldxp, oldyp + 2), PrevColor7%
PSET (oldxp, oldyp - 1), PrevColor8%
PSET (oldxp, oldyp - 2), PrevColor9%
PrevColor1% = POINT(xp, yp)
PrevColor2% = POINT(xp + 1, yp)
PrevColor3% = POINT(xp + 2, yp)
PrevColor4% = POINT(xp - 1, yp)
PrevColor5% = POINT(xp - 2, yp)
PrevColor6% = POINT(xp, yp + 1)
PrevColor7% = POINT(xp, yp + 2)
PrevColor8% = POINT(xp, yp - 1)
PrevColor9% = POINT(xp, yp - 2)
PSET (xp, yp), SatColor%
PSET (xp + 1, yp), SatColor%
PSET (xp + 2, yp), SatColor%
PSET (xp - 1, yp), SatColor%
PSET (xp - 2, yp), SatColor%
PSET (xp, yp + 1), SatColor%
PSET (xp, yp + 2), SatColor%
PSET (xp, yp - 1), SatColor%
PSET (xp, yp - 2), SatColor%
oldxp = xp: oldyp = yp
RETURN

REM ******************* Plot next Orbit *********************************
PlotOrbit:
IF El > 0 THEN SatColor = 14 ELSE SatColor = 13
yp = (90 - Lat) * ymax / 180
xp = (360 - Longit) * xmax / 360
IF ABS(xp - oldx) > 100 THEN PSET (xp, yp), SatColor: PlotFlag = 1: GOTO Plot1
IF PlotFlag = 0 THEN PSET (xp, yp), SatColor: PlotFlag = 1: GOTO Plot1
LINE -(xp, yp), SatColor
Plot1:
oldx = xp: oldy = yp
IF ABS(Phase - 178) < 4 AND Phflag = 0 THEN CIRCLE (xp, yp), 2, 11: Phflag = 1
IF ABS(Phase - 2) < 4 AND Phflag = 0 THEN CIRCLE (xp, yp), 2, 12: Phflag = 1
IF ABS(Phase - 178) > 4 AND ABS(Phase - 2) > 4 THEN Phflag = 0
RETURN

REM ************** Print Header Information *****************************
PrintHeader:
LOCATE dataline + 5, 1
COLOR 12: PRINT "EpochDay   Incl   EpochRAAN   Ecc    ArgPer   MeanAnom  MeanMot   Decay";
LOCATE dataline, 1
COLOR 12: PRINT "Satellite  Azimuth  Elev  Range   Height   Lat    Long   Orbit     Phase"
RETURN

REM ************** Print Simulation Data *********************************
PrintSimTime:
CurTime = STime: GOSUB GetLocation
PrintSimData:
LOCATE dataline + 2, 1
COLOR 10
PRINT "            ";
IF El > 0 THEN COLOR 14 ELSE COLOR 13
PRINT USING "###.#  "; Az;
PRINT USING "###.#  "; El;
PRINT USING "#####  "; Dist;
PRINT USING "#####.#  "; Height;
PRINT USING "###.#  "; Lat;
PRINT USING "###.#  "; Longit;
PRINT USING "#####.###"; CurrentOrbit;
PRINT USING "  ###.#  "; Phase;
RETURN

REM ***************** Print Satellite Element Data ***********************
PrintElemData:
LOCATE dataline + 6, 1
COLOR 13
PRINT USING "####.###"; EpochDay;
PRINT USING "  ###.###"; Incl / RadDeg;
PRINT USING "  ###.###"; EpochRAAN / RadDeg;
PRINT USING "  .######"; Ecc;
PRINT USING "  ###.###"; EpochArgPer / RadDeg;
PRINT USING "  ###.###"; EpochMeanAnomaly / RadDeg;
PRINT USING "  ###.###"; EpochMeanMotion;
PRINT USING "  #.#######"; Decay;
RETURN

REM ***************** Print Current Data **********************************
PrintCurData:
LOCATE dataline + 1, 1
COLOR 10: PRINT SatName$(Sat); " ";
IF El > 0 THEN COLOR 14 ELSE COLOR 13
PRINT USING "###.#  "; Az;
PRINT USING "###.#  "; El;
PRINT USING "#####  "; Dist;
PRINT USING "#####.#  "; Height;
PRINT USING "###.#  "; Lat;
PRINT USING "###.#  "; Longit;
REM PRINT USING "###.###  "; Velocity;
REM PRINT USING "###.####  "; Freq / 1000000
PRINT USING "#####.###"; CurrentOrbit;
PRINT USING "  ###.#  "; Phase;
RETURN

REM **************** Load in Satellite data *******************
GetSatelliteParams:
CLS : PRINT "Reading Satellite Data ...."
OPEN satfile$ FOR INPUT AS #1
	WHILE NOT EOF(1)
		BeaconFreq = 145.825
		SatIndex(Sat) = SEEK(1)
		LINE INPUT #1, SatName$(Sat)
		LINE INPUT #1, lineb$
		LINE INPUT #1, lined$
		LOCATE 1, 30: PRINT Sat, SatName$(Sat); "    ";
		IF Sat = MaxSats - 1 THEN GOTO Havesats1
		Sat = Sat + 1
	WEND
Sat = Sat - 1
NumSats = Sat
Havesats1:
IF Sat > DispLines * 5 THEN SatDispEnd = (DispLines * 5) - 1 ELSE SatDispEnd = Sat
SatDispStart = 0
CLOSE #1

REM **************** Read in Satellite data *******************
ReadSatelliteParam:
		BeaconFreq = 145.825

		OPEN satfile$ FOR INPUT AS #1
		SEEK #1, SatIndex(Sat)
		LINE INPUT #1, a$
		LINE INPUT #1, b$
		LINE INPUT #1, d$
		CLOSE #1
  
		EpochDay = VAL(MID$(b$, 21, 12))
		EpochYear = VAL(MID$(b$, 19, 2))
		EpochDay = GetDay(EpochYear, 1, EpochDay)

		Incl = VAL(MID$(d$, 9, 8)) * RadDeg
		EpochRAAN = VAL(MID$(d$, 18, 8)) * RadDeg
		Ecc = VAL("0." + MID$(d$, 27, 7))
		EpochArgPer = VAL(MID$(d$, 35, 8)) * RadDeg
		EpochMeanAnomaly = VAL(MID$(d$, 44, 8)) * RadDeg
		EpochMeanMotion = VAL(MID$(d$, 53, 11))
		Decay = VAL(MID$(b$, 34, 10))
		EpochOrbNum! = VAL(MID$(d$, 64, 5))
	 
		IF EpochMeanMotion > 0 THEN OrbitalPeriod = 24 / EpochMeanMotion * 60
		RETURN

REM ****************** Draw the world map ********************************
DrawMap:
CLS
OUT Addr, 5: OUT Vga, 2: REM Read Mode 0 - Write Mode 2
OUT Addr, 3: OUT Vga, &H10: REM Or in the data

CALL vgadata(VARSEG(MData1%(1)), VARPTR(MData1%(1)), 2, DataLen)
CALL vgadata(VARSEG(MData2%(1)), VARPTR(MData2%(1)), 4, DataLen)

OUT Addr, 5: OUT Vga, 0: REM Reset Values
OUT Addr, 8: OUT Vga, &HFF

LINE (1, 1)-(1, ymax), 1: LINE -(xmax, ymax), 1: LINE -(xmax, 1), 1: LINE -(1, 1), 1

FOR y = 1 TO ymax STEP ymax / 6
	x = x + 1
	LINE (1, y)-(xmax, y), 1
	IF x = 4 THEN LINE (1, y)-(xmax, y), 11
NEXT y: y = 0
FOR x = 1 TO xmax STEP xmax / 12
	y = y + 1
	LINE (x, 1)-(x, ymax), 1
	IF y = 7 THEN LINE (x, 1)-(x, ymax), 11
NEXT x
RETURN

PrintUTC:
a$ = TIME$: hour$ = MID$(a$, 1, 2): hour = VAL(hour$): h = hour
min$ = MID$(a$, 4, 2): sec$ = MID$(a$, 7, 2): m = VAL(min$): s = VAL(sec$)
b$ = DATE$: month$ = MID$(b$, 1, 2)
day$ = MID$(b$, 4, 2): day = VAL(day$)
year$ = MID$(b$, 7, 4)
hour = hour + UTC
IF hour > 23 THEN
	hour = hour - 24
	day = day + 1
END IF
day$ = LTRIM$(STR$(day)): IF LEN(day$) = 1 THEN day$ = "0" + day$
hour$ = LTRIM$(STR$(hour)): IF LEN(hour$) = 1 THEN hour$ = "0" + hour$
UtcTime$ = month$ + "-" + day$ + "-" + year$ + "  " + hour$ + ":" + min$ + ":" + sec$
PRINT UtcTime$;
RETURN

REM ******************** Print Command Menu ****************************
PrintMenu:
CLS
COLOR 14
PRINT "                    Satelite Orbit Display Program"
PRINT "              By Robert W. Beyer (WA6JJY) CIS:71021,2074"
PRINT "                      Version 1.50   30 Nov 1991"
COLOR 13
PRINT
PRINT "                  Command Summary"
COLOR 10
PRINT
PRINT "    a            - Advance orbit"
PRINT "    c            - Advance orbit (Slow)"
PRINT "    f            - Advance orbit (Fast)"
PRINT "    v            - Advance orbit until above horizon (visible)"
PRINT "    m            - Advance orbit until next maximum visible elevation"
PRINT "    e            - Erase all orbit path data"
PRINT "    o            - Display next orbit"
PRINT "    s            - Select Satellite"
PRINT "    F1           - Help screen"
PRINT "    (arrow keys) - Select different Satellites"
PRINT "    ESC          - Exit Program"
COLOR 11
PRINT
PRINT "NOTES: A yellow orbit path indicates that the satellite is currently"
PRINT "       visible, a violet path is below the horizon.  The red circle"
PRINT "       indicates perigee and the blue circle apogee. "
PRINT
COLOR 12
PRINT "                 - Hit any key to run -"
WHILE INKEY$ = "": WEND
RETURN

REM ***************** Load in the Map Data ******************************
LoadMap:
DEF SEG = VARSEG(MData1%(1))
BLOAD "dplane1.dat", VARPTR(MData1%(1))
DEF SEG = VARSEG(MData2%(1))
BLOAD "dplane2.dat", VARPTR(MData2%(1))
RETURN

REM **************** Read in Time and convert ************************
GetSimulationParams:
year = VAL(MID$(DATE$, 9, 2))
month = VAL(MID$(DATE$, 1, 2))
day = VAL(MID$(DATE$, 4, 2))
CTime = GetDay(year, month, day)
hour = VAL(MID$(TIME$, 1, 2))
minute = VAL(MID$(TIME$, 4, 2))
second = VAL(MID$(TIME$, 7, 2))
hour = hour + minute / 60
hour = hour + second / 3600
CTime = CTime + hour / 24 + UTC / 24: REM Convert to UTC
REM GetSimulationParams = CTime
CurTime = CTime
RETURN

REM ******************** Functions start here **************************



REM ******************** Arc cosine function ***************************
FUNCTION acos (x)
acos = PI / 2 - ATN(x / SQR(1 - x * x))
END FUNCTION

REM ******************** Arc sine Function ******************************
FUNCTION asin (x)
asin = ATN(x / SQR(1 - x * x))
END FUNCTION

FUNCTION GetDay (year, month, day)
REM  Get the Day Number for a given date. January 1 of the reference year
REM  is day 1. Note that the Day Number may be negative, if the sidereal
REM  reference is in the future.
MonthDays(1) = 31
MonthDays(2) = 28
MonthDays(3) = 31
MonthDays(4) = 30
MonthDays(5) = 31
MonthDays(6) = 30
MonthDays(7) = 31
MonthDays(8) = 31
MonthDays(9) = 30
MonthDays(10) = 31
MonthDays(11) = 30
MonthDays(12) = 31
TmpYear = year
TempDay = day
TempMon = month

WHILE (TmpYear > RefYear)
	TmpYear = TmpYear - 1
	IF (TmpYear MOD 4 = 0) THEN TempDay = TempDay + 366 ELSE TempDay = TempDay + 365
WEND

IF (year MOD 4 = 0) THEN MonthDays(2) = MonthDays(2) + 1: REM Add in Leap year
WHILE (TempMon > 1)
	TempMon = TempMon - 1
	TempDay = TempDay + MonthDays(TempMon)
WEND

IF (year MOD 4 = 0) THEN MonthDays(2) = MonthDays(2) - 1: REM Take out Leap year
GetDay = TempDay
END FUNCTION

REM **************** Solve Kepler's equation **************************
FUNCTION Kepler (MeanAnomaly, TempEcc)
REM  Solve Kepler's equation
REM  Inputs:
REM       MeanAnomaly     Time since last perigee, in radians.
REM                       PI2 = one complete orbit.
REM       TempEcc         Eccentricity of orbit's ellipse.
REM  Output:
REM       TrueAnomaly     Angle between perigee, geocenter, and
REM                       current position.

E = MeanAnomaly: REM /* Initial guess */

Er = (E - TempEcc * SIN(E) - MeanAnomaly) / (1 - TempEcc * COS(E))
E = E - Er

WHILE (ABS(Er) >= .000001)
	Er = (E - TempEcc * SIN(E) - MeanAnomaly) / (1 - TempEcc * COS(E))
	E = E - Er
WEND

IF (ABS(E - PI) < .000001) THEN TrueAnomaly = PI ELSE TrueAnomaly = 2 * ATN(SQR((1 + TempEcc) / (1 - TempEcc)) * TAN(E / 2))
IF (TrueAnomaly < 0) THEN TrueAnomaly = TrueAnomaly + PI2
Kepler = TrueAnomaly
END FUNCTION

