/* domain.c */
/*
 * history:
 * 1.0  ka9q	22-2-89
 * 1.1  pa0gri	3-3-89	added basic server function
 * 1.2	pa0gri	7-3-89	made get_rr() more compliant with rfc1033
 * 1.3  ka9q    30-3-89 various enhancements and pwait()
 * 1.4  pa0gri  5-4-89  various comments from Anders SM0RGV
 * 1.5  pa0gri  7-4-89  added routine dqualify() and phil's 5-4-89 mods
 * 1.6  pa0gri  20-4-89 added db_load and rudgidized get_rr()
 * 1.7  pa0gri  07-8-89 adapted to 7 aug version (v17)
 * 1.8  ka9q    29-9-89 minor changes
 * 1.9  pa0gri  5-10-89 major rewrite in suffix handling. dqualify() gone.
 * 2.0  sm0rgv  8-10-89 major changes to the server and numerous bugs fixed.
 * 2.1  pa0gri  12-11-89 more bugg fixes and nslookup() added. (debug tool)
 * 2.2  pa0gri  16-11-89 domain file in memory, domain save added.
 * 2.3  pa0gri  20-11-89 domain bootp basis developement.
 * 3.0  pa0gri  15-12-89 Major rewrite.
 * 3.1  pa0gri  18-12-89 Some more features. trace splitup.
 * 3.2  pa0gri  09-01-90 add timeout count to resolve and Mprunning test.
 * 3.3  pa0gri  14-01-90 added analsuf and fixed some ommisions. (bugs!)
 * 3.4  pa0gri  15-02-90 haveaa() cleanup and domain retry counter.
 * 3.5  pa0gri  19-04-90 fixed incompatibility with NET-NOS and some cleanup
 * 4.0  pa0gri  19-05-90 Major restructure on socket handling 
 */
#include <stdio.h>
#include <ctype.h>
#include "global.h"
#include "mbuf.h"
#include "timer.h"
#include "netuser.h"
#include "socket.h"
#include "iface.h"
#include "cmdparse.h"
#include "proc.h"
#include "domain.h"
#include "commands.h"

extern int Mprunning;
static struct dserver *Dlist;		/* List of potential servers */
static struct dserver *Dserver;		/* Current one being used */
struct rr *Rrlist[NRLIST];
static struct rr *Rrlistl[NRLIST];
static struct rr *db_SOA;
static char *Dsuffix;	/* Default suffix for names without trailing period */
static struct dlist *Lorigin;	/* list of origins loaded */
static char *Dorigin;		/* Origin of domains during domain load */
static int Dsocket;			/* Socket to use for domain queries */
static struct proc *Drx;		/* Task ID of domain receiver */
static int Ddebug = 0;
static int Dloopback = 0;
static int Dtrace = 0;
int Db_initialize = 0;
static int DBloaded = 0;
static unsigned short Dretries = 0;
static unsigned short Dtimeout = 30;
static int16 Dhaveaa = 1;
static char *Dtypes[] = {
	"",
	"A",
	"NS",
	"MD",
	"MF",
	"CNAME",
	"SOA",
	"MB",
	"MG",
	"MR",
	"NULL",
	"WKS",
	"PTR",
	"HINFO",
	"MINFO",
	"MX",
	"TXT"
};
static int Dstypes[] = {
	TYPE_SOA,
	TYPE_NS,
	TYPE_MX,
	TYPE_MD,
	TYPE_MF,
	TYPE_CNAME,
	TYPE_PTR,
	TYPE_A,
	TYPE_MB,
	TYPE_MG,
	TYPE_MR,
	TYPE_NULL,
	TYPE_WKS,
	TYPE_HINFO,
	TYPE_MINFO,
	TYPE_TXT,
	0
};
static int Ndtypes = 17;
static char delim[] = " \t\r\n";
static struct {
	char *name;
	int32 address;
} cache;

static char *Dopcode[] = {
	"QUERY",
	"IQUERY",
	"STATUS",
	"Undef",
	"Undef",
	"Undef",
	"Undef",
	"Undef",
	"Undef",
	"UPDATEA",
	"UPDATED",
	"UPDATEDA",
	"UPDATEM",
	"UPDATEMA",
	"ZONEINIT",
	"ZONEREF"
};
static char *Drcode[] = {
	"NOERROR",
	"FORMERR",
	"SERVFAIL",
	"NXDOMAIN",
	"NOTIMP",
	"REFUSED",
	"?",
	"?",
	"?",
	"?",
	"?",
	"?",
	"?",
	"?",
	"?",
	"NOCHANGE"
};
static int add_rr __ARGS((FILE *fp,struct rr *rrp1));
static struct dhdr *bld_dhdr __ARGS((int16 qr,int16 op,int16 flags,int16 rcode, char *name,int class,int type));
static struct rr *copy_rr __ARGS((struct rr *rr));
static struct rr *dfind __ARGS((FILE *dbase,char *name,int type,struct rr_memory *memory));
static struct rr *dfindf __ARGS((FILE *dbase,char *name,int type,struct rr_memory *memory));
static int dfind_cmp __ARGS((struct rr *rr,char *name,int nlen,int dlen));
static char *dn_compress __ARGS((char *cp,char *name));
static int doadds   __ARGS((int argc,char *argv[],void *p));
static int dobootp  __ARGS((int argc,char *argv[],void *p));
static int dodropds __ARGS((int argc,char *argv[],void *p));
static int dolistds __ARGS((int argc,char *argv[],void *p));
static int doload  __ARGS((int argc,char *argv[],void *p));
static int donslookup  __ARGS((int argc,char *argv[],void *p));
static int doretries  __ARGS((int argc,char *argv[],void *p));
static int dosave  __ARGS((int argc,char *argv[],void *p));
static int dosuffix __ARGS((int argc,char *argv[],void *p));
static int dotimeout  __ARGS((int argc,char *argv[],void *p));
static int doddebug __ARGS((int argc,char *argv[],void *p));
static int dodloopback __ARGS((int argc,char *argv[],void *p));
static int dodtrace __ARGS((int argc,char *argv[],void *p));
static void drx __ARGS((int unused,void *u, void *p));
static struct dserver *dslookup __ARGS((int32 server));
static void free_dhdr __ARGS((struct dhdr *dp));
static void free_qu __ARGS((struct quest *qp));
static struct rr *get_rr __ARGS((FILE *fp,struct rr_memory *memory));
static int isaddr __ARGS((char *s));
static void proc_bootp __ARGS((int unused,void *d,void *b));
static void proc_answer __ARGS((struct dhdr *dhdr,struct dserver *dp));
static void proc_query __ARGS((int unused,void *d,void *b));
static void put_rr __ARGS((FILE *fp,struct rr *rrp,int addorg));
static int cmp_rr __ARGS((struct rr *rr1,struct rr *rr2));
static int haveaa __ARGS((char *name));
static struct rr *resolverec __ARGS((char *name,int type));
static char *htonrr __ARGS((struct rr *rr,char *cp));
static char *type2str __ARGS((int type));
static int res_mkbuf __ARGS((struct dhdr *dhdr,char *buffer,int16 bufsize));
static void analsuf __ARGS((struct rr *rrp));
static void dumpdhdr __ARGS((struct dhdr *dhdr));
static int sendquery __ARGS((int32 address,char *buf,int len,struct mbuf **bp,int32 timeout));

static struct cmds Dcmds[] = {
	"addserver",	doadds,		0, 2,
		"domain addserver <domainserver> [port]",
	"bootp",	dobootp,	0, 2,
		"domain bootp <domainserver> [filename]",
	"debug",	doddebug,	0, 0, NULLCHAR,
	"dropserver",	dodropds,	0, 2,
		"domain dropserver <domainserver> [port]",
	"listservers",	dolistds,	0, 0, NULLCHAR,
	"load",		doload,		0, 0, 
		"domain load [file]",
	"loopback",	dodloopback,	0, 0, NULLCHAR,
	"nslookup",	donslookup,	0, 3,
		"domain nslookup <server> <record type> <name>",
	"retries",	doretries,	0, 0, NULLCHAR,
	"save",		dosave,		0, 0, 
		"domain save [file]",
	"suffix",	dosuffix,	0, 0, NULLCHAR,
	"timeout",	dotimeout,	0, 0, NULLCHAR,
	"trace",	dodtrace,	0, 0, NULLCHAR,
	NULLCHAR,
};
static int
doddebug(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	return setbool(&Ddebug,"Domain debug",argc,argv);
}
static int
dodloopback(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	return setbool(&Dloopback,"Domain loopback",argc,argv);
}
static int
dodtrace(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	return setbool(&Dtrace,"Domain trace",argc,argv);
}
int
dodomain(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	return subcmd(Dcmds,argc,argv,p);	
}
int
doretries(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	return setshort(&Dretries,"Domain server retries",argc,argv);	
}
int
dotimeout(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	return setshort(&Dtimeout,"Domain server timeout",argc,argv);	
}
static int
dosuffix(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	if(argc < 2){
		if(Dsuffix != NULLCHAR)
			tprintf("%s\n",Dsuffix);
		else
			tprintf("No domain suffix defined.\n");
		return 0;
	}
	if(strcmp(argv[1],"none") == 0) {
		free(Dsuffix);
		Dsuffix = NULLCHAR;	/* clear out suffix */
	} else
		if(argv[1][strlen(argv[1])-1] != '.')
			tprintf("%s is not a valid suffix.\n",argv[1]);
		else {
			free(Dsuffix);
			Dsuffix = strdup(argv[1]);
		}
	return 0;
}
static int
doadds(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	struct dserver *dp;
	int32 address;

	if((address = resolve(argv[1])) == 0){
		tprintf("Resolver %s unknown\n",argv[1]);
		return 1;
	}
	dp = (struct dserver *)callocw(1,sizeof(struct dserver));
	dp->address = address;
	if (argc > 2)
		dp->port = (int16) atoi(argv[2]);
	else
		dp->port = IPPORT_DOMAIN;
	/* Pick a wait time for response: it takes about 15 sec on my AT
	   to scan trough the domain.txt file. Taking 3 scans (max)
	   and giving a little strech to the channel i come to 60 secs */
	dp->srtt = Dtimeout * 1000L / MSPTICK;
	dp->mdev = 0;
	dp->timeout = dp->srtt + 3;
	dp->next = Dlist;
	if(dp->next != NULLDOM)
		dp->next->prev = dp;
	if(Dtrace)
		printf("adding %s to Dlist and make it Dserver\n",argv[1]);
	Dlist = dp;
	Dserver = dp;	/* Make this the first one we try next */
	if(Ip_addr != address)
		Dhaveaa = 0; /* another is also responsible */
	else {
		if(Drx == NULLPROC)
		/* Start domain server task upon my own addserver */
			Drx = newproc("Domain listener",1024,drx,0,NULL,NULL);
	}
	return 0;
}
static int
dobootp(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	struct sockaddr_in nslook;
	struct dhdr *dhdr;
	int32 address;
	int16 flags;
	char *buf, *name, *bootname;
	int len;

	DBloaded = 1;		/* disable domain file access */
	name = argv[1];
	if(*name == '[')
		address = aton(name+1);
	else
		if(isaddr(name))
			address = aton(name);
		else {
			tprintf("Cannot resolve %s currently\n",argv[1]);
			return 1;
		}
	if(argc > 2)
		bootname = argv[2];
	else
		bootname = "/domain.txt";
/* do zoneinit from remote server */
	flags = DOM_DORECURSE | DOM_CANRECURSE;	/* Recursion desired */
	dhdr = bld_dhdr(QUERY,ZONEINIT,flags,NO_ERROR,bootname,CLASS_IN,TYPE_ANY);
	buf = mallocw(512);
	len = res_mkbuf(dhdr,buf,512);
	nslook.sin_family = AF_INET;
	nslook.sin_port = IPPORT_DOMAIN;
	nslook.sin_addr.s_addr = address;
	if(Dtrace)
		dumpdhdr(dhdr);
	sendto(Dsocket,buf,len,0,(char *)&nslook,sizeof(nslook));
	free_dhdr(dhdr);
	free(buf);
	return 0;
}
static int
dodropds(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	struct dserver *dp;
	int32 addr;
	int port;

	if((addr = resolve(argv[1])) == 0) {
		tprintf("Resolver %s unknown\n",argv[1]);
		return 1;
	}
	if (argc > 2)
		port = (int16) atoi(argv[2]);
	else
		port = IPPORT_DOMAIN;
	for(dp = Dlist;dp != NULLDOM;dp = dp->next)
		if(addr == dp->address && port == dp->port)
			break;

	if(dp == NULLDOM){
		if(Dtrace)
			tprintf("resolver %s on port %d not found in list.\n"
				,argv[1],port);
		return 1;
	}
	if(dp->prev != NULLDOM)
		dp->prev->next = dp->next;
	else
		Dlist = dp->next;
	if(dp->next != NULLDOM)
		dp->next->prev = dp->prev;

	if(Dserver == dp)
		Dserver = Dlist;
	if(Dlist == NULLDOM){
		tprintf("Last domain server deleted!\n");
		Dhaveaa = 1;
	}
	free((char *)dp);
	return 0;
}

static int
dolistds(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	register struct dserver *dp;
	struct socket sock;

	tprintf("Server socket         srtt    mdev    timeout\
 queries responses timeouts\n");
	for(dp = Dlist; dp != NULLDOM; dp = dp->next) {
		sock.address = dp->address;
		sock.port = dp->port;
		tprintf("%-22s%-8lu%-8lu%-8lu%-8lu%-8lu  %-8lu\n",
		 pinet(&sock),
		 dp->srtt * MSPTICK,
		 dp->mdev * MSPTICK,
		 dp->timeout * MSPTICK,
		 dp->queries,dp->responses,dp->missers);
	}
	return 0;
}
static int
doload(argc,argv,pp)
int argc;
char *argv[];
void *pp;
{
	FILE *dbase;
	char *arg[2];
	int type;
	int numbscan,numbload;
	long rrsize;
	struct rr *rrp, *rrl;
	struct rr_memory *memory;

	arg[0] = NULLCHAR;
	arg[2] = NULLCHAR;
	if(argc == 2) {
		if((dbase = fopen(argv[1],READ_TEXT)) == NULLFILE) {
			tprintf("Could not open domain file %s.\n",argv[1]);
			return 1;
		}
	} else
		if((dbase = fopen(Dfile,READ_TEXT)) == NULLFILE)
			return 1;
	memory = (struct rr_memory *)callocw(1,sizeof(struct rr_memory));
	rrsize = 0l;
	numbscan = 0;
	numbload = 0;
	db_SOA = NULLRR;
	if(Lorigin == NULLDLIST) {
		Lorigin = (struct dlist *)callocw(1,sizeof(struct dlist));
		Lorigin->name = "";	/* start with empty domain name */
		Db_initialize = 1;
		while((rrp = get_rr(dbase,memory)) != NULLRR) {
			type = rrp->type;
			numbscan++;
			numbload++;
			rrsize += (long)rrp->rdlength;
			if(Ddebug)
				printf("Adding to %s list - %s\n"
					,type2str(type),rrp->name);
			if(type == TYPE_SOA)
				db_SOA = rrp;	/* save for reference */
			else
				rrp->soarec = db_SOA;
			if(Rrlist[type] == NULLRR) {
				Rrlist[type] = rrp; /* initialize linked list */
				Rrlistl[type] = rrp;
			} else {
				Rrlistl[type]->next = rrp;
				Rrlistl[type] = rrp; /* append to linked list */
			}
		}
		/* Lorigin stayes as a list of loaded domain origin names.
		 * it is refered to in rrp->origins loaded in memory.
		 * having a common origin saves significantly in used
		 * memory oposed to giving every record a copy of that
		 * origin. Domain save will reconstruct $origin statements.
		 */
		printf("Number of records scanned = %d, loaded = %d\n"
			,numbscan,numbload);
		printf(" Memory record size = %d, extra = %ld\n"
			,sizeof(struct rr),rrsize);
		Db_initialize = 0;
		Dorigin = NULLCHAR;	/* not loading anymore */
		DBloaded = 1;		/* disable domain file access */
	} else {
		Db_initialize = 1;
		while((rrp = get_rr(dbase,memory)) != NULLRR) {
			numbscan++;
			type = rrp->type;
			if(type == TYPE_SOA)
				db_SOA = rrp;	/* save for reference */
			else
				rrp->soarec = db_SOA;
			fflush(stdout);
			if(add_rr(NULLFILE,rrp))
				free_rr(rrp);
			else
				numbload++;
		}
		printf("Number of additional records scanned = %d,loaded = %d\n"
			,numbscan,numbload);
		Db_initialize = 0;
		Dorigin = NULLCHAR;	/* not loading anymore */
		fclose(dbase);
		free((char *)memory);
		return 0;
	}
	fclose(dbase);
	free(memory->dorigin);
	free((char *)memory);
	if((rrp = Rrlist[TYPE_NS]) != NULLRR) {
		rrl = rrp;
		while(rrl != NULLRR) {
			arg[1] = rrl->rdata.name;
			if(Ddebug)
				printf("Name Server - %s\n",arg[1]);
			doadds(2,arg,pp);
			rrl = rrl->next;
		}
	}
	return 0;
}

static int
donslookup(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	int i, type;
	int len;
	int s;
	int32 address;
	char *buf;
	struct sockaddr_in nslook;
	struct dhdr *dhdr;
	struct mbuf *bp;
	int16 flags;

	if((address = resolve(argv[1])) == 0){
		tprintf("Station %s to query is unknown.\n",argv[1]);
		return 1;
	}

	type = 0;
	if(isdigit(*argv[2])){
		type = atoi(argv[2]);
		if(type > 255)
			type = 0;
	} else {
		for(i=0;i<NRLIST;i++){
			if(strcmp(argv[2],Dtypes[i]) == 0){
				type = i;
				break;
			}
		}
	}
	if(type == 0){
		tprintf("Unknown record type - %s.\n",argv[2]);
		return 1;
	}
	if(Dtrace)
		tprintf("Questioning %s for record type %s\n",argv[1],argv[2]);

	flags = DOM_DORECURSE | DOM_CANRECURSE;	/* Recursion desired */
	dhdr = bld_dhdr(QUERY,QUERY,flags,NO_ERROR,argv[3],CLASS_IN,type);
	buf = mallocw(512);
	len = res_mkbuf(dhdr,buf,512);
	if(Dtrace)
		dumpdhdr(dhdr);
	free_dhdr(dhdr);
	s = socket(AF_INET,SOCK_DGRAM,0);
	nslook.sin_family = AF_INET;
	nslook.sin_port = IPPORT_DOMAIN;
	nslook.sin_addr.s_addr = address;
	sendto(s,buf,len,0,(char *)&nslook,sizeof(nslook));
	free(buf);
	alarm(30L * 1000 / MSPTICK);
	/* Wait for something to happen */
	len = recv_mbuf(s,&bp,0,NULLCHAR,0);
	alarm(0L);
	close_s(s);
	if(len > 0){
		dhdr = (struct dhdr *)mallocw(sizeof(struct dhdr));
		ntohdomain(dhdr,&bp);
		dumpdhdr(dhdr);
		free_dhdr(dhdr);
	} else
		tprintf("No answer from server\n");
	return 0;
}
static int
dosave(argc,argv,pp)
int argc;
char *argv[];
void *pp;
{
	FILE *dbase;
	char *p;
	int i,j;
	struct rr *rrp;
	struct dlist *llist;

	if(argc == 2) {
		if((dbase = fopen(argv[1],WRITE_TEXT)) == NULLFILE) {
			tprintf("Could not open domain file %s.\n",argv[1]);
			return 1;
		}
	} else
		if((dbase = fopen("/domain.new",WRITE_TEXT)) == NULLFILE)
			return 1;
	if((llist = Lorigin) != NULLDLIST) {
	    while(llist != NULLDLIST) {
		fprintf(dbase,"$origin\t%s\n",llist->name);
		for(i = 0, j = Dstypes[i]; (j = Dstypes[i]) != 0; i++)  {
			if((rrp = Rrlist[j]) != NULLRR) {
				while(rrp != NULLRR) {
					if((*llist->name == '\0')
						&& (rrp->origin == NULLCHAR)) {
						if(rrp->type != TYPE_A)
						    put_rr(dbase,rrp,0);
						else 
						   if( rrp->rdata.addr != (int32)0)
							put_rr(dbase,rrp,0);
					} else
					if(rrp->origin == llist->name) {
						if(rrp->type != TYPE_A)
						    put_rr(dbase,rrp,0);
						else 
						   if( rrp->rdata.addr != (int32)0)
							put_rr(dbase,rrp,0);
					}
					rrp = rrp->next;
				}
			}
		}
		llist = llist->next;
	   }
	} else
	for(i = 0, j = Dstypes[i]; (j = Dstypes[i]) != 0; i++)  {
		p = NULLCHAR;
		if((rrp = Rrlist[j]) != NULLRR) {
			while(rrp != NULLRR) {
				if(rrp->name[strlen(rrp->name) - 1] != '.') {
				    if(p == NULLCHAR && rrp->origin != NULLCHAR)
					fprintf(dbase,"$origin %s\n",rrp->origin);
				    if(p != NULLCHAR && rrp->origin == NULLCHAR)
					fprintf(dbase,"$origin \n");
				    if(p != NULLCHAR && rrp->origin != NULLCHAR) 
					if(stricmp(p,rrp->origin))
						fprintf(dbase,"$origin %s\n"
							,rrp->origin);
				    p = rrp->origin;
				}
				if(rrp->type != TYPE_A)
				    put_rr(dbase,rrp,0);
				else 
				   if( rrp->rdata.addr != (int32)0)
					put_rr(dbase,rrp,0);
				rrp = rrp->next;
			}
		}
	}
	fclose(dbase);
	return 0;
}
/* test a rr against a given name and type. The name can be:
 1 a fully qualified name    (EG: pa0gri.ampr.org.)
    or (if not fully qualified)
 2 a name without a dot and no origin and suffix
 3 a name without a dot and an origin and no suffix
 4 a name without a dot and an origin and a suffix
 *
 * return 0 if not comparable, case value if it is good
 */
static int
dfind_cmp(rrp,name,nlen,dlen)
struct rr *rrp;
char *name;
int nlen;
int dlen;
{
	int olen, rlen;

	rlen = strlen(rrp->name);

	if(rrp->origin != NULLCHAR) {
		olen = strlen(rrp->origin);
		if(rrp->origin[olen - 1] == '.')
			--olen;
	} else
		olen = 0;

/* first test: is the name equal to the fully qualified rrp name?
	   (minus traling .) */
	if(rrp->name[rlen - 1] == '.') {
		--rlen;
		if(rlen == nlen
		 && strnicmp(name,rrp->name,rlen) == 0)
			return 1;
	}
/* second test: is the name equal to the rrp name */
	if(Dsuffix == NULLCHAR && rrp->origin == NULLCHAR) {
		if(nlen == rlen
		 && strnicmp(name,rrp->name,rlen) == 0)
			return 2;
		if(nlen+9 == rlen /* special case for .ampr.org. */
		 && strnicmp(name,rrp->name,nlen) == 0
		 && strnicmp(&rrp->name[nlen+1],"ampr.org",8) == 0
		 && rrp->name[nlen] == '.')
			return 3;
	} else
	if(Dsuffix == NULLCHAR && rrp->origin != NULLCHAR) {
/* third test: name equal to rrp name (excluding origin) */
		if(nlen == rlen
		 && strnicmp(name,rrp->name,rlen) == 0)
		 	return 4;
		else 
/* fourth test: name equal rrp name + origin */
			if((rlen + olen + 1) == nlen
			 && strnicmp(name,rrp->name,rlen) == 0
			 && name[rlen] == '.'
			 && strnicmp(&name[rlen + 1],rrp->origin,olen) == 0)
				return 5;
	} else 
	if(Dsuffix != NULLCHAR && rrp->origin != NULLCHAR) {
/* fifth test: name equal to rrp name and origin equal to domain suffix */
		if(rlen == nlen && dlen == olen
		 && strnicmp(name,rrp->name,rlen) == 0
		 && strnicmp(Dsuffix,rrp->origin,dlen) == 0)
			return 6;
		else
/* sixth test: name equal to rrp name + origin) */
			if((rlen + olen + 1) == nlen
			 && strnicmp(name,rrp->name,rlen) == 0
			 && name[rlen] == '.'
			 && strnicmp(&name[rlen + 1],rrp->origin,olen) == 0)
				return 7;
	} else
	if(Dsuffix != NULLCHAR && rrp->origin == NULLCHAR) {
/* seventh test: name equal to rrp name and no origin (ignore domain suffix) */
		if(rlen == nlen 
		 && strnicmp(name,rrp->name,rlen) == 0)
			return 8;
		else
/* eight test: name plus suffix equal to rrp name ) */
			if((nlen + dlen + 1) == rlen
			 && strnicmp(name,rrp->name,nlen) == 0
			 && rrp->name[nlen] == '.'
			 && strnicmp(&rrp->name[nlen + 1],Dsuffix,dlen) == 0)
				return 9;
	}
	return 0;
}
/* Search local domain file for resource record of specified name and type.
 * If records were found, the domain file pointer is left just after the last
 * one. If not, the file is rewound.
 */
static struct rr *
dfindf(dbase,name,type,memory)
FILE *dbase;
char *name;
int type;
struct rr_memory *memory;
{
	struct rr *rrp;
	int rval, nlen, dlen;

	if(DBloaded)
		return NULLRR;

	nlen = strlen(name);
	if(name[nlen - 1] == '.') /* forget trailing . for tests. */
		--nlen;
	if(Dsuffix != NULLCHAR)
		dlen = strlen(Dsuffix) - 1; /* Do not account for trailing .*/
	else
		dlen = 0;
	/* Search file */
	while((rrp = get_rr(dbase,memory)) != NULLRR){
		if(rrp->class == CLASS_IN)
			if(type == rrp->type || type == TYPE_ANY)
				if((rval = dfind_cmp(rrp,name,nlen,dlen)) != 0)
					break;
		free_rr(rrp);
		pwait(NULL);	/* Give up CPU for a while, this takes time */
	}
	if(rrp == NULLRR) 
		rewind(dbase);
	if(Ddebug)
		if(rrp == NULLRR) 
			printf("dfindf(): %s not found.\n",name);
		else
			printf("dfindf(): %s found. rval = %d\n",name,rval);
	return rrp;
}
#ifdef forgetme
		/* If the first character is a '*' it is replaced by the
		   first part of the sought domain. */
/* forget about this till other areas know what to do with multiple rr's.
		if(*rrp->name == '*'){
			if(rrp->name[rlen-1] == '.')
				p = mallocw(rlen);
			else
				p = mallocw(rlen + olen);
			strcpy(p,rrp->name+2);
			if(rrp->name[rlen-1] != '.' && rrp->origin != NULLCHAR)
				strcat(p,rrp->origin);
			rrp->origin = p;
			olen = strlen(rrp->origin);
			if(rrp->origin[olen-1] == '.')
				--olen;
			free(rrp->name);
			rrp->name = (char *) callocw(1,nlen + 1);
			if(nlen > olen
			 && strnicmp(name+nlen-olen,rrp->origin, olen) == 0)
				strncpy(rrp->name,name,nlen-olen-1);
			else {
				free_rr(rrp);
				continue;
			}
			rlen = strlen(rrp->name);
		} 
end of skip code */
#endif
static struct rr *
dfind(dbase,name,type,memory)
FILE *dbase;
char *name;
int type;
struct rr_memory *memory;
{
	struct rr *rrp, *rrl;
	int rval, nlen, dlen, i, rtype;
	struct rr *rrsave = NULLRR;

	if(Ddebug)
		printf("dfind(): search name = %s, type = %s\n",name,type2str(type));
	if(type >= Ndtypes)
		if(type == TYPE_ANY) {
			for(i = 0 ; (rtype = Dstypes[i]) != 0; ) {
				rrp = dfind(dbase,name,rtype,memory);
				if(rrp != NULLRR) {
					if(rrsave == NULLRR){
						rrsave = rrp;
						rrl = rrp;
					} else {
						rrl->next = rrp;
						rrl = rrl->next;
					}
				} else {
					i++;
					memory->rrp = NULLRR;
				}
			}
			if(rrsave != NULLRR)
				return rrsave;
		} else {
			goto filetest; /* to big a type for memory */
		}
	if((rrp = Rrlist[type]) != NULLRR) {
		if(Ddebug)
			printf("dfind(): searching memory data base\n");
		if(memory->rrp != NULLRR){
			rrp = memory->rrp; /* go off where we left before ! */
			rrp = rrp->next; /* but take the next */
		}
		nlen = strlen(name);
		if(name[nlen - 1] == '.') /* forget trailing . for tests. */
			--nlen;
		if(Dsuffix != NULLCHAR)
			dlen = strlen(Dsuffix) - 1;
		else
			dlen = 0;
	/* Search core data beast */
		while(rrp != NULLRR){
			if(rrp->class == CLASS_IN)
				if((rval = dfind_cmp(rrp,name,nlen,dlen)) != 0)
						break;
			rrp = rrp->next;
		}
		if(rrp != NULLRR){
			if(Ddebug)
				printf("dfind():record found in memory %d \n",rval);
			rrl = copy_rr(rrp);
			memory->rrp = rrp;
			return rrl;
		}
		memory->rrp = NULLRR; /* end of list encountered */
	} /* not found in memory, resort to file */
	if(Ddebug)
		printf("dfind():record not found in memory \n");
filetest:
	if(dbase != NULLFILE)
		rrp = dfindf(dbase,name,type,memory);
	return rrp;
}
static struct rr *
get_rr(fp,memory)
FILE *fp;
struct rr_memory *memory;
{
	char *line,*contline,*strtok();
	struct rr *rrp;
	struct dlist *lorigin,*ltemp;
	char *name,*ttl,*class,*type,*data;
	int i;
	int32 value, atol();

	if(fp == NULLFILE)
		return NULLRR;
	line = mallocw(256);
	/* Search file */
	while(fgets(line,256,fp),!feof(fp)){
		if(line[0] == '#') {
			continue;
		}
		if(line[0] == ';') {
			continue;
		}
		if(line[0] == '$') {
			data = strtok(line,delim);
			if(strnicmp(data,"$origin",7) == 0) {
				data = strtok(NULLCHAR,delim);
				if(Db_initialize) {
				   if(data != NULLCHAR) {
					ltemp = Lorigin;
					while(ltemp != NULLDLIST) {
						lorigin = ltemp;
						if(strcmp(ltemp->name,data) == 0)
							break;
						ltemp = ltemp->next;
					}
					if(ltemp == NULLDLIST) {
						ltemp = (struct dlist *)callocw(1,
						   sizeof(struct dlist));
						ltemp->name = strdup(data);
						lorigin->next = ltemp;
						Dorigin = ltemp->name;
					} else
						Dorigin = ltemp->name;
				   } else
					Dorigin = NULLCHAR; /* empty origin */
				}
				if(memory->dorigin != NULLCHAR)
					free(memory->dorigin);
				if(data != NULLCHAR)
					memory->dorigin = strdup(data);
				else
					memory->dorigin = NULLCHAR;
			}
			continue;
		}
		data = line;	/* check for empty line */
		while(*data)
			if(!isspace(*data++)) {
				break;
			}
		/*
		 * now it is nor a # , ; or $ line 
		 */
		if(*data)
			break;	/* there is something on it.. */
	}

	if(feof(fp)){
		free(line);
		return NULLRR;
	}
	rrp = (struct rr *)callocw(1,sizeof(struct rr));
	if(isspace(line[0])) {	/* in case of previous name usage*/
		name = "";
		ttl = strtok(line,delim);
	} else {
		name = strtok(line,delim);
		if(!strcmp(name,"@") && (memory->dorigin != NULLCHAR ||
					 Dsuffix != NULLCHAR)){
			if(memory->dorigin != NULLCHAR)
				name = memory->dorigin;
			else
				name = Dsuffix;
		}
		ttl = strtok(NULLCHAR,delim);
	}
	if(ttl == NULLCHAR) 	/* If a line only contains a name */
		goto badline;
	
	if(strlen(name) == 0)
		name = memory->Dname;
	else {
		if(strlen(name) > 127) {
			tprintf("Domain: reading name exceding 127 characters!\n");
				strncpy(memory->Dname,name,127);
		} else
			strcpy(memory->Dname,name);
	}

	i = strlen(name);
	if(Dorigin != NULLCHAR)
		rrp->origin = Dorigin; /* glue to origin list */
	else
		rrp->origin = memory->dorigin;
	if(name[i-1] == '.')	/* if a fully qualified name comes along */
		rrp->origin = NULLCHAR;	/* then it does not belong to an origin */
	rrp->name = strdup(name);

	if(!isdigit(ttl[0])){
		/* Optional ttl field is missing; slide field over */
		class = ttl;
		ttl = NULLCHAR;
		rrp->ttl = memory->Dttl;
	} else {
		memory->Dttl = atol(ttl);
		rrp->ttl = memory->Dttl;
		class = strtok(NULLCHAR,delim);
	}

	if(class == NULLCHAR) 	/* If a line only contains a name and ttl */
		goto badline;

	if(strcmp(class,"IN") != 0){
		/* Optional class field is missing; slide field over */
		type = class;
		class = NULLCHAR;
		rrp->class = memory->Dclass;
	} else {
		memory->Dclass = CLASS_IN;
		rrp->class = memory->Dclass;
		type = strtok(NULLCHAR,delim);
	}

	if(type == NULLCHAR) {	/* If a class field is missing */
badline:	tprintf("get_rr() errornous line in %s file.\n-%s-\n",Dfile,line);
		free(line);
		free_rr(rrp);
		return NULLRR;
	}

	for(i=0;i<NRLIST;i++){
		if(strcmp(type,Dtypes[i]) == 0){
			rrp->type = i;
			break;
		}
	}

	data = strtok(NULLCHAR,delim);

	if(data == NULLCHAR){
		free(line);
		return rrp;
	}
	switch(rrp->type){
	case TYPE_MD:		/* Unsupported yet */
	case TYPE_MF:		/* Unsupported yet */
	case TYPE_NULL:		/* Unsupported yet */
		rrp->rdlength = strlen(data) + 1;
		rrp->rdata.name = strdup(data);
		break;
	case TYPE_WKS:		/* Unsupported yet */
/*
		rrp->wks = (struct wks *)callocw(1,sizeof(struct wks));
 */
		rrp->rdlength = strlen(data) + 1;
		rrp->rdata.name = strdup(data);
		break;
	case TYPE_CNAME:
	case TYPE_MB:
	case TYPE_MG:
	case TYPE_MR:
	case TYPE_NS:
	case TYPE_PTR:
	case TYPE_TXT:
		rrp->rdlength = strlen(data) + 1;
		rrp->rdata.name = strdup(data);
		break;
	case TYPE_A:
		rrp->rdlength = 4;
		rrp->rdata.addr = aton(data);
		break;
	case TYPE_HINFO:
		rrp->rdlength = 2 + strlen(data);
		rrp->rdata.hinfo.cpu = strdup(data);
		if((data = strtok(NULLCHAR,delim)) != NULLCHAR){
			rrp->rdlength += strlen(data);
			rrp->rdata.hinfo.os = strdup(data);
		}
		break;
	case TYPE_MINFO:
		rrp->rdlength = 2 + strlen(data);
		rrp->rdata.minfo.rmailbx = strdup(data);
		if((data = strtok(NULLCHAR,delim)) != NULLCHAR){
			rrp->rdlength += strlen(data);
			rrp->rdata.minfo.emailbx = strdup(data);
		}
		break;
	case TYPE_MX:
		rrp->rdata.mx.pref = (int16) atoi(data);
		rrp->rdlength = 2;

		/* Get domain name of exchanger */
		if((data = strtok(NULLCHAR,delim)) != NULLCHAR){
			rrp->rdlength += strlen(data) + 1;
			rrp->rdata.mx.exch = strdup(data);
		}
		break;
	case TYPE_SOA:
		rrp->rdata.soa = (struct soa *)callocw(1,sizeof(struct soa));
		/* Get domain name of master name server */
		rrp->rdlength = strlen(data) + 1;
		rrp->rdata.soa->mname = strdup(data);

		/* Get domain name of irresponsible person */
		if((data = strtok(NULLCHAR,delim)) != NULLCHAR){
			rrp->rdata.soa->rname = strdup(data);
			rrp->rdlength += strlen(data) + 1;
		}
		data = strtok(NULLCHAR,delim);
		if(*data == '(') {	/* multiple line data */
			contline = mallocw(256);
			/* Search file */
			fgets(contline,256,fp);
			if(feof(fp))
				break;
			data = strtok(contline,delim);
			for(i=0; i<5;i++) {
				if(*data == ';' || data == NULLCHAR) {
					fgets(contline,256,fp);
					if(feof(fp)){
							break;
					}
					data = strtok(contline,delim);
				}
				if(*data == ')')
					break;
				if(isdigit(*data)) {
					value = atol(data);
					switch (i) {
					case 0:
					rrp->rdata.soa->serial = value;
						break;
					case 1:
					rrp->rdata.soa->refresh = value;
						break;
					case 2:
					rrp->rdata.soa->retry = value;
						break;
					case 3:
					rrp->rdata.soa->expire = value;
						break;
					case 4:
					rrp->rdata.soa->minimum = value;
						break;
					}
				}
			data = strtok(NULLCHAR,delim);
			}
			free(contline);
		} else {
			rrp->rdata.soa->serial = atol(data);
			data = strtok(NULLCHAR,delim);
			rrp->rdata.soa->refresh = atol(data);
			data = strtok(NULLCHAR,delim);
			rrp->rdata.soa->retry = atol(data);
			data = strtok(NULLCHAR,delim);
			rrp->rdata.soa->expire = atol(data);
			data = strtok(NULLCHAR,delim);
			rrp->rdata.soa->minimum = atol(data);
		}
		rrp->rdlength += 20;
		break;
	}
	free(line);
	return rrp;
}
/* Search for address record in local database, looking first for PTR
 * and CNAME records. Return values:
 *  0xffffffff	Not found (domain name may exist, but we don't know yet)
 *  0		Domain name definitely doesn't exist (we have a null record)
 */
int32
dresolve(name)
char *name;
{
	register struct rr *rrp;
	int32 result;

	if(cache.name != NULLCHAR && strcmp(cache.name,name) == 0)
		return cache.address;

	rrp = resolverec(name,TYPE_A);
	if(rrp == NULLRR){
		result = 0xffffffff;		/* No record in database */
	} else {
		if(rrp->rdlength == 0)
			result = 0;		/* Negative response record */
		else
			result = rrp->rdata.addr;	/* Normal return */
		free(cache.name);
		cache.name = strdup(name);
		cache.address = result;
		free_rr(rrp);
	}
	return result;
}
/* Resolve an arbitrary record */
static struct rr *
resolverec(name,type)
char *name;
int type;
{
	register struct rr *rrp;
	char *pname = NULLCHAR;
	char *cname = NULLCHAR;
	FILE *dbase;
	struct rr_memory *memory;

	if(name == NULLCHAR)
		return NULLRR;
	if(DBloaded)
		dbase = NULLFILE;
	else
		if((dbase = fopen(Dfile,READ_TEXT)) == NULLFILE)
			return NULLRR;

	memory = (struct rr_memory *)callocw(1,sizeof(struct rr_memory));
	memory->Dname[0] = '\0';	/* initialize global area */
	memory->Dttl = 3600;
	memory->Dclass = 1;

	/* This code can handle a few weird cases. It works when there's
	 * a PTR to a CNAME to the sought record, as well as when there's a
	 * CNAME to a PTR to the record. But it allows only one of each kind
	 * of indirection to prevent infinite loops.
	 */
	while((rrp = dfind(dbase,name,type,memory)) == NULLRR){
		/* No matching record was found, let's see if it's an alias */
		if(cname == NULLCHAR 
		  && (rrp = dfind(dbase,name,TYPE_CNAME,memory)) != NULLRR){
			if((cname = strdup(rrp->rdata.name)) == NULLCHAR)
				break;
			name = cname;
			rewind(dbase);
			memory->Dname[0] = '\0'; /* initialize global area */
			memory->Dttl = 3600;
			memory->Dclass = 1;
			memory->rrp = NULLRR;
			free(memory->dorigin);
			memory->dorigin = NULLCHAR;
			free_rr(rrp);
			continue;	/* Try again */
		}
		/* Lacking that, try a pointer entry... */
		if(pname == NULLCHAR 
		  && (rrp = dfind(dbase,name,TYPE_PTR,memory)) != NULLRR){
			if((pname = strdup(rrp->rdata.name)) == NULLCHAR)
				break;
			name = pname;
			rewind(dbase);
			memory->Dname[0] = '\0';   /* initialize global area */
			memory->Dttl = 3600;
			memory->Dclass = 1;
			memory->rrp = NULLRR;
			free(memory->dorigin);
			memory->dorigin = NULLCHAR;
			free_rr(rrp);
			continue;
		}
		/* Nope, nothing. Give up */
		break;
	}
	fclose(dbase);
	free(pname);
	free(cname);	
	free(memory->dorigin);
	free((char *)memory);
	return rrp;
}

/* Main entry point for domain name -> address resolution. Returns 0 if
 * name is definitely not valid.
 */
int32
resolve(name)
char *name;
{
	char *buf, *sname;
	int i, len, auth = 0, count = 0;
	int rval, recurse = 0;
	int16 flags;
	int32 addr;
	struct dserver *dp;
	struct dhdr *dhdr;
	struct mbuf *bp;
	struct rr *rrp;

	if(*name == '[')
		return aton(name + 1);

	if(isaddr(name))
		return aton(name);

	sname = strdup(name);
	dp = Dserver;
	while((addr = dresolve(sname)) == 0xffffffff){
		if(dp == NULLDOM){
			addr = 0;	/* Unknown, and no servers */
			break;
		}
		if(ismyaddr(dp->address) != NULLIF) { /* Don't ask ourselves */
			if((dp = dp->next) == NULLDOM)
				dp = Dlist;
			if(ismyaddr(dp->address) != NULLIF) { /* If I am the only one */
				addr = 0;
				goto quit;
			}
		}
		if(!Mprunning){    /* test Multy tasker running yet */
			addr = 0;  /* No, cant resolve. (will hang if tried) */
			break;
		}
		if(auth == 0)
			if(haveaa(name)){ /* Quit if we are an authority */
				addr = 0;
	/*			break; */
			}
			else
				auth = -1;
		/* Not in file, send query */
		if(Dtrace){
			tprintf("resolve: querying server %s for %s\n",
			 inet_ntoa(dp->address),name);
		}
		flags = DOM_DORECURSE | DOM_CANRECURSE;	/* Recursion desired */
		dhdr = bld_dhdr(QUERY,QUERY,flags,NO_ERROR,name,CLASS_IN,TYPE_A);
		buf = mallocw(512);
		len = res_mkbuf(dhdr,buf,512);
		if(Dtrace)
			dumpdhdr(dhdr);
		free_dhdr(dhdr); /* actualy we need to keep the question for
				    later reference when an answer comes in */
		dp->queries++;
		rval = sendquery(dp->address,buf,len,&bp,dp->timeout);
		if(rval > 0){
			dhdr = (struct dhdr *)mallocw(sizeof(struct dhdr));
			ntohdomain(dhdr,&bp);
			dp->responses++;
			if(Dtrace)
				dumpdhdr(dhdr);
			proc_answer(dhdr,dp);
			if(dhdr->rcode != NO_ERROR) {
				addr = 0;
				goto quit;
			} else {
				for(i=0; i<dhdr->ancount; i++){
					rrp = dhdr->ans[i];
					if(strnicmp(rrp->name,sname,
							strlen(sname)) != 0)
						continue;
					/* Got one */
					switch(rrp->type) {
					case TYPE_CNAME:
						/* Change the query name to the
						 * cname and go back again
						 */
						if(++recurse == MAXCNAME) {
							/* too many recursions */
							addr = 0;
							break;
						}
						free(sname);
						sname=strdup(rrp->rdata.name);
						/* rescan the responces */
						i = -1;
						continue;
					case TYPE_A:
						if(rrp->rdlength == 0)
							addr = 0L;
						else
							addr = rrp->rdata.addr;
						break;
					}
					break;
				}
			}
			free_dhdr(dhdr);
			free(dhdr);
			break;
		}
		if(errno == EABORT) {
			addr = 0L;
			goto quit;
		}
		/* Timeout; back off this one and try another server */
		dp->missers++;
		dp->timeout += (5L * 1000 / MSPTICK);
		if((dp = dp->next) == NULLDOM)
			dp = Dlist;
		tprintf("Domain query to %s timed out.\n",
			inet_ntoa(dp->address));
		count++;
		if(count > Dretries){	/* no responses , no one there! */
			addr = 0;
			goto quit;
		}
		break;
	}
quit:	free(sname);
	return addr;
}

/* Main entry point for domain name -> arbitrary record resolution. Returns a
 * null pointer if name is definitely not valid.
 */
struct rr *
rresolve(name,type)
char *name;
int type;
{
	char *buf;
	int len, rval, auth = 0;
	int lcount;
	int16 flags;
	struct rr *rrp, *rr, *rrtmp;
	struct dserver *dp;
	struct dhdr *dhdr;
	struct mbuf *bp;

	dp = Dserver;
	lcount = 0;
	while((rrp = resolverec(name,type)) == NULLRR){
		if(dp == NULLDOM)
			break;		/* Unknown, and no servers */
		if(ismyaddr(dp->address) != NULLIF) { /* Don't ask ourselves */
			if((dp = dp->next) == NULLDOM)
				dp = Dlist;
			if(ismyaddr(dp->address) != NULLIF) /* If I am the only one */
				break;
		}
		if(auth == 0)
			if(haveaa(name)) /* Quit if we are an authority */
				break;
			else
				auth = -1;
		/* Not in file, send query */
		flags = DOM_DORECURSE;	/* Recursion desired */
		dhdr = bld_dhdr(QUERY,QUERY,flags,NO_ERROR,name,CLASS_IN,type);
		buf = mallocw(512);
		len = res_mkbuf(dhdr,buf,512);
		if(Dtrace)
			dumpdhdr(dhdr);
		free_dhdr(dhdr);   /* actualy we need to keep the question for
				      later reference when an answer comes in */
		dp->queries++;
		rval = sendquery(dp->address,buf,len,&bp,dp->timeout);
		if(rval > 0){
			dhdr = (struct dhdr *)mallocw(sizeof(struct dhdr));
			ntohdomain(dhdr,&bp);
			if(Dtrace)
				dumpdhdr(dhdr);
			proc_answer(dhdr,dp);
			free_dhdr(dhdr);
			free(dhdr);
			break;
		}
		if(errno == EABORT) {
			rrp = NULLRR;
			goto quit;
		}
		/* Timeout; back off this one and try another server */
		dp->timeout += 5L * 1000 / MSPTICK;
		dp->missers++;
		lcount++;
		if(Dtrace)
			tprintf("Domain query to %s timed out.\n",
				inet_ntoa(dp->address));
		if((dp = dp->next) == NULLDOM)
			dp = Dlist;
		if(lcount > Dretries){
			rrp = NULLRR;
			goto quit;
		}
		break;
	}
	rr = rrp; 	/* Remove any NULL records */
	while(rr != NULLRR && rr->rdlength == 0){
		rrp = rr->next;
		rr->next = NULLRR;
		free_rr(rr);
		rr = rrp;
	}
	while(rr != NULLRR && rr->next != NULLRR){
		if(rr->next->rdlength == 0){
			rrtmp = rr->next->next;
			rr->next->next = NULLRR;
			free_rr(rr->next);
			rr->next = rrtmp;
		    }
		    else
			rr = rr->next;
	}
quit:	return rrp;
}
static int
sendquery(address,buf,len,bp,timeout)
int32 address;
char *buf;
int len;
struct mbuf **bp;
int32 timeout;
{
	int resp;
	int s;
	struct sockaddr_in server;

	s = socket(AF_INET,SOCK_DGRAM,0);
	server.sin_family = AF_INET;
	server.sin_port = IPPORT_DOMAIN;
	server.sin_addr.s_addr = address;
	sendto(s,buf,len,0,(char *)&server,sizeof(server));
	free(buf);
	alarm(max(timeout,1));
	resp = recv_mbuf(s,bp,0,NULLCHAR,0);
	/* Wait for something to happen */
	alarm(0L);
	close_s(s);
	return resp;
}
static struct dhdr *
bld_dhdr(qr,op,flags,rcode,name,class,type)
int16 qr;
int16 op;
int16 flags;
int16 rcode;
char *name;
int class;
int type;
{
	struct dhdr *dhdr;

	dhdr = (struct dhdr *)callocw(1,sizeof(struct dhdr));
	dhdr->qr = qr;
	dhdr->opcode = op;
	if(flags & DOM_AUTHORITY)
		dhdr->aa = 1;
	if(flags & DOM_TRUNC)
		dhdr->tc = 1;
	if(flags & DOM_DORECURSE)
		dhdr->rd = 1;
	if(flags & DOM_CANRECURSE)
		dhdr->ra = 1;
	dhdr->rcode = rcode;
	dhdr->id = (int16)Clock;
	dhdr->qdcount = 1;
	dhdr->qlist = (struct quest **)mallocw(dhdr->qdcount *
				sizeof(struct quest *));
	dhdr->qlist[0] = (struct quest *)mallocw(sizeof(struct quest));
	dhdr->qlist[0]->qname = strdup(name);
	dhdr->qlist[0]->qtype = type;
	dhdr->qlist[0]->qclass = class;
	return dhdr;
}
static int
res_mkbuf(dhdr,buffer,buflen)
struct dhdr *dhdr;
char *buffer;	/* Area for query */
int16 buflen;	/* Length of same */
{
	char *cp;
	int16 parameter;
	int i, count;

	cp = buffer;
	cp = put16(cp,dhdr->id);	
	if(dhdr->qr)
		parameter = 0x8000;
	else
		parameter = 0;
	parameter |= (dhdr->opcode & 0x0f) << 11;
	if(dhdr->aa)
		parameter |= DOM_AUTHORITY;
	if(dhdr->tc)
		parameter |= DOM_TRUNC;
	if(dhdr->rd)
		parameter |= DOM_DORECURSE;
	if(dhdr->ra)
		parameter |= DOM_CANRECURSE;
	parameter |= (dhdr->rcode & 0x0f);
	cp = put16(cp,parameter);
	cp = put16(cp,dhdr->qdcount);
	cp = put16(cp,dhdr->ancount);
	cp = put16(cp,dhdr->nscount);
	cp = put16(cp,dhdr->arcount);
	if((count = dhdr->qdcount) > 0) {
		for(i = 0; i < count; i++) {
			cp = dn_compress(cp,dhdr->qlist[i]->qname);
			cp = put16(cp,dhdr->qlist[i]->qtype);
			cp = put16(cp,dhdr->qlist[i]->qclass);
		}
	}
	if((count = dhdr->ancount) > 0)
		for(i = 0; i < count; i++) {
			cp = htonrr(dhdr->ans[i],cp);
		}
	if((count = dhdr->nscount) > 0)
		for(i = 0; i < count; i++) {
			cp = htonrr(dhdr->ns[i],cp);
		}
	if((count = dhdr->arcount) > 0)
		for(i = 0; i < count; i++) {
			cp = htonrr(dhdr->add[i],cp);
		}
	return cp - buffer;
}
static char *
htonrr(rr,buffer)
struct rr *rr;
char *buffer;
{
	struct rr *rrp;
	char *cp, *p;
	int i, len;

	cp = buffer;
	for(rrp = rr; rrp != NULLRR; rrp = rrp->next) {
		i = strlen(rrp->name);
		if(rrp->name[i-1] != '.'
		   && rrp->origin != NULLCHAR) {
			p = mallocw(i + strlen(rrp->origin) + 2);
			sprintf(p,"%s.%s",rrp->name,rrp->origin);
			cp = dn_compress(cp,p);
			free(p);
		}
		else
			cp = dn_compress(cp,rrp->name);
		cp = put16(cp,rrp->type);
		cp = put16(cp,rrp->class);
		cp = put32(cp,3600L);		/* time to live */
		cp = put16(cp,rrp->rdlength);
		switch(rrp->type) {
		case TYPE_A:
			cp = put32(cp,rrp->rdata.addr);
			break;
		case TYPE_SOA:
			cp = dn_compress(cp,rrp->rdata.soa->mname);
			cp = dn_compress(cp,rrp->rdata.soa->rname);
			cp = put32(cp,rrp->rdata.soa->serial); 
			cp = put32(cp,rrp->rdata.soa->refresh); 
			cp = put32(cp,rrp->rdata.soa->retry); 
			cp = put32(cp,rrp->rdata.soa->expire); 
			cp = put32(cp,rrp->rdata.soa->minimum); 
			break;
		case TYPE_HINFO:
			*cp++ = len = strlen(rrp->rdata.hinfo.cpu); 
			strncpy(cp,rrp->rdata.hinfo.cpu,len); 
			cp += len;
			*cp++ = len = strlen(rrp->rdata.hinfo.os);
			strncpy(cp,rrp->rdata.hinfo.os,len); 
			cp += len;
			break;
		case TYPE_MX:
			cp = put16(cp,rrp->rdata.mx.pref); 
			cp = dn_compress(cp,rrp->rdata.mx.exch);
			break;
		case TYPE_CNAME:
		case TYPE_MB:
		case TYPE_MG:
		case TYPE_MR:
		case TYPE_NS:
		case TYPE_PTR:
			cp = dn_compress(cp,rrp->rdata.data);
			break;
		case TYPE_MINFO:	/* Unsupported type */
		cp = dn_compress(cp,rrp->rdata.minfo.rmailbx);
		cp = dn_compress(cp,rrp->rdata.minfo.emailbx);
		case TYPE_MD:		/* Unsupported type */
		case TYPE_MF:		/* Unsupported type */
		case TYPE_NULL:		/* Unsupported type */
		case TYPE_WKS:		/* Unsupported type */
			cp = dn_compress(cp,rrp->rdata.data);
			break;
		case TYPE_TXT:
		default:
			cp = put16(cp,rrp->rdlength);
			for(i=0 ; i < rrp->rdlength ; i++)
				*cp++ = rrp->rdata.data[i];
			break;
		}
	}
	return cp;
}
/* Process to receive all domain server related messages */
static void
drx(unused,u,p)
int unused;
void *u;
void *p;
{
	struct sockaddr_in sock,from;
	int fromlen;
	struct mbuf *bp;
	struct dhdr *dhdr;
	struct dserver *dp;
	int foo;

	Dsocket = socket(AF_INET,SOCK_DGRAM,0);
	sock.sin_family = AF_INET;
	sock.sin_addr.s_addr = INADDR_ANY;
	sock.sin_port = IPPORT_DOMAIN;
	if(bind(Dsocket,(char *)&sock,sizeof(sock)) == -1) {
		tprintf("Domain: can't bind (is ip address set?)\n");
		Drx = NULLPROC;
		return;
	}
	for(;;){
		fromlen = sizeof(from);
		foo = recv_mbuf(Dsocket,&bp,0,(char *)&from,&fromlen);
		if(Dtrace)
			printf("drx(): %u bytes from %s\n",foo,
			 psocket((struct sockaddr *)&from));

		if(foo == 0)
			continue;
		dhdr = mallocw(sizeof(struct dhdr));
		ntohdomain(dhdr,&bp);
		if(Dtrace)
			dumpdhdr(dhdr);
		if(dhdr->qr != RESPONSE) {
			if((dp = dslookup(Ip_addr)) == NULLDOM) {
				if(Dtrace)
				    printf("drx(): question for unknown server\n");
				free_dhdr(dhdr);
				continue;
			} else {
				if(ismyaddr(from.sin_addr.s_addr) != NULLIF &&
					    Dloopback == 0) {
					printf("drx() question to myself ignored\n");
					free_dhdr(dhdr);
					continue;
				} else {
					dp->queries++;
					dp=(struct dserver *)callocw(1,sizeof
						(struct dserver));
					dp->address = from.sin_addr.s_addr;
					dp->srtt = (Dtimeout * 1000L) / MSPTICK;
					dp->timeout = dp->srtt * 2;
					dp->port = from.sin_port;
					if(dhdr->opcode == ZONEINIT)
					    newproc("Domain zoneinit",1024,proc_bootp
						,0,(void *)dp,(void *)dhdr);
					else
					    newproc("Domain query",1024,proc_query
						,0,(void *)dp,(void *)dhdr);
				   }
			}
		}
	}
}

static void
proc_query(unused,d,b)
int unused;
void *d;
void *b;
{
	struct dserver *dp;
	struct dhdr *dhdr;
	int i,len,auth;
	char *buf;
	struct sockaddr_in server;
	struct rr *rrp, *rrans, *rrns, *rradd, *rrtmp;
	struct quest *qp;

	dp = (struct dserver *) d;
	dhdr = (struct dhdr *) b;

	rrans = rrns = rradd = NULLRR;
	for(i=0;i< dhdr->qdcount;i++){
		qp = dhdr->qlist[i];
		/* A server is testing here. ASSUMING INTERNET STYLE*/
		if(haveaa(qp->qname))
			auth = DOM_AUTHORITY | DOM_CANRECURSE;
		else
			auth = DOM_CANRECURSE;
		switch(qp->qtype) {
		case TYPE_A:
		/*	if((rrp->rdata.addr = resolve(qp->qname)) != 0) {*/
			if((rrp = rresolve(qp->qname,TYPE_A)) != NULLRR) {
			/* we found an entry, go tell him */
				dhdr->rcode = NO_ERROR;
				dhdr->aa = auth;
				dhdr->qr = RESPONSE;
				rrans = rrp;
			} else {
			/* we did not find an entry, go tell him */
				rrp = (struct rr *)callocw(1,sizeof(struct rr));
				rrp->name = strdup(qp->qname);
				rrp->type = qp->qtype;
				rrp->class = qp->qclass;
				rrp->ttl = 500L;
				rrp->rdata.addr = 0L;
				rrp->rdlength = 4;	/* size of addr data */
				dhdr->rcode = NAME_ERROR;
				dhdr->aa = auth;
				dhdr->qr = RESPONSE;
				rrans = rrp;
			}
			break;
		case TYPE_MB:
		case TYPE_MG:
		case TYPE_MR:
		case TYPE_MX:
		case TYPE_NS:
		case TYPE_SOA:
		case TYPE_PTR:
		case TYPE_TXT:
		case TYPE_CNAME:
		case TYPE_HINFO:
		case TYPE_ANY:
			if((rrp = rresolve(qp->qname,qp->qtype)) != NULLRR){
				dhdr->rcode = NO_ERROR;
				dhdr->aa = auth;
				dhdr->qr = RESPONSE;
			} else {
				dhdr->rcode = NAME_ERROR;
				dhdr->aa = auth;
				dhdr->qr = RESPONSE;
			}
			rrans = rrp;
			break;
		/* if I cannot give a reasonable answer , dont give it ... */
		case TYPE_MD:	/* unsupported */
		case TYPE_MF:	/* unsupported */
		case TYPE_WKS:	/* unsupported */
		case TYPE_NULL:	/* unsupported */
		case TYPE_MINFO:	/* unsupported */
		default:
			dhdr->rcode = NOT_IMPL;
			dhdr->aa = auth;
			dhdr->qr = RESPONSE;
		}
	}
	i = 0;
	rrtmp = rrans;
	while(rrtmp != NULLRR) {
		i++;
		rrtmp = rrtmp->next;
	}
	dhdr->ancount = i;
	if(i > 0) {
		dhdr->ans = (struct rr **)callocw(i,sizeof(struct rr *));
		rrtmp = rrans;
		i = 0;
		while(rrtmp != NULLRR) {
		dhdr->ans[i] = rrtmp;
			i++;
			rrans = rrtmp;
			rrtmp = rrtmp->next;
			rrans->next = NULLRR; /* break link */
		}
	}
	i = 0;
	rrtmp = rrns;
	while(rrtmp != NULLRR) {
		i++;
		rrtmp = rrtmp->next;
	}
	dhdr->nscount = i;
	if(i > 0) {
		dhdr->ns = (struct rr **)callocw(i,sizeof(struct rr *));
		rrtmp = rrns;
		i = 0;
		while(rrtmp != NULLRR) {
		dhdr->ns[i] = rrtmp;
			i++;
			rrns = rrtmp;
			rrtmp = rrtmp->next;
			rrns->next = NULLRR; /* break link */
		}
	}
	i = 0;
	rrtmp = rradd;
	while(rrtmp != NULLRR) {
		i++;
		rrtmp = rrtmp->next;
	}
	dhdr->arcount = i;
	if(i > 0) {
		dhdr->add = (struct rr **)callocw(i,sizeof(struct rr *));
		rrtmp = rradd;
		i = 0;
		while(rrtmp != NULLRR) {
		dhdr->add[i] = rrtmp;
			i++;
			rradd = rrtmp;
			rrtmp = rrtmp->next;
			rradd->next = NULLRR; /* break link */
		}
	}
	if(Dtrace)
		dumpdhdr(dhdr);
	buf = mallocw(512);
	len = res_mkbuf(dhdr,buf,512);
	free_dhdr(dhdr);
	server.sin_family = AF_INET;
	server.sin_port = dp->port;
	server.sin_addr.s_addr = dp->address;
	sendto(Dsocket,buf,len,0,(char *)&server,sizeof(server));
	free(buf);
	free((char *)dp);
}

static void
proc_bootp(unused,d,b)
int unused;
void *d;
void *b;
{
	struct dserver *dp;
	struct dhdr *dhdr;
	int i,count,len,auth;
	char *buf, *name;
	struct sockaddr_in server;
	struct rr *rrp;
	struct quest *qp;
	struct rr_memory *memory;
	FILE *fp;
	int answers = 5;

	dp = (struct dserver *) d;
	dhdr = (struct dhdr *) b;

	if(dhdr->qdcount != 1) {
		free_dhdr(dhdr);
		free((char *)dp);
		return;
	}
	qp = dhdr->qlist[0]; /* expecting only 1 question */
	if(qp->qtype ==  TYPE_ANY) {
		auth = DOM_AUTHORITY | DOM_CANRECURSE;
		name = strdup(qp->qname);
		i = strlen(name);
		if(name[i - 1] == '.')
			name[i - 1] = '\0'; /* undo effect of dn_compress */
		dhdr->rcode = NO_ERROR;
		dhdr->aa = auth;
		dhdr->qr = RESPONSE;
		dhdr->opcode = QUERY; /* change opcode for answers */
		if((fp = fopen(name,READ_TEXT)) == NULLFILE) {
			free_dhdr(dhdr);
			free((char *)dp);
			if(Dtrace) printf("bootp: unknown file %s\n",name);
			return;
		}
		memory = (struct rr_memory *)callocw(1,sizeof(struct rr_memory));
		dhdr->ans = (struct rr **)callocw(answers,sizeof(struct rr *));
		server.sin_family = AF_INET;
		server.sin_port = dp->port;
		server.sin_addr.s_addr = dp->address;
		count = 0;
		buf = mallocw(512);
		while((rrp = get_rr(fp,memory)) != NULLRR) {
			dhdr->ans[count++] = rrp;
			if(count == answers) {
				dhdr->ancount = answers;
				len = res_mkbuf(dhdr,buf,512);
				if(Dtrace)
					dumpdhdr(dhdr);
				sendto(Dsocket,buf,len,0,(char *)&server,sizeof(server));
				for(i = 0; i < answers; i++) {
					free_rr(dhdr->ans[i]);
					dhdr->ans[i] = NULLRR;
				}
				count = 0;
				alarm(max(dp->timeout,500));
				/* Wait for something to happen */
				pwait(&Dsocket);
				alarm(0L);
			}
		}

		dhdr->ancount = count;
		len = res_mkbuf(dhdr,buf,512);
		if(Dtrace)
			dumpdhdr(dhdr);
		free_dhdr(dhdr);
		sendto(Dsocket,buf,len,0,(char *)&server,sizeof(server));
		free(buf);
		free((char *)dp);
		free(name);
		fclose(fp);
	}
}

static void
proc_answer(dhdr,dp)
struct dhdr *dhdr;
struct dserver *dp;
{
	FILE *fp;
	int i;
	int32 rtt,abserr;
	long ttl = 500;	/* Default TTL for negative records without SOA */
	struct rr *rrp;
	struct quest *qp;

	/* Compute and update the round trip time */
	rtt = (int32) ((int16)Clock - dhdr->id);
	abserr = rtt > dp->srtt ? rtt - dp->srtt : dp->srtt - rtt;
	dp->srtt = ((AGAIN-1) * dp->srtt + rtt + (AGAIN/2)) >> LAGAIN;
	dp->mdev = ((DGAIN-1) * dp->mdev + abserr + (DGAIN/2)) >> LDGAIN;
	dp->timeout = 2 * dp->mdev + dp->srtt + 3;


	if(DBloaded == 1)
		fp = NULLFILE;	/* append to memory */
	else
		if((fp = fopen(Dfile,APPEND_TEXT)) == NULLFILE){
			tprintf("Can't append to %s\n",Dfile);
			free_dhdr(dhdr);
			return;
		}
	if(dhdr->aa && (dhdr->rcode == NAME_ERROR || dhdr->ancount == 0)){
		/* Add negative reply to file. This assumes that there was
		 * only one question, which is true for all questions we send.
		 */
		qp = dhdr->qlist[0];
		rrp = (struct rr *)callocw(1,sizeof(struct rr));
		rrp->name = strdup(qp->qname);
		rrp->type = qp->qtype;
		rrp->class = qp->qclass;
		rrp->ttl = ttl;
		rrp->rdlength = 0;	/* no data */
		if(rrp->type == TYPE_A){
			if(add_rr(fp,rrp))
				free_rr(rrp);	/* was already there */
		}
	}
	if(dhdr->rcode == NO_ERROR) {
		for(i=0;i< dhdr->ancount;i++){
			rrp = copy_rr(dhdr->ans[i]);
			if(rrp->type == TYPE_SOA)
				ttl = rrp->ttl;
			if(add_rr(fp,rrp))
				free_rr(rrp);
		}
		for(i=0;i< dhdr->nscount;i++){
			rrp = copy_rr(dhdr->ns[i]);
			if(rrp->type == TYPE_SOA)
				ttl = rrp->ttl;
			if(add_rr(fp,rrp))
				free_rr(rrp);
		}
		for(i=0;i< dhdr->arcount;i++){
			rrp = copy_rr(dhdr->add[i]);
			if(rrp->type == TYPE_SOA)
				ttl = rrp->ttl;
			if(add_rr(fp,rrp))
				free_rr(rrp);
		}
	}
	if(fp != NULLFILE)
		fclose(fp);
}

/* Print a resource record */
static void
put_rr(fp,rrp,addorg)
FILE *fp;
struct rr *rrp;
int addorg;
{
	if(fp == NULLFILE || rrp == NULLRR)
		return;

	fprintf(fp,"%s",rrp->name);
	if(addorg)
	    if(rrp->name[strlen(rrp->name)-1] != '.' && rrp->origin != NULLCHAR)
		fprintf(fp,".%s",rrp->origin);
	if(rrp->ttl)
		fprintf(fp,"\t%lu",rrp->ttl);
	if(rrp->class == CLASS_IN)
		fprintf(fp,"\tIN");
	else {
		if(rrp->ttl == 0)
			fprintf(fp,"\t%lu",rrp->ttl);
		fprintf(fp,"\t%u",rrp->class);
	}
	if(rrp->type < Ndtypes)
		fprintf(fp,"\t%s",type2str(rrp->type));
	else
		fprintf(fp,"\t%u",rrp->type);
	if(rrp->rdlength == 0){
		/* Null data portion, indicates nonexistent record */
		fprintf(fp,"\n");
		return;
	}
	switch(rrp->type){
	case TYPE_MINFO:	/* Unsupported type */
		fprintf(fp,"\t%s\t%s\n",
			rrp->rdata.minfo.rmailbx,
			rrp->rdata.minfo.emailbx);
		break;
	case TYPE_MD:		/* Unsupported type */
	case TYPE_MF:		/* Unsupported type */
	case TYPE_NULL:		/* Unsupported type */
	case TYPE_WKS:		/* Unsupported type */
		fprintf(fp,"\t%s\n",rrp->rdata.data);
		break;
	case TYPE_CNAME:
	case TYPE_MB:
	case TYPE_MG:
	case TYPE_MR:
	case TYPE_NS:
	case TYPE_PTR:
	case TYPE_TXT:
		/* These are all printable text strings */
		fprintf(fp,"\t%s\n",rrp->rdata.data);
		break;
	case TYPE_A:
		fprintf(fp,"\t%s\n",inet_ntoa(rrp->rdata.addr));
		break;
	case TYPE_MX:
		fprintf(fp,"\t%u\t%s\n",rrp->rdata.mx.pref,
		 rrp->rdata.mx.exch);
		break;
	case TYPE_SOA:
		fprintf(fp,"\t%s\t%s\t%lu\t%lu\t%lu\t%lu\t%lu\n",
		 rrp->rdata.soa->mname,rrp->rdata.soa->rname,
		 rrp->rdata.soa->serial,rrp->rdata.soa->refresh,
		 rrp->rdata.soa->retry,rrp->rdata.soa->expire,
		 rrp->rdata.soa->minimum);
		break;
	case TYPE_HINFO:
		fprintf(fp,"\t%s\t%s\n",
		 rrp->rdata.hinfo.cpu,
		 rrp->rdata.hinfo.os);
		break;
	default:
		fprintf(fp,"\n");
		break;
	}
}
/* Add a record to the database only if it doesn't already exist */
/* Return 1 if record is not added to memory list. This means the rr
   can be freed. If 0 is returned the rr is added to the memory list */
static int
add_rr(fp,rrp1)
FILE *fp;
struct rr *rrp1;
{
	int type, nlen;
	char *name;
	register struct rr *rrp, *rr;
	struct rr_memory *memory;

	analsuf(rrp1);
	type = rrp1->type;
	name = rrp1->name;
	nlen = strlen(name);
	if(Ddebug)
		printf("add_rr() add %s of type %s \n",name,type2str(type));
	memory = (struct rr_memory *)callocw(1,sizeof(struct rr_memory));
	memory->rrp = NULLRR;
	if(fp == NULLFILE) {
		name = strdup(rrp1->name);
		while((rrp = dfind(fp,name,type,memory)) != NULLRR){
			if(cmp_rr(rrp,rrp1) == 0){
				if(Ddebug)
					printf("add_rr(m): Duplicate record found in memory.\n");
				free_rr(rrp);
				free(memory->dorigin);
				free((char *)memory);
				free(name);
				return 1;
			}
			if(Ddebug)
				printf("add_rr(m): Record found in memory not same.\n");
			free_rr(rrp);
		}
/* Not found in list, go add it */
		free(memory->dorigin);
		free((char *)memory);
		free(name);
		/* see if we have this kind of records. */
		if((rr = Rrlist[type]) != NULLRR) {
			rr = Rrlistl[type];
			rr->next = rrp1;
		} else  /* No, initialize first */
			Rrlist[rrp1->type] = rrp1;
		/* now set last */
		Rrlistl[type] = rrp1;
		if(Ddebug)
			printf("add_rr(): Record added to memory.\n");
		return 0;
	} else {
		if(name[nlen - 1] != '.') {
			if(rrp1->origin != NULLCHAR){
				name = (char *)callocw(1,nlen+strlen(rrp1->origin)+2);
				sprintf(name,"%s.%s",rrp1->name,rrp1->origin);
			}
		} else
			name = strdup(rrp1->name);
		rewind(fp);
		while((rrp = dfind(fp,name,type,memory)) != NULLRR){
			if(cmp_rr(rrp,rrp1) == 0){
				if(Ddebug)
					printf("add_rr(f): Duplicate record found on file.\n");
				free_rr(rrp);
				free(memory->dorigin);
				free((char *)memory);
				free(name);
				return 1;
			}
			free_rr(rrp);
		}
		free(memory->dorigin);
		free((char *)memory);
		free(name);
		fseek(fp,0L,2);
		put_rr(fp,rrp1,1);
		if(Ddebug)
			printf("add_rr(): Record added to file.\n");
		return 1;
	}
}
static struct dserver *
dslookup(server)
int32 server;
{
	struct dserver *dp;

	for(dp = Dlist;dp != NULLDOM;dp = dp->next) {
		if(dp->address == server)
			break;
	}
	return dp;
}
/* Free a domain message */
static void
free_dhdr(dp)
struct dhdr *dp;
{
	int i;

	if(dp->qdcount != 0){
		for(i=0;i<dp->qdcount;i++)
			free_qu(dp->qlist[i]);
		free((char *)dp->qlist);
	}
	if(dp->ancount != 0){
		for(i=0;i<dp->ancount;i++)
			free_rr(dp->ans[i]);
		free((char *)dp->ans);
	}
	if(dp->nscount != 0){
		for(i=0;i<dp->nscount;i++)
			free_rr(dp->ns[i]);
		free((char *)dp->ns);
	}
	if(dp->arcount != 0){
		for(i=0;i<dp->arcount;i++)
			free_rr(dp->add[i]);
		free((char *)dp->add);
	}
	free((char *)dp);
}

/* Free a question record */
static void
free_qu(qp)
struct quest *qp;
{
	free(qp->qname);
	free((char *)qp);
}

/* Free a chain of resource records */
void
free_rr(rrp)
struct rr *rrp;
{
	struct rr *rr;

	rr = rrp;
	while(rr != NULLRR) {
		rrp = rr;
		free(rrp->name);
		if(rrp->rdlength != 0){
			switch(rrp->type){
			case TYPE_CNAME:
			case TYPE_MB:
			case TYPE_MG:
			case TYPE_MR:
			case TYPE_NS:
			case TYPE_PTR:
				free(rrp->rdata.name);
				break;
			case TYPE_A:
				break;	/* Nothing allocated in rdata section */
			case TYPE_HINFO:
				free(rrp->rdata.hinfo.cpu);
				free(rrp->rdata.hinfo.os);
				break;
			case TYPE_MX:
				free(rrp->rdata.mx.exch);
				break;
			case TYPE_SOA:
				free(rrp->rdata.soa->mname);
				free(rrp->rdata.soa->rname);
				free((char *)rrp->rdata.soa);
				break;
			case TYPE_TXT:
				free(rrp->rdata.data);
				break;
			case TYPE_MD:		/* Unsupported type */
			case TYPE_MF:		/* Unsupported type */
			case TYPE_NULL:		/* Unsupported type */
				free(rrp->rdata.data);
				break;
			case TYPE_WKS:		/* Unsupported type */
/*
				free((char *)rrp->wks);
 */
				break;
			case TYPE_MINFO:	/* Unsupported type */
				free(rrp->rdata.minfo.rmailbx);
				free(rrp->rdata.minfo.emailbx);
				break;
			}
		}
		rr = rrp->next;
		free((char *)rrp);
	}
}
/* Compare two resource records, returning 0 if equal, nonzero otherwise */
static int
cmp_rr(rr1,rr2)
register struct rr *rr1,*rr2;
{
	int i;

	if(rr1 == NULLRR || rr2 == NULLRR)
		return -1;
	if((i = strlen(rr1->name)) != strlen(rr2->name))
		return 1;
	if((i = strnicmp(rr1->name,rr2->name,i)) != 0)
		return i;
	if(rr1->type != rr2->type)
		return 2;
	if(rr1->class != rr2->class)
		return 3;
	if(rr1->rdlength == 0 && rr2->rdlength == 0)
		return 0;
	if(rr1->rdlength == 0 || rr2->rdlength == 0)
		return 4;
	/* Note: rdlengths are not compared because they vary depending
	 * on the representation (ASCII or encoded) this record was
	 * generated from.
	 */
	switch(rr1->type){
	case TYPE_A:
		i = rr1->rdata.addr != rr2->rdata.addr;
		break;
	case TYPE_SOA:
		i = rr1->rdata.soa->serial != rr2->rdata.soa->serial;
		break;
	case TYPE_HINFO:
		i = strcmp(rr1->rdata.hinfo.cpu,rr2->rdata.hinfo.cpu) ||
			strcmp(rr1->rdata.hinfo.os,rr2->rdata.hinfo.os);
		break;
	case TYPE_MX:
		i = strcmp(rr1->rdata.mx.exch,rr2->rdata.mx.exch);
		break;
	case TYPE_MB:
	case TYPE_MG:
	case TYPE_MR:
	case TYPE_PTR:
	case TYPE_TXT:
	case TYPE_NS:
		i = strcmp(rr1->rdata.data,rr2->rdata.data);
		break;
	case TYPE_CNAME:
		i = strcmp(rr1->rdata.data,rr2->rdata.data);
		break;
	case TYPE_MINFO:	/* Unsupported type */
		i = strcmp(rr1->rdata.minfo.rmailbx,rr2->rdata.minfo.rmailbx) ||
			strcmp(rr1->rdata.minfo.emailbx,rr2->rdata.minfo.emailbx);
		break;
	case TYPE_MD:		/* Unsupported type */
	case TYPE_MF:		/* Unsupported type */
	case TYPE_NULL:		/* Unsupported type */
	case TYPE_WKS:		/* Unsupported type */
		i = strcmp(rr1->rdata.data,rr2->rdata.data);
		break;
	}
	return i;
}

/* Return 1 if string appears to be an IP address in dotted decimal;
 * return 0 otherwise (i.e., if string is a domain name)
 */
static int
isaddr(s)
register char *s;
{
	char c;

	if( s == NULLCHAR)
		return 1;	/* Cant't happen */

	while((c = *s++) != '\0'){
		if(!isdigit(c) && c != '.')
			return 0;
	}
	return 1;
}

static struct rr *
copy_rr(rr)
struct rr *rr;
{
	struct rr *rrp;

	rrp = (struct rr *)callocw(1,sizeof(struct rr));
	rrp->next = NULLRR;
	rrp->soarec = rr->soarec;
	rrp->name = strdup(rr->name);
	rrp->type = rr->type;
	rrp->class = rr->class;
	rrp->ttl = rr->ttl;
	rrp->origin = rr->origin;
	rrp->rdlength = rr->rdlength;
	switch(rr->type){
	case TYPE_A:
		rrp->rdata.addr = rr->rdata.addr;
		break;
	case TYPE_CNAME:
	case TYPE_MB:
	case TYPE_MG:
	case TYPE_MR:
	case TYPE_NS:
	case TYPE_PTR:
	case TYPE_TXT:
		rrp->rdata.name = strdup(rr->rdata.name);
		break;
	case TYPE_HINFO:
		rrp->rdata.hinfo.cpu = strdup(rr->rdata.hinfo.cpu);
		rrp->rdata.hinfo.os = strdup(rr->rdata.hinfo.os);
		break;
	case TYPE_MX:
		rrp->rdata.mx.pref = rr->rdata.mx.pref;
		rrp->rdata.mx.exch = strdup(rr->rdata.mx.exch);
		break;
	case TYPE_SOA:
		rrp->rdata.soa = (struct soa *)callocw(1,sizeof(struct soa));
		rrp->rdata.soa->mname = strdup(rr->rdata.soa->mname);
		rrp->rdata.soa->rname = strdup(rr->rdata.soa->rname);
		rrp->rdata.soa->serial = rr->rdata.soa->serial;
		rrp->rdata.soa->refresh = rr->rdata.soa->refresh;
		rrp->rdata.soa->retry = rr->rdata.soa->retry;
		rrp->rdata.soa->expire = rr->rdata.soa->expire;
		rrp->rdata.soa->minimum = rr->rdata.soa->minimum;
		break;
	case TYPE_MINFO:	/* Unsupported type */
		rrp->rdata.minfo.rmailbx = strdup(rr->rdata.minfo.rmailbx);
		rrp->rdata.minfo.emailbx = strdup(rr->rdata.minfo.emailbx);
		break;
	case TYPE_MD:		/* Unsupported type */
	case TYPE_MF:		/* Unsupported type */
	case TYPE_NULL:		/* Unsupported type */
	case TYPE_WKS:		/* Unsupported type */
		rrp->rdata.name = strdup(rr->rdata.name);
		break;
	}
	return rrp;
}
static char *
dn_compress(cp,name)
char *cp, *name;
{
	int len,dlen;
	char *cp1;
	dlen = strlen(name);
	for(;;){
		/* Look for next dot */
		cp1 = strchr(name,'.');
		if(cp1 != NULLCHAR)
			len = cp1-name;	/* More to come */
		else
			len = dlen;	/* Last component */
		*cp++ = len;		/* Write length of component */
		if(len == 0)
			return cp;
		/* Copy component up to (but not including) dot */
		strncpy(cp,name,len);
		cp += len;
		if(cp1 == NULLCHAR){
			*cp++ = 0;	/* Last one; write null and finish */
			return cp;
		}
		name += len+1;
		dlen -= len+1;
	}
}
/* Try to figure out if we are an authority */
/* current stratigy is: if a domain server is defined we do not have athority */
static int
haveaa(name)
char *name;
{
#ifdef notdef
	char *cp;
	struct rr *rrp;
	int len, cnt = 0;

	cp = name;
#endif
	if(Dtrace) printf("Haveaa: called for %s\n",name);
	return Dhaveaa;
#ifdef notdef
	/* Try to find a SOA for the whole address and by stripping off
	   the first part of the address. */
	while(cnt < 4 && (rrp = resolverec(cp,TYPE_SOA)) == NULLRR){
		if((cp = strchr(cp,'.')) == NULLCHAR)
			break;
		if(*(++cp) == '\0')
			break;
		++cnt;
	}
	if(rrp != NULLRR){
		len = strlen(rrp->rdata.soa->mname);
		if(rrp->rdata.soa->mname[len-1] == '.')
			--len;
		if(len == strlen(Hostname) &&
		   strncmp(Hostname,rrp->rdata.soa->mname,len) == 0){
			free_rr(rrp);
	if(Dtrace) printf("Haveaa: returning 1 for %s\n",name);
			return 1;
		}
		free_rr(rrp); 
	}
	if(Dtrace) printf("Haveaa: returning 0 for %s\n",name);
	return 0;
#endif  /* notdef */
}
static char *
type2str(type)
int type;
{
	if(type < Ndtypes) 
		return Dtypes[type];
	else
		switch (type) {
			case TYPE_AXFR:
				return "AXFR";
			case TYPE_MAILB:
				return "MAILB";
			case TYPE_MAILA:
				return "MAILA";
			case TYPE_ANY:
				return "ANY";
		}
	return "UNKNOWN";
}
/* analyse suffixes for names to be added to the memory list */
/* if Lorigin is empty, PA0GRI's domain stuf is not active.  */
static void
analsuf(rrp)
struct rr *rrp;
{
	char *n1, *n2;
	int i;
	struct dlist *llist, *lorigin;

	if(rrp->origin != NULLCHAR)
		return;	/* no job for me. */
	if((llist = Lorigin) != NULLDLIST) { /* Am i working for PA0GRI?*/
		i = strlen(rrp->name);
		if(rrp->name[i-1] == '.') {
			n1 = &rrp->name[i-1];
			n2 = strchr(rrp->name,'.');
			if(n1 == n2){
				return; /* only 1 dot */
			}
			n2++;	/* point byond dot */
			while(llist != NULLDLIST) {
				lorigin = llist;
				if(strcmp(llist->name,n2) == 0)
					break;
				llist = llist->next;
			}
			if(llist == NULLDLIST) {
				llist = (struct dlist *)callocw(1,
				   sizeof(struct dlist));
				llist->name = strdup(n2);
				lorigin->next = llist;
			}
			rrp->origin = llist->name;
			n2--;
			*n2 = '\0';	/* dot to EOS */
			n1 = strdup(rrp->name);
			free(rrp->name);
			rrp->name = n1;
		}
	}
}
static void
dumpdhdr(dhdr)
struct dhdr *dhdr;
{
	struct rr *rrp;
	struct quest *qp;
	int i;

	printf("->Domain header opcode: %s, status: %s, id: %u \n",
	       Dopcode[dhdr->opcode],Drcode[dhdr->rcode],dhdr->id);
	printf("-> flags: %s%s%s%s%s Ques: %d, Ans: %d, Auth: %d, Addit: %d\n"
	        ,dhdr->qr == 0 ? "" : "qr "
		,dhdr->aa == 0 ? "" : "aa "
		,dhdr->tc == 0 ? "" : "tc "
		,dhdr->rd == 0 ? "" : "rd "
		,dhdr->ra == 0 ? "" : "ra "
		,dhdr->qdcount
		,dhdr->ancount
		,dhdr->nscount
		,dhdr->arcount);
	if(dhdr->qdcount != 0) {
		printf("-> %u question records\n",dhdr->qdcount);
		for(i=0;i< dhdr->qdcount;i++){
			qp = dhdr->qlist[i];
			printf("%s type=%s class=%s\n",qp->qname,
				type2str(qp->qtype),
				(qp->qclass == 1 ? "Internet" : "Unknown"));
		}
	}
	if(dhdr->ancount != 0) {
		printf("-> %u answer records\n",dhdr->ancount);
		for(i=0;i< dhdr->ancount;i++){
			rrp = dhdr->ans[i];
			put_rr(stdout,rrp,0);
		}
	}
	if(dhdr->nscount != 0) {
		printf("-> %u authority records\n",dhdr->nscount);
		for(i=0;i< dhdr->nscount;i++){
			rrp = dhdr->ns[i];
			put_rr(stdout,rrp,0);
		}
	}
	if(dhdr->arcount != 0) {
		printf("-> %u additional records\n",dhdr->arcount);
		for(i=0;i< dhdr->arcount;i++){
			rrp = dhdr->add[i];
			put_rr(stdout,rrp,0);
		}
	}
	printf("-> end of domain message.\n");
}
