/*  AX25 Level 2 interface  --  KE3Z 08/9/87  */

#include "buffer.h"
#include "iface.h"
#include "timer.h"
#include "ax25.h"
#include "ax25l2.h"
#include "netuser.h"
#include "data.h"

void print_call(), dump_pkt();

/* axmon -- monitor received AX.25 frames. */
void
axmon(iface, bpx)
struct interface *iface;	/* Interface frame was received on */
struct datastr *bpx;		/* Received frame */
{
	static struct datastr *bp;	/* Received frame */
	static int control;
	static struct ax25_addr adr[9];
	static char rxversion;
	static char iscmd, pf, frmrej;
	static char digi, swpos;
	static char ccmd, cpf;
	static int i,j, nr, ns, pid;
	static char numadr, numadrm;	/* Number of addresses (callsigns) */

	bp=dup_pkt(bpx);
	iscmd=0;
	pf=0;
	frmrej=0;

	/* Move the address field into local storage */
	numadr=digi=swpos=0;
	if (buflen(bp) > AXALEN) {
		for (;numadr<7;numadr++) {
			for (i=0;i<ALEN;i++) {
				if (((j=bgetch(&bp)) == EOF) || (j & E)) {
					j= EOF;
					break;
				}
				adr[numadr].call[i]=j;
			}
			if (j == EOF) break;
			if ((j=bgetch(&bp)) == EOF) break;
			adr[numadr].ssid=j;

			if(!digi && numadr>1 && !(j & REPEATED)) {
				digi=numadr;	/* Digi Req? */
			}
			if (j & E) break;
		}
		if (j == EOF) numadr=0;
		else numadr++;
	}

	printf("%s :", iface->name);	/* Port */
	print_call(&adr[1]);	/* Source */
	printf(">");
	print_call(&adr[0]);	/* Dest */
	for (j=2;j<numadr;j++) /* Print all digi calls */ {
		printf(",");
		print_call(&adr[j]);
		if (j == digi) printf("*");
	}

	control = bgetch(&bp);

	ccmd = 0x20;	/* Default to version 1 */
	cpf = 0x3f;	/* Default to confused */ 
	/* Determine the protocol version of the received frame */
	if ((adr[0].ssid ^ adr[1].ssid) & C)
		rxversion = V2;
	else
		rxversion = V1;

	/* Determine if received frame is command or response */
	if ((adr[0].ssid & C) && !(adr[1].ssid & C)) iscmd = 1;
	if (control & PF) pf = 1;
	if ( iscmd && pf) cpf = 0x50;	/* P */
	if (!iscmd && pf) cpf = 0x52;	/* R */

	nr = (control &0xe0)>>5;
	ns = (control & 0x0e)>>1;

	/* Process the frame based on control field type
	 * I frames: process the receive-sequence number N(R), followed
	 * by the I field.
	 */

	if (!(control & 1)) 	/* I frame */ {
		printf(" <I %d %d>",ns, nr);
		pid = bgetch(&bp);
		if (pid == PID_X25) printf("CCITT X.25 Level 3\n");
		else if (pid == PID_TNC) printf("ARRL AX.25 Level 2 PAD\n");
		else printf("Unknown PID %X\n", pid&0xff);
		dump_pkt(pid, bp);

	/* S frames: process the N(R), then respond via state table
	 * nothing tricky here
	 */
	} else if ((control & 3) == S) { /* S frame */
		switch (control & 0x0f) {
		case RR:
			printf(" <RR %c %c %d>",ccmd, cpf, nr);
			break;
		case REJ:
			printf(" <REJ %c %c %d>",ccmd, cpf, nr);
			break;
		case RNR:
			printf(" <RNR %c %c %d>",ccmd, cpf, nr);
			break;
		default:	/* Invalid control field received */
			printf(" <SS? %c %c %d>",ccmd, cpf, nr);
			break;
		}
	} else {
	/* U frames: respond via state table except for SABM and UA,
	   which require a little special handling.
	 */
		switch (control & 0xef) { /* U frame */
		case SABM:
			printf(" <SABM %c %c>",ccmd, cpf);
			break;
		case DISC:
			printf(" <DISC %c %c>",ccmd, cpf);
			break;
		case DM:
			printf(" <DM %c %c>",ccmd, cpf);
			break;
		case UA:
			printf(" <UA %c %c>",ccmd, cpf);
			break;
		case FRMR:
			printf(" <FRMR %c %c>",ccmd, cpf);
			break;
		default:	/* Invalid control field */
			printf(" <U?? %c %c>",ccmd, cpf);
			break;
		}
	}
	if (buflen(bp) > 0) /* We have a data field */ {
		printf(":");
		dump_pkt( -1, bp);
	}
	printf("\n");
}

void
print_call(call)
struct ax25_addr *call;
{
	static int i;

	for (i=0;i<6;i++)
		if (call->call[i] != 0x40)
			printf("%c", (call->call[i]>>1)&0x7f);
	if (call->ssid & SSID) printf("-%d",(call->ssid&SSID)>>1);
}

void
dump_pkt(pid, pkt)
int pid;
struct datastr *pkt;
{
	static unsigned char chbuf[16];	/* For the ASCII dump */
	static int chlen;		/* Length of what we have in chbuf */
	static int i, j, ch;

	if (pid == PID_TNC) /* ASCII Data */ {
		while (buflen(pkt)) /* There is more data... */ {
			ch = bgetch(&pkt);
			if (ch == EOF) break;
			if (ch == 0x0d) printf("\n");
			else printf("%c",ch);
		}
	}
	else /* Unknown, print it all in hex */ {
		while (buflen(pkt)) /* There is more data... */ {
			chlen = 0;
			for (i=0;i<16;i++) /* Only 16 Bytes/line */ {
				ch = bgetch(&pkt); /* Get next byte */
				if (ch == EOF) break; /* No more in pkt */
				printf("%2.2X ", (ch&0x0ff));
				chbuf[chlen++]=(ch&0xff);
			}
			while (i++ < 16) printf("   ");
			printf("      *");
			for (i=0;i<chlen;i++) {
				ch = chbuf[i] & 0x7f;
				if (ch < 0x20) ch = 0x2e;
				printf("%c",ch);
			}
			while (i++ < 16) printf(" ");
			printf("*\n");
		}
	}
}
