/*  AX25 Level 2 interface  --  KE3Z 08/9/87  */

#include "buffer.h"
#include "iface.h"
#include "timer.h"
#include "ax25.h"
#include "ax25l2.h"
#include "netuser.h"
#include "data.h"
#include "hdlctl.h"

void print_call(), dump_pkt();
struct datastr *bappstr();

extern char crlf[];

unsigned char axI[]=" <I ";
unsigned char axRR[]=" <RR ";
unsigned char axRNR[]=" <RNR ";
unsigned char axREJ[]=" <REJ ";
unsigned char axSS[]=" <SS? ";
unsigned char axSABM[]=" <SABM ";
unsigned char axUA[]=" <UA ";
unsigned char axUI[]=" <UI ";
unsigned char axDISC[]=" <DISC ";
unsigned char axDM[]=" <DM ";
unsigned char axFRMR[]=" <FRMR ";
unsigned char axU[]=" <U? ";
unsigned char *ftyp[10]={axI,	axRR,	axRNR,	axREJ,	axUI,
			axDISC,	axUA,	axFRMR,	axDM,	axSABM};
unsigned char decstr[]="0123456789ABCDEF";
unsigned char xstr[40];

/* axmon -- monitor received AX.25 frames. */
void
axmon(monid, bpx)
char *monid;
struct datastr *bpx;		/* Received frame */
{
	static struct datastr *bp;	/* Received frame */
	register struct datastr *wpkt;
	static int control;
	static struct ax25_addr adr[9];
	static unsigned char ctl, iscmd;
	static char digi;
	static int j, pid;
	static unsigned char numadr;	/* Number of addresses (callsigns) */

	bp=dup_pkt(bpx);

	/* Move the address field into local storage */
	numadr=get_addr(bp, adr, TRUE);

	control = bgetch(&bp);
	ctl = decode_ctl(control);

	/* Determine if received frame is command or response */

	iscmd = ((adr[0].ssid & C) && !(adr[1].ssid & C));

	wpkt=new_buffer(80);
	wpkt=bappstr(wpkt, monid);	/* Identify where called from */
	wpkt=bappstr(wpkt, " :");
	print_call(wpkt, &adr[1]);	/* Source */
	bappch(wpkt, ( iscmd ? 0x3e : 0x3c));
	print_call(wpkt, &adr[0]);	/* Dest */
	for (j=2;j<numadr;j++) /* Print all digi calls */ {
		bappch(wpkt, 0x2c);
		print_call(wpkt, &adr[j]);
		if (j == digi) bappch(wpkt, 0x2a);
	}
	wpkt=bappstr(wpkt, ftyp[ctl]);

	/* Process the frame based on control field type
	 * I frames: process the receive-sequence number N(R), followed
	 * by the I field.
	 */

	pid = -1;

	if (ctl < 4)	/* I frame or Supervisory */ {
		if (ctl == 0) {
			bappch(wpkt, decstr[ctl_NS&0x0f]); /* I frame */
			bappch(wpkt, 0x20);
		}
		else wpkt=bappstr(wpkt, (iscmd ? "P " : "F "));
		bappch(wpkt, decstr[ctl_NR&0x0f]);
		wpkt=bappstr(wpkt, ">");

		if (ctl == 0) pid = bgetch(&bp);	/* I Frame */
			
	}
	else /* Unnumbered frame */ {
		wpkt=bappstr(wpkt, (iscmd ? "P>" : "F>"));
		if (ctl == ctl_UI) pid = bgetch(&bp);
	}
	if (pid != -1) {
		if (pid == PID_X25)
			wpkt=bappstr(wpkt, " CCITT X.25 Level 3");
		else if (pid == PID_TNC)
			wpkt=bappstr(wpkt," AX.25 Level 2 PAD");
		else {
			wpkt=bappstr(wpkt, " Unknown PID ");
			putxch(wpkt, pid&0xff);
 		}
		wpkt=bappstr(wpkt, crlf);
 	}
	padtx(wpkt);
	if (buflen(bp) > 0) /* We have a data field */ {
		dump_pkt(pid, bp);
	}
	padtxs(crlf);
	free_pkt(bp);
}

void
print_call(pkt, callx)
struct datastr *pkt;
struct ax25_addr *callx;
{
	static int i;
	register unsigned char *xp;
	static struct ax25_addr *call;

	xp=xstr;
	call = callx;
	for (i=0;i<6;i++)
		if (call->call[i] != 0x40)
			*xp++ = (call->call[i]>>1)&0x7f;
	if ((i=call->ssid & SSID)) {
		*xp++ = 0x2d;
		i= i>>1;
		if (i>9) {
			*xp++ = 0x31;
			i -= 10;
		}
		*xp++ = 0x30+i;
	}
	*xp = 0;
	pkt=bappstr(pkt,xstr);
}

void
dump_pkt(pid, pktx)
int pid;
struct datastr *pktx;
{
	static unsigned char chbuf[16];	/* For the ASCII dump */
	register unsigned char *xp;
	static int chlen;		/* Length of what we have in chbuf */
	static int i, ch;
	static struct datastr *pkt;
	static struct datastr *wpkt;

	wpkt = NULL;
	pkt=pktx;
	if (pid == PID_TNC) /* ASCII Data */ {
		wpkt=new_buffer(128);
		while (buflen(pkt)) /* There is more data... */ {
			ch = bgetch(&pkt);
			if (ch == EOF) break;
			if (bappch(wpkt, ch)) /* Full */  {
				padtx(wpkt);
				wpkt=new_buffer(128);
			}
			if (ch == 0x0d) bappch(wpkt, 0x0a);
		}
		padtx(wpkt);
	}
	else /* Unknown, print it all in hex */ {
		while (buflen(pkt)) /* There is more data... */ {
			wpkt=new_buffer(80);
			chlen = 0;
			for (i=0;i<16;i++) /* Only 16 Bytes/line */ {
				ch = bgetch(&pkt); /* Get next byte */
				if (ch == EOF) break; /* No more in pkt */
				putxch(wpkt, ch);
				bappch(wpkt, 0x20);
				chbuf[chlen++]=(ch&0x7f);
			}
			xp = xstr;
			while (i++ < 18) {
				*xp++ = 0x20;
				*xp++ = 0x20;
				*xp++ = 0x20;
			}

			*xp++ = 0x2a;
			for (i=0;i<chlen;i++) {
				ch = chbuf[i] & 0x7f;
				if (ch <= 0x20 || ch == 0x7f) ch = 0x2e;
				*xp++ = ch;
			}
			while (i++ < 16) *xp++ = 0x20;
			*xp++ = 0x2a;
			*xp++ = 0x0d;
			*xp++ = 0x0a;
			*xp++ = 0;
			wpkt=bappstr(wpkt, xstr);
			padtx(wpkt);
		}
	}
}
