/*
 * Copyright 1988 by the Radio Amateur Telecommunications Society
 * and Thomas A. Moulton, W2VY
 *
 * This software may only be modified, copied, distributed or
 * executed for non-profit purposes by individuals operating
 * systems in the Amateur Radio Service.  Credit to the
 * author(s) and to the Radio Amateur Telecommunications Society
 * must be made in modules where RATS provided software is used,
 * and in any announcements and documentation.  
 *
 * As a non-profit, research and development organization,  the
 * Radio Amateur Telecommunications Society distributes software
 * in both executable and source forms.  This policy is in place
 * to encourage the development and distribution of OSI-based,
 * networking tools.  In order to protect the interests of the
 * Society and the authors, we have placed some conditions
 * of use on the software.  Other groups are encouraged
 * to place the same or similar guidelines on
 * software they produce.
 *
 * The Radio Amateur Telecommunications Society reserves the right
 * to specify and alter the terms under which software provided by
 * the Society may be used.  This policy is consistent with the 
 * objective of uniform and consistent "Open Systems Interconnections."
 * 
 * All acceptable Amateur Radio related uses of this software
 * will be outlined in the "ROSE Implementer's Guide".  Individuals
 * or organizations wishing to add to, or modify the provisions of
 * the guide to accommodate local or evolutionary requirements
 * should document the proposed change(s) and forward them to the
 * Society.  If accepted, written notification will be provided by
 * the Society to the submitting organization or individual(s).
 * The Society will then issue a "ROSE Implementer's Guide Change
 * Notice".  Periodically, the Society will re-issue the "ROSE 
 * Implementer's Guide" and incorporate the text of the change 
 * notices.  This procedure has been put in to place to ensure
 * compatibility between systems and to ensure their "Openness"
 * and interoperability.
 *
 * No part of this software may be used in other packages 
 * without prior authorization from the author or the Society.  
 * Software incorporating this module, all or in part, must be 
 * provided to the Society prior to distribution or use by
 * anyone not directly involved in testing of the revised  
 * environment.  Current releases of the combined software must
 * be provided to the Society in both source and executable
 * forms.  Adequate documention to produce an executable module 
 * from the provided source must also be included.
 *
 * Non-Amateur Radio non-profit uses may be authorized on a case
 * by case basis.  Inquiries for such use may be made in writing
 * to the Society. Non-commercial uses consistent with the
 * general principles of Open Systems Interconnection Reference
 * Model will be generally considered with favor.
 *
 * Commercial licensing of the software is also available based
 * on normal commercial terms.  Licensing inquiries should be
 * directed to the Society.  Commercial licensing of the standard
 * software will be done in situations which materially benefit
 * the Amateur Radio Packet Network.  Additional licensing is
 * reserved by the individual authors.
 * 
 * The Radio Amateur Telecommunications Society provides this software
 * on an "as is" basis.  The Society assumes no liability for
 * loss incurred through the use of this software.  Amateur Radio
 * use of this software implies non-commercial and voluntary 
 * development, deployment and use of this software in a "Amateur",
 * non-commercial service.  Commercial users are encouraged to
 * inspect their copies of the source code.  Source code modification
 * licenses are available if a combined Object and Source Code
 * license was not originally established.
 * 
 * The Society may be contacted by writing or calling at:
 * 
 * The Radio Amateur Telecommunications Society 
 * 206 North Vivyen Street.
 * Bergenfield, New Jersey 07621
 *
 * Telephone: 201-387-8896
 *
 */
/* This file has the level 2 routines that are needed for the Level 3 test
   environment and will be replaced with real level 2 counterparts when
   the level 3 code is intergerated with the level 2 (from KE3Z) */

#include "l3calls.h"

extern struct axcb *links;
extern struct interface *ifaces[8];

extern int USER_L3_W;	/* User Level 3 Window */

extern struct ax25_parms *l2parms, *l3parms;
extern struct ax25_parms dl2parms, dl3parms;

struct ax25_addr L2_source, L2_destin[6];

int find_call();
char *calloc();
 
struct axcb *
L2_CONN(port, d1, d2, d3, d4, d5, l2user)
char port;
struct ax25_addr *d1, *d2, *d3, *d4, *d5;
char l2user;
{
	static struct axcb *lnk;
	static unsigned char i, alen;
	static struct ax25_addr *d[5];
	register struct ax25_addr *ax;

	d[0]=d1;
	d[1]=d2;
	d[2]=d3;
	d[3]=d4;
	d[4]=d5;

	ax = &L2_destin[0];	/* ax points to the last address field */
	if (L2_source.call[0] == 0 || ax->call[0] == 0) return NULL;

	alen=1;
	for (i=0;i<5;i++) {
		if (d[i] != NULL) {
			if (d[i]->call[0]) {
				callcpy(++ax, d[i]);
				alen++;
			}
		}
	}

	ax->ssid |= E; /* Set eof bit in address */

	lnk=open_ax25(&L2_source, &L2_destin, ifaces[port], alen);
	if (!lnk) return lnk;
	if (!l2user) lnk_l3_pars(lnk);
	lnk->port = port;
	lnk->R = 0;
	axconnect(lnk);
	return lnk;
}

struct VCS *
new_vc()
{
	register struct VCS *vc;
	extern char node_addr[16];
	extern int (*L3PARS[])();
/*	int l3_reset(), l3_clear(), l3_send(), l3_recv(), l3_connected(); */

	vc=(struct VCS *)calloc(1,sizeof(struct VCS));
	if (vc != NULL) {
		vc->WI=USER_L3_W;	/* Assume it's a user */
		vc->lcn = 255;		/* It's still Pending */
		bmove(&L3PARS, &vc->RESET, 5*sizeof(vc->RESET));
		adr_copy(0,node_addr,0,vc->addr);
#if 0
11/1/89		vc->RESET=l3_reset;
		vc->CLEAR=l3_clear;
		vc->SEND=l3_send;
		vc->RECV=l3_recv;
		vc->CONNECTED=l3_connected;
#endif
	}
	return (vc);
}

struct VCS *
new_vcs()
{
	register struct VCS *vc;

	if (!(vc=new_vc())) return NULL;
	if (!(vc->peer=new_vc())) {
		free(vc);
		return NULL;
	}
	vc->peer->peer = vc;
	return (vc);
}

struct axcb *
NET_CONN(trunk)
unsigned char trunk;
{
	callcpy(&L2_source, &L3CALL[0]);	/* Switch is source */
	callcpy(&L2_destin[0], &L3CALL[trunk]);	/* Network Dest */
	return (L2_CONN( L3PORT[trunk],		/* Port */
			&L3DIGI[trunk],		/* Optional digi */
			NULL,NULL,NULL,NULL,	/* Only 1 allowed */
			FALSE) );		/* Level 3 Link */
}

struct axcb *
USER_CONN(port, vc)
char port;
register struct VCS *vc;
{
	if (vc && vc->peer)
	return (L2_CONN(port,	/* Port to get user on */
				/* The guy I'm acting for is in L2_source */
				/* Who he wants to talk to is in L2_destin[0] */
		&vc->peer->digi, /* Optional Source Digi */
		&L2_destin[2],	/* Optional DNIC */
		&L2_destin[3],	/* Source Address */
		&L3CALL[0],	/* I'm the transmitter */
		&vc->digi,	/* Optional exit digi */
		TRUE) );	/* Level 2 Call */
	else return NULL;
}
