/*
 * Copyright 1988 by the Radio Amateur Telecommunications Society
 * and Thomas A. Moulton, W2VY
 *
 * This software may only be modified, copied, distributed or
 * executed for non-profit purposes by individuals operating
 * systems in the Amateur Radio Service.  Credit to the
 * author(s) and to the Radio Amateur Telecommunications Society
 * must be made in modules where RATS provided software is used,
 * and in any announcements and documentation.  
 *
 * As a non-profit, research and development organization,  the
 * Radio Amateur Telecommunications Society distributes software
 * in both executable and source forms.  This policy is in place
 * to encourage the development and distribution of OSI-based,
 * networking tools.  In order to protect the interests of the
 * Society and the authors, we have placed some conditions
 * of use on the software.  Other groups are encouraged
 * to place the same or similar guidelines on
 * software they produce.
 *
 * The Radio Amateur Telecommunications Society reserves the right
 * to specify and alter the terms under which software provided by
 * the Society may be used.  This policy is consistent with the 
 * objective of uniform and consistent "Open Systems Interconnections."
 * 
 * All acceptable Amateur Radio related uses of this software
 * will be outlined in the "ROSE Implementer's Guide".  Individuals
 * or organizations wishing to add to, or modify the provisions of
 * the guide to accommodate local or evolutionary requirements
 * should document the proposed change(s) and forward them to the
 * Society.  If accepted, written notification will be provided by
 * the Society to the submitting organization or individual(s).
 * The Society will then issue a "ROSE Implementer's Guide Change
 * Notice".  Periodically, the Society will re-issue the "ROSE 
 * Implementer's Guide" and incorporate the text of the change 
 * notices.  This procedure has been put in to place to ensure
 * compatibility between systems and to ensure their "Openness"
 * and interoperability.
 *
 * No part of this software may be used in other packages 
 * without prior authorization from the author or the Society.  
 * Software incorporating this module, all or in part, must be 
 * provided to the Society prior to distribution or use by
 * anyone not directly involved in testing of the revised  
 * environment.  Current releases of the combined software must
 * be provided to the Society in both source and executable
 * forms.  Adequate documention to produce an executable module 
 * from the provided source must also be included.
 *
 * Non-Amateur Radio non-profit uses may be authorized on a case
 * by case basis.  Inquiries for such use may be made in writing
 * to the Society. Non-commercial uses consistent with the
 * general principles of Open Systems Interconnection Reference
 * Model will be generally considered with favor.
 *
 * Commercial licensing of the software is also available based
 * on normal commercial terms.  Licensing inquiries should be
 * directed to the Society.  Commercial licensing of the standard
 * software will be done in situations which materially benefit
 * the Amateur Radio Packet Network.  Additional licensing is
 * reserved by the individual authors.
 * 
 * The Radio Amateur Telecommunications Society provides this software
 * on an "as is" basis.  The Society assumes no liability for
 * loss incurred through the use of this software.  Amateur Radio
 * use of this software implies non-commercial and voluntary 
 * development, deployment and use of this software in a "Amateur",
 * non-commercial service.  Commercial users are encouraged to
 * inspect their copies of the source code.  Source code modification
 * licenses are available if a combined Object and Source Code
 * license was not originally established.
 * 
 * The Society may be contacted by writing or calling at:
 * 
 * The Radio Amateur Telecommunications Society 
 * 206 North Vivyen Street.
 * Bergenfield, New Jersey 07621
 *
 * Telephone: 201-387-8896
 *
 */

extern char node_addr[];

struct VCS *clear_vcs(vc)
register struct VCS *vc;
{
	static struct VCS *ret, *last, *ptr;
	static struct VCS *vcp;

	ret=NULL;
	last=NULL;
	while (vc != NULL) {
		if (vc->lcn != 255) {
			vcp=vc->peer;
			if (vcp != NULL) {
				vcp->peer=NULL;
				set_p(vcp, P6,DTE_Orig+233);
				vc->peer=NULL;
			}
			ptr=vc->next;
			set_p(vc, P1, 0);
			vc=ptr;
		}
		else {
			if (last==NULL) /* first one */ {
				ret=vc;
				last=vc;
			}
			else /* Got another */ {
				last->next=vc;
				last=vc;
			}
			vc=last->next;
			last->next=NULL;
		}
	}
	return ret;
}

process_calls(lnkx)
struct axcb *lnkx;
{
	static struct VCS *vcn;
	register struct VCS *vcp;
	static struct axcb *lnk;

	lnk=lnkx;
	if ((vcp = lnk->llcn) != NULL) {
		while (vcp->lcn == 255) /* First on list is pending */ {
			lnk->llcn = vcp->next;
			vcp->next = NULL;
			set_p(vcp,P2,0); /* Issue call request if you can */
			vcp = lnk->llcn;
		}
		/* The above loop will loop until there is 1 successful call */

		while (vcp->next != NULL) /* There are more calls to do */ {
			vcn=vcp->next;
			if (vcn->lcn == 255) /* Got a call to process */ {
				vcp->next=vcn->next;
				vcn->next=NULL;
				set_p(vcn,P2,0);
			}
			vcp=vcp->next;
		}
	}
}

adr_copy(src,istrx,des,ostrx)
unsigned char src;
unsigned char istrx[16];
unsigned char des;
unsigned char ostrx[16];

{
	static unsigned char ipos, opos, ilow, olow, len, i, acc;
	static unsigned char *istr, *ostr;

	istr=istrx;
	ostr=ostrx;
	if (src) /* Copy High order call (calling address) */ {
		ilow=istr[0]&1;
		ipos= ( (istr[0] & 0x0f) + ilow) >> 1;
		len=(istr[0] & 0x0f0)>>4;
		if (!ilow) ipos++;
	}
	else /* Copy low order call (called address) */ {
		ipos=1;
		ilow=0;
		len=istr[0] & 0x0f;
	}
	if (des) /* Copy to high order call (calling) */ {
		olow=ostr[0] & 1;
		opos=( (ostr[0] & 0x0f) + olow) >> 1;
		ostr[0]=ostr[0] & 0x0f | len<<4;
		if (!olow) opos++;
	}
	else /* Copy to low order call (called) */ {
		olow=0;
		opos=1;
		ostr[0]=ostr[0] & 0xf0 | len;
	}
	acc=0;
	if (olow) acc=ostr[opos] & 0xf0; /* Init acc */
	for (i=1;i<=len;i++) {
		if (ilow) acc += istr[ipos++] & 0x0f;
		else acc += (istr[ipos] & 0x0f0) >> 4;
		ilow = !ilow;
		if (olow) /* Output a byte */ {
			ostr[opos++]=acc;
			acc=0;
		}
		else acc=acc<<4;
		olow = !olow;
	}
	if (olow) ostr[opos]=acc;
}

#if 0
int
lcncnt(vc)
register struct VCS *vc;
{
	static int i;

	i=0;
	while (vc) {
		i++;
		vc = vc->next;
	}
	return i;
}
#endif

unsigned char
glcnlow(lnk)	/* Get lcn, return 0 if all in use */
struct axcb *lnk;
{
	static unsigned char i;
	register struct VCS *p;

	i=1;
	p=lnk->llcn;
	while (p) /* There are more vc's */ {
		if (p->lcn != i) /* Found a Hole ! */ return i;
		i++;
		p=p->next;
	}
	if (i<=lnk->maxvc) return i;
	return 0;
}

unsigned char
glcnhigh(lnkx)	/* Get lcn, return 0 is all in use */
struct axcb *lnkx;
{
	static unsigned char i, j, max;
	register struct VCS *p;
	static struct axcb *lnk;

	lnk=lnkx;
	max=lnk->maxvc;
	p=lnk->llcn;
	if (!p) /* No vc's yet */ return max;
	if (p->lcn == 255) /* No active vc's yet */ return max;
	j=0;
	i=1;
	while (p) /* There are more to check */ {
		if (p->lcn != i) /* Found Hole */ {
			i=p->lcn;
			j=i-1; /* Save highest lcn # in hole */
		}
		i++;
		p=p->next;
		if (p) if (p->lcn == 255) break; /* Pending calls are at end */
	}
	if (i <= lnk->maxvc) return max;
	return j;
}

insert_lcn(vcx)	/* Insert a vc onto it's parent */
struct VCS *vcx;
{
	register struct VCS *p;
	static unsigned char i;
	static struct VCS *vc;
	static struct axcb *lnk;

	vc=vcx;
	lnk=vc->parent;
	i=vc->lcn;
	p=lnk->llcn;
	vc->next=NULL;	/* Just in case... */
	if (!p) /* No VC's yet, it's easy... */ {
		lnk->llcn=vc;
		return;
	}
	if (p->lcn > i) /* Insert at head */ {
		vc->next=p;	
		lnk->llcn=vc;
		return;
	}
	while (p->next) /* More to check */ {
		if (p->next->lcn > i) /* Insert before the next */ {
			vc->next=p->next;
			p->next=vc;
			return;
		}
		p=p->next;
	}
	p->next=vc;	/* Fell through, append this guy */
}

addlcn(vc)
register struct VCS *vc;

{
	static struct axcb *p;

	if (p=vc->parent) {
#if 0
		if (p->am_dce) vc->lcn=glcnlow(p);
		else vc->lcn=glcnhigh(p);
#endif
		vc->lcn = (p->am_dce ? glcnlow(p) : glcnhigh(p));
		if (vc->lcn) /* Not zero, no error */ {
			insert_lcn(vc);
			return TRUE;
		}
	}
	return FALSE;
}

dellcn(vc)
register struct VCS *vc;
{
	static struct VCS **l;
	static struct VCS *vc1;
	extern struct VCS *pending_calls;

	if (!vc->parent) /* on pending_calls */ l= &pending_calls;
	else l= &vc->parent->llcn;
	if (*l) /* Not NULL */ {
		vc1= *l;
		if (vc1==vc) *l=vc->next;
		else {
			while (vc1) /* Not NULL */ {
			   if (vc1->next==vc) break;
			   vc1=vc1->next;
			}
		if (vc1) vc1->next=vc->next;
		}
	}
	vc->next=NULL;
	vc->parent=NULL;
}

int
check_x121(ptrx,adr)
register unsigned char adr[];
unsigned char ptrx[];
{
	static unsigned char i, j, alen, *ptr;

	ptr=ptrx;
	alen=ptr[0] & 0x0f;
	if ( (adr[0] & 0x0f < alen) || (alen == 0) ) return -1;
	j=alen>>1;

	for (i=1; i<=j; i++)
		if (adr[i] != ptr[i]) return -1;

	if (alen & 1 && /* Length is odd, check last nibble */
		 ( ptr[i] &0xf0 != (adr[i] & 0x0f0) )) return -1;
	if (alen == adr[0]&0x0f) return 0;
	return 1;
}

int
x121_ch(x121, pos)
unsigned char x121[];
char pos;
{
	static unsigned char ch;

	ch=x121[((pos+1) >> 1)];
	ch = ((pos & 1) ? ((ch & 0x0ff) >> 4) : (ch & 0x0f));
	return (ch | 0x30);
}

l2_adr(x121,strd,str)
unsigned char x121[16];
struct ax25_addr *strd;
struct ax25_addr *str;
{
	static unsigned char i, j,l,ch;
	static char dn[3];
	register struct ax25_addr *s;

	s=str;
	for (j=0;j<2;j++) {
		s->ssid=0;
		for (i=0;i<ALEN;i++) s->call[i]=0x40;
		s=strd; /* will need strd later */
	}
	if ( (l=x121[0]) < 5) return;
	if (l>11) l=11;

	dn[0] = 4;	/* DNIC's are 4 digits long */
	dn[1] = node_addr[1];
	dn[2] = node_addr[2];
	if (check_x121(dn,x121) != 1) /* Not same DNIC as this switch */ {
		j=0;
		for (i=1;i<=4;i++) {
			s->call[j++] = x121_ch(x121, i) << 1;
		}
		s->ssid |= 0x60;	/* Set Reserved Bits */
	}
	else s->call[0]=0;	/* No call here */

	s=str;
	j=0;
	for (i=5;i<=l;i++) {
		ch = x121_ch(x121, i);
		if (i == 11) s->ssid= (ch & 0x0f) << 1;
		else s->call[j++] = ch << 1;
	}
	s->ssid |= 0x60;	/* Set Reserved Bits */
}

l3_adr(str,x121)
struct ax25_addr *str;
unsigned char x121[16];
{
	static unsigned char i,len,ch,hi,l;
	static unsigned char x[7];
	register unsigned char *c;

	l = x121[0];
	x121[0] = 0;
	c=str->call;
	for (len=0;len<ALEN;) /* Insure valid [shifted] numbers */ {
		ch = (*c++)>>1;
		if (ch == 0x20) break; /* End of "callsign" */
		if (ch == 0x4f) ch = 0x30;		/* O = 0 */
		if (ch == 0x49 || ch == 0x4c) ch = 0x31; /* I or L = 1 */
		if ((ch < 0x30) || (ch > 0x39)) return; /* Non-digit */
		x[len++] = (ch - 0x30);
	}
	ch = ((str->ssid)>>1 & 0x0f);
	if (ch>0 && ch<10) x[len++] = ch;	/* Ignore if not 1-9 */

	c = x121;
	*c = l;
	if (!l) {
		if (len == 4) *c++ = 0; /* DNIC was specified */
		else /* Use default DNIC */ {
			*c++ = 4;		/* DNIC's are 4 digits long */
			*c++ = node_addr[1];	/* In the same DNIC as this */
			*c++ = node_addr[2];	/* Switch, use Node Address */
		}
	} else c += (*c>>1)+1; /* Adding to the address */

	hi = (! (x121[0] & 1) );
	x121[0] += len;

	for (i=0;i<len;i++) {
		ch = x[i];
		if (hi) *c = ch<<4;
		else {
			*c |= ch;
			c++;
		}
		hi = !hi;
	}
}

int
call_str(stnx, str)
unsigned char stnx[];
register char *str;
{
	static unsigned char i, j;
	static unsigned char *stn, ch;

	stn = stnx;
	j=0;
	for (j=0; j<6; j++) /* Copy callsign to str */ {
		ch = *stn++;
		if ( (ch == 0x40) || !ch ) break;
		*str++ = (ch >> 1);
	}
	i = (stnx[6]>>1) & 0x0f;

	*str++ = 0x2d;	/* Dash */

	j++;
	if (i > 9) {
		*str++ = 0x31;
		j++;
		i -= 10;
	}
	*str++ = i+0x30;
	j++;
	return j;
}

int
x121_str(x121, str)
char x121[];
register char *str;
{
	static unsigned char l, i;

	l = x121[0] & 0x0f;	/* Only Supports low order address */
	for (i=1; i<=l; i++) /* Copy and convert address */ {
		*str++ = x121_ch(x121, i);
	}
	return l;
}
