{$I-,V-}
{$M 65520,0,256000}

{  ROUTINE:	N R D

   PURPOSE:	Control an NRD525 / NRD535 Receiver + database

   USAGE:	nrd

   AUTHOR:	Tom Whiteside 11505 Oak View, Austin, TX 78759 (512) 258-5924

   REVISION:	1.0  04-30-90  TGW  Initial Release
                1.1  07-07-90  TGW  Serial routines changed to use BIOS for
                                       Windows
                1.2  07-15-90  TGW  Efficiency tweaks to inc/dec bw/mode
                                    Cursor Highlighting for current line
                                    Cursor tracking data for inc/dec freq
                1.3  07-29-90  TGW  Remove H(ide from prompt, fix Delete
                                       leaving wrong line highlighted
                                    Added Revision to prompt
                1.4  09-03-90  TGW  Fixed inc/dec mode past boundary crashing
                                       program; asserted black backgrd on
                                       title and journal
                1.5  09-29-90  TGW  Added MAP feature

                1.6  11-18-90  TGW  Fixed 2 bugs where cursor got out of
                                    sync with "status line"
                2.0  12/01-90  TGW  Mods to support other com ports, optional
                                    MAP, help                                 easier for o
                2.1  12-02-90  TGW  Added time offset to config.dat; fixed
                                    home not putting cursor at top of screen

                2.2  12-25-90  TGW  Added "Com 0" feature to allow using prgm
                                    without serial port...
                2.3  12-27-90  TGW  Fix to eliminate hang if radio off

                2.4  03-02-91  TGW  Fix for monochrome users (in screen.pas)
                2.5  03-10-91  TGW  Added Import of Sundstrom data
                2.6  03-30-91  TGW  Added Active and Inactive log concept
                                    including write from the inactive log
                                    Added "*" command to find radio freq
                                    in database.  Fixed potential hang in
                                    comreadln.  Reduced edit field length
                                    for comments by 1 char.  (Fixed wrap bug
                                    for bottom line)  Removed dangerous Read
                                    command from Journal
                2.7  04-14-91  TGW  Fixed display bug in inc_freq
                3.0  04-26-91  TGW  Added NRD535 features
                3.1  04-27-91  TGW  Fix to journal name select; Added 535
                                    mode to auto-update receiver display
                3.2  05-12-91  TGW  Added S-meter to 535; changed mode order
                                    to AM -> ECSS_U; added graphics command
                                    for 535; misc bug fixes for 535

                3.3  05-19-91  TGW  Graphics enhancements; commands for time
                                    and spectral displays.  Performance
                                    enhancement to Spectral display
                3.4  05-31-91  TGW  Misc clean-up to 535 additions mainly to
                                    isolate receiver specific stuff to nrdio.
                                    Fixed minor BWC bug with Spectral plot.
                                    Fixed bug in Import not setting
                                    attenuator.  Fixed bug with Cursor going
                                    below end of file.  Improved key-stroke
                                    performance.  Added date and time stamp
                                    to the status line.  Cleaned up various
                                    bugs in Journal.  Added Find mode to
                                    Spectral plot.
                3.5  06-02-91  TGW  Insured that "Continue" to Spectral plot
                                    resumes where it left off.  Insured
                                    Stop Frequency is higher than Start
                                    Frequency for Spectral Plots.  Added "8"
                                    alias for "*" command.  Made Search put
                                    cursor at top of screen.  Added config-
                                    uration string for printer to config.dat.
                                    modified config.dat to rebuild file on
                                    error.

                3.6  06-16-91  TGW  Bug fix in time stamp

                4.0  07-05-91  TGW  Added "F" command
                                    Added CTL-M and CTL-U commands
                                    Change Mark to range based cmd
                                    Added CTL-S and CTL-A search commands
                                    Added comment field to sort menu
                                    Added ASCII Import and Export commands
                                    Added All and Marked to print options
                4.1  07-05-91  TGW  Fixed data integrity problem.  Hopefully
                                    no one got 4.0
                4.2  07-20-91  TGW  Fixed AGC not being set correctly on 535
                                    Fixed minor bug with Search cursor pos
                                    Added new modes to inc and dec
                                    Fixed bug in printer config for 525 vers.
                                    Added moVe command
                                    changed CTL-U to CTL-N to get around
                                       conflict with right arrow key...
                                    added information mode for 525 users
                                    printout tweak for longer location field
                4.3  07-21-91  TGW  Fixed bug in scan not updating display
                                       for 535 when s-meter not enabled
                                    Added S-Meter to inc/dec freq scan
                4.4  08-24-91  TGW  Reduced receiver display overhead
                                    Minimized S-Meter/receiver display
                                       interaction and hang condition
                                    Changed CTL-N to CTL-L to fix conflict
                                       with CTL-PgUp
                                    Added Red highlighting for cursor over
                                       marked area
                                    Fixed minor bug in threshold display
                                       (read 0.00 instead of 0.10)
                                    Fixed problem with program not working
                                       in B & W mode....
                                    Fixed bug in Marked search that kept
                                       receiver locked after target is found
                                    Added squelch mode to scan and marked
                                       search modes when threshold = 0dB
                4.5  09-13-91  TGW  Increased sample count for time plot
                                    Turn Information mode off when exiting
                                       program for the 525
                                    Limit number of retries when 525 returns
                                       zeros (an NRD525 error condition)
                                    Now remember the last scan_threshold
                                       setting
                                    Increased the number of databases allowed
                                       from 15 to 84
                                    Misc cleanup of journal screen to fit 84
                                       databases and run efficiently
                                    Fixed bug when deleting empty file in
                                       journal
                                    Fixed minor edit problem on page 2.
                                    Fixed Edit highlighting title line
                                    Added clear all channels mode
                                    Fixed cursor highlight color for marked
                                       lines for CGA displays
                                    Added "Overall" command
                                    Fixed Minor clock bug
}
program nrd(input,output);

uses async,crt,dos,screen,nrdio,nrdutil,grautil,nrdjour,nrdscan;

const
       CTL_A = chr(ord('a') - ord('a') + 1);
       CTL_M = chr(ord('m') - ord('a') + 1);
       CTL_L = chr(ord('l') - ord('a') + 1);
       CTL_S = chr(ord('s') - ord('a') + 1);
       CTL_T = chr(ord('t') - ord('a') + 1);

  
begin { nrd }
  repeat
    repeat
      if async_buffer_check(ch) then
        begin
          check_status(ch,s); { they changed dial }
          show_receiver;
        end

      { check for timer tick; handle time dependent displays }
      else if tick then if enable_s_meter then timed_s_meter;

      if update_receiver_display then
        begin
          toggle_remote;
          show_receiver;
          update_receiver_display:=FALSE;
        end;
    until keypressed;
    ch:=upcase(fetch);
    case ch of
      '@':; { nop }
      '+':              do_inc;
      '-':              do_dec;
      'I':              do_inc_class;
      '*','8':          find_freq;
      '/':              begin
                          case prompt_num of
                            PAGE1: prompt_num:=PAGE2;
                            PAGE2: prompt_num:=PAGE3;
                            PAGE3: prompt_num:=PAGE4;
                            PAGE4: prompt_num:=PAGE1;
                          end;
                          cmd_prompt(prompt_num);
                          bottom_window;
                        end;
      'A':              begin
                          close(logbuf);
                          do_alternate;
                        end;
      'C':              do_confirm;
      'P':              do_page;
      'S':              do_sort(FALSE);
      'E':              do_edit;
      'F':              do_freq;
      'G':              do_graph;
      'J':              do_journal;
      'D':              do_delete;
      'N':              do_undelete;
      'O':              do_overall;
      'M':              do_mark_range;
      'U':              do_unmark_all(TRUE);
      'V':              do_move;
      CTL_M:            do_mark_line;
      CTL_L:            do_unmark_line;
      'L':              do_log;
      'R':              begin
                          enable_s_meter:=not enable_s_meter;
                          if not enable_s_meter then clear_meter;
                        end;
      CTL_S:            do_search(FALSE);
      CTL_A:            do_search(TRUE);
      'T':              do_tune;
      CTL_T:            do_threshold;
      'K':              if has_map then do_kiwa;
      'W':              do_write;
      '>':              inc_mode;
      '.':              inc_mode;
      '<':              dec_mode;
      ',':              dec_mode;
      ']':              inc_bandwidth;
      '[':              dec_bandwidth;
      'H':              call_do_help;
      PAGEUP:           do_pageup(1);
      PAGEDOWN:         do_pagedown(1);
      UP:               do_up;
      DOWN:             do_down;
      RIGHTARROW:       do_right;
      LEFTARROW:        do_left;
      BACKTAB:          do_backtab;
      TAB:              do_tab;
      CTRLPAGEUP:       do_pageup(10);
      CTRLPAGEDN:       do_pagedown(10);
      HOMEKY:           do_home;
      ENDKY:            do_end;
      else update_receiver_display:=TRUE;
    end;
  until ch = 'Q';
  goodbye;
end.  { nrd }
