{$I-,V-}

unit nrdio;

interface

const PATH        = '';
      LOGFILE     = 'LOG';
      RECDATAFILE = 'RECDAT';
      DATELEN     = 6;
      TIMELEN     = 4;
      LONGSTRLEN  = 255;
      MEDSTRLEN   = 80;
      SHORTSTRLEN = 8;
      CALLSIGNLEN = 19;
      LOCATIONLEN = 27;
      COMMENTLEN  = 69;
      MAXREC      = 2500; { maximum records in a log }
      MAXLOGS     = 84;

      FILE_NOT_FOUND = 2;

      REMOTE_DLY = 300; { msec }

type  longstr       = string[LONGSTRLEN];
      datetype      = string[DATELEN];
      timetype      = string[TIMELEN];
      calltype      = string[CALLSIGNLEN];
      locationtype  = string[LOCATIONLEN];
      commenttype   = string[COMMENTLEN];

      modetype      = (RTTY, CW, USB, LSB, AM, FM, FAX, ECSS_USB, ECSS_LSB);
      bandwidthtype = (NARR, INTER, WIDE, AUX);
      agctype       = (OFF, FAST, SLOW);
      attentype     = (NO, YES);
      rectype       = (SHOW, HIDE, DELETED);

      logtype = record
                  date:       datetype;
                  begin_time,
                  end_time:   timetype;
                  frequency:  real;
                  callsign:   calltype;
                  location:   locationtype;
                  comment:    commenttype;
                  mode:       modetype;
                  bandwidth:  bandwidthtype;
                  agc:        agctype;
                  attenuator: attentype;
                end;

      recarraytype = packed array[1..MAXREC] of 0..MAXREC;

      recdatatype = packed record
                  recptr:   recarraytype;
                  m7000ptr: recarraytype; { not used }
                  recstat:  packed array[1..MAXREC] of rectype;
                end;

      short_str = string[SHORTSTRLEN];

      sort_array_type = array[1..MAXREC] of short_str;

      journaltype = record
                  logname:  short_str;
                  records,
                  rec:      integer;
                end;

      loglisttype = record
                  logcount,
                  currentlog: byte;
                  log:         array[1..MAXLOGS] of journaltype;
                  scan_threshold,
                  tune_rate,
                  filter,
                  bfo,
                  ssb,
                  reserved1,
                  reserved2,
                  reserved3,
                  reserved4:integer;
                  inc_amount:real;
                end;

      configtype = record
                  com_port:byte;
                  receiver_type:word;
                  has_map:boolean;
                  time_offset:integer;
                  compressed_print:string;
                end;

      receivertype = record
                  channel:    byte;
                  frequency:  real;
                  mode:       modetype;
                  bandwidth:  bandwidthtype;
                  agc:        agctype;
                  attenuator: attentype;
                end;

  var recdata:recdatatype;
      records:integer;     { total number of records }
      loglist:loglisttype;
      logbuf:file;
      rec:integer;
      receiverstat:receivertype;
      has_map:boolean;
      map:boolean;
      com_nrd:byte;
      radio_type:word;
      gmt_offset:integer;
      print_code:short_str;
      old_time_str:string;
      day_str,mon_str,year_str,time_str:string;
      old_time_stamp:longint;
      mark_array:array[0..MAXREC] of boolean;
      reccnt: array[1..MAXLOGS] of integer; {tally of records-deletions}


  procedure get_records(var records:integer);
  procedure put_records(records:word);
  procedure get_recdata(thelog:byte; var recdata:recdatatype);
  procedure put_recdata(thelog:byte; recdata:recdatatype);
  procedure time_date_stamp(var mon_str:string; var day_str:string;
                            var yr_str:string; var time_str:string;
                            force:boolean);
  procedure clear_log(var logdata:logtype);
  procedure open_log(var logbuf: file; thelog:byte; var rslt:integer);
  procedure get_log(var logbuf:file; var logdata:logtype; rec:integer);
  procedure put_log(var logbuf:file; logdata:logtype; rec:integer);
  procedure get_loglist(var loglist:loglisttype);
  procedure put_loglist(loglist:loglisttype);
  procedure get_reccnt;
  procedure put_reccnt;
  procedure write_com(port:byte; s:string);
  procedure comreadln(port:byte; var s:string);
  procedure remote_on;{ enable remote control of radio, get receiver status }
  procedure remote_off(dly:word);
  procedure toggle_remote;
  procedure check_status(ch1:char; var s:string);
  procedure set_freq(frequency:real);
  procedure set_mode(mode:modetype);
  procedure set_bandwidth(bandwidth:bandwidthtype);
  procedure set_agc(agc:agctype);
  procedure set_attenuator(attenuator:attentype);
  procedure set_bwc(s:string); { set NRD535 bandwidth control to "s" hz
                      requires leading "0"  -- eg set_bwc('0500') for 500Hz}
  procedure set_auto_tune(s:string); { set NRD535 autotuning;
                     '+' = Frequency up, '-' = Frequency down, '0' = stop }
  procedure set_all(channel:byte; attenuator:attentype;
                     bandwidth:bandwidthtype; mode:modetype; frequency:real;
                     agc:agctype);
  procedure clear_channels;
  procedure filter_bypass(bypass:boolean);
  procedure set_ssb(shift:boolean);
  procedure set_tune_rate(rate:integer);
  procedure set_bfo(bfo:integer);
  procedure information_mode_on;
  procedure information_mode_off;
  function  convert_s_to_db(s_reading:integer):real;
  procedure check_s_meter(var reading:real); { returns s_meter value
                                                   converted to dB }
  function  get_s_reading:integer; { returns raw s_meter reading }
  function  squelch_check:boolean; { TRUE if squelch open }

  implementation

  uses async, crt, dos, screen;

  const
      RECFILE     = 'REC';
      LOGLISTFILE = 'LOGLIST.DAT';
      RECCNTFILE  = 'RECCNT.DAT';

  var recbuf, recdatabuf, loglistbuf, reccntbuf:file;

  procedure open_records(var rslt:integer);
  var ch:char;
  begin
    assign(recbuf,PATH+RECFILE+'.DAT');
    repeat
      reset(recbuf,sizeof(records));
      rslt:=ioresult;
      if rslt = FILE_NOT_FOUND then
        begin
          rewrite(recbuf,sizeof(records));
          records:=0;
          blockwrite(recbuf,records,1);
          close(recbuf);
          reset(recbuf,sizeof(records));
          rslt:=ioresult;
        end;
      hndlerr(FALSE,ch,rslt);
    until (rslt = 0) or (ch = KEYINFO.ESCKEY);
  end;

  procedure get_records;
  var ch:char;
      rslt:integer;
  begin
    open_records(rslt);
    if rslt = 0 then
      begin
        blockread(recbuf,records,1);
        rslt:=ioresult;
        hndlerr(FALSE,ch,rslt);
      end;
    close(recbuf);
  end;

  procedure put_records;
  var ch:char;
      rslt:integer;
  begin
    open_records(rslt);
    if rslt = 0 then
      begin
        blockwrite(recbuf,records,1);
        rslt:=ioresult;
        hndlerr(FALSE,ch,rslt);
      end;
    close(recbuf);
  end;

  procedure open_recdata(thelog:byte; var rslt:integer);
  var ch:char;
      s:string;
      i:integer;
  begin
    str(thelog,s);
    if length(s) = 1 then s:='0' + s;
    assign(recdatabuf,PATH+RECDATAFILE + s + '.DAT');
    repeat
      reset(recdatabuf,sizeof(recdata));
      rslt:=ioresult;
      if rslt = FILE_NOT_FOUND then
        begin
          for i:=1 to MAXREC do with recdata do
            begin
              if i <= records then
                begin
                   recptr[i]:= i;
                   recstat[i]:= SHOW
                end
              else
                begin
                  recptr[i]:=0;
                  recstat[i]:=DELETED;
                end;
              m7000ptr[i]:=0;
            end;
          rewrite(recdatabuf,sizeof(recdata));
          blockwrite(recdatabuf,recdata,1);
          close(recdatabuf);
          reset(recdatabuf,sizeof(recdata));
          rslt:=ioresult;
        end;
      hndlerr(FALSE,ch,rslt);
    until (rslt = 0) or (ch = KEYINFO.ESCKEY);
  end;

  procedure get_recdata;

  var ch:char;
      rslt:integer;
  begin
    open_recdata(thelog,rslt);
    if rslt = 0 then
      begin
        blockread(recdatabuf,recdata,1);
        rslt:=ioresult;
        hndlerr(FALSE,ch,rslt);
      end;
    close(recdatabuf);
  end;

  procedure put_recdata;

  var ch:char;
      rslt:integer;
  begin
    open_recdata(thelog,rslt);
    if rslt = 0 then
      begin
        blockwrite(recdatabuf,recdata,1);
        rslt:=ioresult;
        hndlerr(FALSE,ch,rslt);
      end;
    close(recdatabuf);
  end;

  procedure time_date_stamp;
  { takes a time reading if time has changed or force = TRUE }
   var dy,yr,mo,dyofweek,hour,minute,sec,sec100:word;
       t:string;
       time_stamp:longint;
       thehour:integer;
  begin
    gettime(hour,minute,sec,sec100);
    time_stamp:=hour * 3600 + minute * 60 + sec;
    if not force then if (time_stamp = old_time_stamp) then exit;
    old_time_stamp:=time_stamp;
    getdate(yr,mo,dy,dyofweek);
    thehour:=hour;
    thehour:=thehour + gmt_offset;
    if thehour < 0 then thehour:=thehour + 24;
    if thehour >= 24 then
      begin
        thehour:=thehour - 24;
        dy:=dy + 1;
        if dy > 31 then { kludge date, doesn't allow for 30 day mo, etc }
          begin
            dy:=1;
            mo:=mo + 1;
            if mo > 12 then mo:=1;
          end;
      end;
    {init date to today's date in yymmdd format }
    str(yr,t); delete(t,1,2);
    yr_str:=t;
    str(mo,t);
    if length(t) < 2 then t:=concat('0',t);
    mon_str:=t;
    str(dy,t);
    if length(t) < 2 then t:=concat('0',t);
    day_str:=t;

    { init time in gmt }

    str(thehour,t);
    while length(t) < 2 do t:=concat('0',t);
    time_str:=t;
    str(minute,t);
    while length(t) < 2 do t:=concat('0',t);
    time_str:=concat(time_str,t);
    str(sec,t);
    while length(t) < 2 do t:=concat('0',t);
    time_str:=concat(time_str,t);
  end;


  procedure clear_log;

  var day_str,mon_str,yr_str,time_str,t:string;
  begin
    time_date_stamp(mon_str,day_str,yr_str,time_str,TRUE);
    with logdata do
      begin
        {init date to today's date in yymmdd format }
        date:=yr_str + mon_str + day_str;

        { init time in gmt }
        begin_time:=copy(time_str,1,4);
        end_time:=begin_time;
        frequency:= receiverstat.frequency;
        callsign:=  '';
        location:=  '';
        comment:=   '';
        mode:=      receiverstat.mode;
        bandwidth:= receiverstat.bandwidth;
        agc:=       receiverstat.agc;
        attenuator:=receiverstat.attenuator;
      end;
  end;

  procedure open_log;

  var ch:char;
      s:string;
  begin
    str(thelog,s);
    if length(s) = 1 then s:='0'+s;
    assign(logbuf,PATH+LOGFILE+s+'.DAT');
    reset(logbuf,sizeof(logtype));
    repeat
      rslt:=IORESULT;
      if rslt = FILE_NOT_FOUND then
        begin
          rewrite(logbuf,sizeof(logtype));
          rslt:=ioresult;
        end;
      hndlerr(FALSE,ch,rslt);
    until (rslt = 0) or (ch = KEYINFO.ESCKEY);
  end;

  procedure get_log;

  var rslt:integer;
      ch:char;
  begin
    seek(logbuf,rec - 1);
    blockread(logbuf,logdata,1);
    rslt:=ioresult;
    if rslt > 0 then
      begin
        hndlerr(TRUE,ch,rslt);
        clear_log(logdata);
        logdata.frequency:=0.0;
        logdata.begin_time:='';
        logdata.end_time:='';
      end;
  end;

  procedure put_log;

  var ch:char;
      rslt:integer;

  begin
    seek(logbuf,rec - 1);
    blockwrite(logbuf,logdata,1);
    rslt:=ioresult;
    if rslt > 0 then
      begin
        hndlerr(TRUE,ch,rslt);
      end;
  end;

  procedure open_loglist(var rslt:integer);
  var ch:char;
      i:integer;
  begin
    assign(loglistbuf,PATH+LOGLISTFILE);
    repeat
      reset(loglistbuf,sizeof(loglisttype));
      rslt:=ioresult;
      if rslt = FILE_NOT_FOUND then
        begin
          rewrite(loglistbuf,sizeof(loglisttype));
          loglist.logcount:=0;
          loglist.currentlog:=0;
          for i:=1 to MAXLOGS do
            begin
              loglist.log[i].logname:='';
              loglist.log[i].records:=0;
              loglist.log[i].rec:=0;
            end;
          loglist.scan_threshold:=123;
          loglist.tune_rate     :=9999; { set to "undefined" value }
          loglist.filter        :=9999; { set to "undefined" value }
          loglist.bfo           :=9999; { set to "undefined" value }
          loglist.ssb           :=9999; { set to "undefined" value }
          loglist.reserved1     :=0;
          loglist.reserved2     :=0;
          loglist.reserved3     :=0;
          loglist.reserved4     :=0;
          loglist.inc_amount    :=5.0;
          blockwrite(loglistbuf,loglist,1);
          close(loglistbuf);
          reset(loglistbuf,sizeof(loglisttype));
          rslt:=ioresult;
        end;
      hndlerr(FALSE,ch,rslt);
    until (rslt = 0) or (ch = KEYINFO.ESCKEY);
  end;

  procedure get_loglist;

  var ch:char;
      i, rslt:integer;
  begin
    open_loglist(rslt);
    if rslt = 0 then
      begin
        blockread(loglistbuf,loglist,1);
        if loglist.logcount = 0 then
          begin
            records:=0;
            rec:=1;
          end
        else
          begin
            records:=loglist.log[loglist.currentlog].records;
            rec:=    loglist.log[loglist.currentlog].rec;
          end;
        rslt:=ioresult;
        if rslt = 100
          then { assume they are upgrading to version 4.5 & patch file }
          begin
            for i:=16 to MAXLOGS do
              begin
                loglist.log[i].logname:='';
                loglist.log[i].records:=0;
                loglist.log[i].rec:=0;
              end;
            loglist.scan_threshold:=123;
            loglist.tune_rate     :=9999; { set to "undefined" value }
            loglist.filter        :=9999; { set to "undefined" value }
            loglist.bfo           :=9999; { set to "undefined" value }
            loglist.ssb           :=9999; { set to "undefined" value }
            loglist.reserved1     :=0;
            loglist.reserved2     :=0;
            loglist.reserved3     :=0;
            loglist.reserved4     :=0;
            loglist.inc_amount    :=5.0;
          end
        else hndlerr(FALSE,ch,rslt);
      end;
    close(loglistbuf);
  end;

  procedure put_loglist;

  var ch:char;
      rslt:integer;
  begin
    open_loglist(rslt);
    if rslt = 0 then
      begin
        blockwrite(loglistbuf,loglist,1);
        rslt:=ioresult;
        hndlerr(FALSE,ch,rslt);
      end;
    close(loglistbuf);
  end;

  procedure open_reccnt(var rslt:integer);
  var ch:char;
      i:integer;

    procedure count_records(i:integer);
    var k, deletions:integer;
    begin
      deletions:=0;
      if i <= loglist.logcount then
        begin
          get_recdata(i,recdata);
          for k:=1 to loglist.log[i].records do
            if recdata.recstat[k] = DELETED then inc(deletions);
          reccnt[i]:=loglist.log[i].records - deletions;
        end
      else reccnt[i]:=0;
    end;

  begin
    assign(reccntbuf,PATH+RECCNTFILE);
    repeat
      reset(reccntbuf,sizeof(reccnt));
      rslt:=ioresult;
      if rslt = FILE_NOT_FOUND then
        begin
          for i:=1 to MAXLOGS do count_records(i);
          rewrite(reccntbuf,sizeof(reccnt));
          blockwrite(reccntbuf,reccnt,1);
          close(reccntbuf);
          reset(reccntbuf,sizeof(reccnt));
          rslt:=ioresult;
        end;
      hndlerr(FALSE,ch,rslt);
    until (rslt = 0) or (ch = KEYINFO.ESCKEY);
  end;

  procedure get_reccnt;

  var ch:char;
      i, rslt:integer;
  begin
    open_reccnt(rslt);
    if rslt = 0 then
      begin
        blockread(reccntbuf,reccnt,1);
        hndlerr(FALSE,ch,rslt);
      end;
    close(reccntbuf);
  end;

  procedure put_reccnt;

  var ch:char;
      rslt:integer;
  begin
    open_reccnt(rslt);
    if rslt = 0 then
      begin
        blockwrite(reccntbuf,reccnt,1);
        rslt:=ioresult;
        hndlerr(FALSE,ch,rslt);
      end;
    close(reccntbuf);
  end;



  procedure init_com; { open com and set up for session }
  var baud:integer;
  begin
    if radio_type = 525 then baud:=1200 else
    if radio_type = 535 then baud:=4800;
    async_init;
    if COM_NRD = 0 then exit;
    if not async_open(COM_NRD,baud,'N',8,1) then
      begin
        writeln('Com port failure');
        halt;
      end;
  end;

  procedure write_com;
  begin
    if COM_NRD = 0 then exit;
    if radio_type = 535 then s:=s + chr(13);
    async_send_string(s);
  end;

  procedure comreadln;
  var ch:char;
      gotchar, done:boolean;
      error_count:integer;
  begin
    s:='';
    if COM_NRD = 0 then
      begin
        s:='000000';
        exit;
      end;
    done:=FALSE;
    error_count:=0;
    while not done do
      begin
        gotchar:=async_buffer_check(ch);
        if not gotchar then
          begin
            delay(20);
            inc(error_count);
          end
        else error_count:=0;
        if error_count > 40 then exit;
        done:=ch = #013;
        if not done and gotchar then s:=s + ch;
      end;
  end;

  procedure check_status; { used to read status of nrd 5x5 }
  var ch:char;
      s1:string;
      dummy:integer;

    procedure get_channel(s:string);
    begin
      val(s,receiverstat.channel,dummy);
    end;

    procedure get_mode(ch:char);
    begin
      case ch of
        '0': receiverstat.mode:=RTTY;
        '1': receiverstat.mode:=CW;
        '2': receiverstat.mode:=USB;
        '3': receiverstat.mode:=LSB;
        '4': receiverstat.mode:=AM;
        '5': receiverstat.mode:=FM;
        '6': receiverstat.mode:=FAX;
        '7': receiverstat.mode:=ECSS_USB;
        '8': receiverstat.mode:=ECSS_LSB;
        else receiverstat.mode:=FM; { error condition }
      end;
    end;

    procedure get_agc(ch:char);
    begin
      case ch of
        '0': receiverstat.agc:=SLOW;
        '1': receiverstat.agc:=FAST;
        '2': receiverstat.agc:=OFF;
        else receiverstat.agc:=OFF; { error condition }
      end;
    end;

    procedure get_attenuator(ch:char);
    begin
      case ch of
        '0': receiverstat.attenuator:=NO;
        '1': receiverstat.attenuator:=YES;
        else receiverstat.attenuator:=YES; { error condition }
      end;
    end;

    procedure get_bandwidth(ch:char);
    begin
      case ch of
        '0': receiverstat.bandwidth:=WIDE;
        '1': receiverstat.bandwidth:=INTER;
        '2': receiverstat.bandwidth:=NARR;
        '3': receiverstat.bandwidth:=AUX;
        else receiverstat.bandwidth:=AUX; { error condition }
      end;
    end;

    procedure get_frequency(s:string);
    begin
      val(s,receiverstat.frequency,dummy);
      receiverstat.frequency:=receiverstat.frequency / 100.0;
    end;

  begin
    if radio_type = 535 then
      begin
        comreadln(COM_NRD,s);
        s:=ch1+s;
        if (length(s) <> 13) or (s[1] <> 'I')  then exit;
        with receiverstat do
          begin
            get_attenuator(s[2]);
            get_bandwidth(s[3]);
            get_mode(s[4]);
            s1:=copy(s,5,8);
            val(s1,frequency,dummy);
            frequency:=frequency / 1000.0;
            get_agc(s[13]);
          end;
      end
    else { its an NRD525 }
      begin
        repeat
          comreadln(COM_NRD,s);
          s:=ch1 + s;
          case s[1] of
            'C':get_channel(copy(s,2,3));
            'D':get_mode(s[2]);
            'G':get_agc(s[2]);
            'A':get_attenuator(s[2]);
            'B':get_bandwidth(s[2]);
            'F':begin
                  delete(s,1,1);
                  get_frequency(s);
                end;
          end;
        until not async_buffer_check(ch1);
      end;
  end;

  procedure remote_on;
  { enable remote control of radio, get receiver status }
  const ECHO   = FALSE;
        STRLEN = 40;
  var s,s1:string;
      dummy:integer;
      ch:char;

    procedure initiate_remote; { send remote on string }
    begin
      write_com(COM_NRD,'H1');      { enable receiver remote mode }
    end;

    procedure check_serial_port(var s1:string);

    { This is a bizarre procedure to minimize program hanging in the event
      that the radio is left off or the serial port is selected to another
      device.  It uses an algorithm that assumes that no response to the
      remote command is an indication of error as is receiving a response
      back that does not begin in "C".  The latter can happen if the serial
      port is attached to a modem for example...}

      procedure check_response(ch:char; var s1:string);
      var ch1:char;
      begin
        if ch = 'C' then { all is (assumed) ok }
          begin
            s1:=' '; s1[1]:=ch;
          end
        else
          begin
            writeln(output,
  'Improper response from radio --- check connections and re-run program');
            writeln(output,'Hit any key to continue');
            repeat until keypressed;
            ch1:=readkey;
            halt;
          end;
      end;

    var ch, ch1:char;
    begin
      if com_nrd = 0 then { abort check }
        begin
          s1:='';
          exit;
        end;
      if not async_buffer_check(ch) then { something is wrong }
        begin
          delay(250); { just in case }
          if not async_buffer_check(ch) then { hung for sure }
            begin
              writeln(output,
              'No response from receiver.  Correct and hit any key');
              repeat until keypressed;
              ch1:=readkey;
              s1:='';
              initiate_remote;
            end
          else check_response(ch,s1);
        end
      else check_response(ch,s1);
    end;

  begin
    if com_nrd > 0 then while async_buffer_check(ch) do;
            { nrd sometimes leaves stuff behind }
    initiate_remote;
    if radio_type = 535 then exit
    else
      begin
        check_serial_port(s1); { will return 1st char in s1 }
        with receiverstat do
          begin
            { get channel }
            comreadln(COM_NRD,s);
            s:=s1 + s;
            delete(s,1,1); { remove "C" }
            val(s,channel,dummy);

            { get receiver mode }
            comreadln(COM_NRD,s);
            ch:=s[2];
            case ch of
              '0': mode:=RTTY;
              '1': mode:=CW;
              '2': mode:=USB;
              '3': mode:=LSB;
              '4': mode:=AM;
              '5': mode:=FM;
              '6': mode:=FAX;
              else mode:=FM; { error condition }
            end;

            { get agc setting }
            comreadln(COM_NRD,s);
            ch:=s[2];
            case ch of
              '0': agc:=SLOW;
              '1': agc:=FAST;
              '2': agc:=OFF;
              else agc:=OFF; { error condition }
            end;

            { get attenuator status }
            comreadln(COM_NRD,s);
            ch:=s[2];
            case ch of
              '0': attenuator:=NO;
              '1': attenuator:=YES;
              else attenuator:=YES; { error condition }
            end;

            { get bandwidth }
            comreadln(COM_NRD,s);
            ch:=s[2];
            case ch of
              '0': bandwidth:=WIDE;
              '1': bandwidth:=INTER;
              '2': bandwidth:=NARR;
              '3': bandwidth:=AUX;
              else bandwidth:=AUX; { error condition }
            end;

            { get frequency }
            comreadln(COM_NRD,s);
            delete(s,1,1); { get rid of "F" }
            val(s,frequency,dummy);
            frequency:=frequency / 100.0;
          end;
      end;
  end;

  procedure remote_off;
  begin
    write_com(COM_NRD,'H0'); { disable receiver remote mode }
    delay(dly);
  end;

  procedure toggle_remote; { unlock radio }
  var ch:char;
      s:string;
      count:integer;

    procedure do_toggle;
    begin
      write_com(COM_NRD,'I0');
      write_com(COM_NRD,'I1');
    end;

  begin
    if COM_NRD = 0 then { no connection to radio, initialize receiver stats}
      begin
        with receiverstat do
          begin
             mode:=RTTY;
             agc:=SLOW;
             attenuator:=NO;
             bandwidth:=WIDE;
             frequency:=0.0;
          end;
        exit;
      end;
    count:=0;
    repeat
      do_toggle;
      inc(count);
      while not async_buffer_check(ch) do;
      check_status(ch,s);
    until (receiverstat.frequency > 0) or (count > 3);
  end;

  procedure set_freq;
  var s:string;
  begin
    str(frequency:8:2,s);
    while s[1] = ' ' do delete(s,1,1);
    while length(s) < 9 do s:=concat('0',s);
    s:=concat('F',s);
    delete(s,8,1); { remove "." }
    if radio_type = 535 then
      begin
        delete(s,2,1);
        s:=s + '0';
      end;
    write_com(COM_NRD,s);
    if radio_type = 525 then comreadln(COM_NRD,s);
  end;

  procedure set_mode;
  var ch:char;
      s:string;
  begin
    case mode of
      RTTY:     s:='0';
      CW:       s:='1';
      USB:      s:='2';
      LSB:      s:='3';
      AM:       s:='4';
      FM:       s:='5';
      FAX:      s:='6';
      ECSS_USB: s:='7';
      ECSS_LSB: s:='8';
    end;
    s:=concat('D',s);
    write_com(COM_NRD,s);
    if radio_type = 525 then comreadln(COM_NRD,s);
  end;

  procedure set_bandwidth;
  var ch:char;
      s:string;
  begin
    case bandwidth of
      WIDE:  s:='0';
      INTER: s:='1';
      NARR:  s:='2';
      AUX:   s:='3';
    end;
    s:=concat('B',s);
    write_com(COM_NRD,s);
    if radio_type = 525 then comreadln(COM_NRD,s);
  end;

  procedure set_agc;
  var ch:char;
      s:string;
  begin
    case agc of
      SLOW:  s:='0';
      FAST:  s:='1';
      OFF:   s:='2';
    end;
    s:=concat('G',s);
    write_com(COM_NRD,s);
    comreadln(COM_NRD,s);
  end;

  procedure set_attenuator;
  var ch:char;
      s:string;
  begin
    case attenuator of
      NO:  s:='0';
      YES: s:='1';
    end;
    s:=concat('A',s);
    write_com(COM_NRD,s);
    comreadln(COM_NRD,s);
  end;

  procedure set_bwc;
  begin
    write_com(COM_NRD,'W'+ s); { set bw to "s" Hz }
  end;

  procedure set_auto_tune;
  begin
    write_com(COM_NRD,'Y' + s);
  end;

  procedure set_channel(channel:byte); { used with the NRD525 }
  var s:string;
  begin
    str(channel,s);
    while length(s) < 3 do s:='0' + s;
    write_com(COM_NRD,'C' + s);
    comreadln(COM_NRD,s);
  end;

  procedure set_all; { used to set all parameters for an NRD535 }
  var s,s1:string;
      ch:char;
  begin
    if radio_type = 535 then
      begin
        s:='S';
        str(channel,s1);
        while length(s1) < 3 do s1:='0' + s1;
        s:=s + s1;
        case attenuator of
          NO: s:=s + '0';
          YES:s:=s + '1';
        end;
        case bandwidth of
          WIDE:  s:=s + '0';
          INTER: s:=s + '1';
          NARR:  s:=s + '2';
          AUX:   s:=s + '3';
        end;
        case mode of
          RTTY:     s:=s + '0';
          CW:       s:=s + '1';
          USB:      s:=s + '2';
          LSB:      s:=s + '3';
          AM:       s:=s + '4';
          FM:       s:=s + '5';
          FAX:      s:=s + '6';
          ECSS_USB: s:=s + '7';
          ECSS_LSB: s:=s + '8';
        end;
        str(frequency:8:2,s1);
        while s1[1] = ' ' do delete(s1,1,1);
        while length(s1) < 8 do s1:=concat('0',s1);
        delete(s1,6,1); { remove "." }
        s:=s + s1 + '0';
        case agc of
          SLOW:  s:=s + '0';
          FAST:  s:=s + '1';
          OFF:   s:=s + '2';
        end;
        write_com(COM_NRD,s);
        delay(REMOTE_DLY);
      end
    else { handle channel load for a 525 }
      begin
        set_channel(channel);
        set_freq(frequency);
        set_mode(mode);
        set_bandwidth(bandwidth);
        set_agc(agc);
        set_attenuator(attenuator);
        delay(150);
        write_com(COM_NRD,'E1'); { write into the memory channel }
        delay(500);
      end;
  end;

  procedure clear_channels;
  begin
    write_com(COM_NRD,'Z1');
  end;

  procedure filter_bypass;
  begin
    if bypass then
      begin
        write_com(COM_NRD,'U10');
        loglist.filter:=0;
      end
    else
      begin
        write_com(COM_NRD,'U11');
        loglist.filter:=1;
      end;
  end;

  procedure set_ssb;
  begin
    if shift then
      begin
        write_com(COM_NRD,'U60');
        loglist.ssb:=0;
      end
    else
      begin
        write_com(COM_NRD,'U61');
        loglist.ssb:=1;
      end
  end;

  procedure set_tune_rate;
  var s:string;
  begin
    loglist.tune_rate:=rate;
    str(rate,s);
    write_com(COM_NRD,'V' + s);
  end;

  procedure set_bfo;
  var s,s1:string;
  begin
    loglist.bfo:=bfo;
    if bfo >= 0 then s:='+'
    else
      begin
         s:='-';
         bfo:=-bfo;
      end;
    str(bfo,s1);
    while length(s1) < 4 do s1:='0' + s1;
    write_com(COM_NRD,'U2' + s + s1);
  end;

  procedure information_mode_on;
  begin
    write_com(COM_NRD,'I1');
  end;

  procedure information_mode_off;
  begin
    write_com(COM_NRD,'I0'); { unlock radio }
  end;

  procedure read_s_meter;
  begin
    write_com(COM_NRD,'M');  { request s-meter reading }
  end;

  function get_s_reading;
  const last_reading:integer = 0;
  var s,s1:string;
      ch:char;
      dummy,reading:integer;
      freq:real;
  begin
    read_s_meter;  { request s-meter reading }
    comreadln(COM_NRD,s);
    ch:=s[1];
    delete(s,1,1);
    if ch = 'M' then
      begin
        val(s,reading,dummy);
        last_reading:=reading;
      end
    else
      begin
         reading:=last_reading;
         while async_buffer_check(ch) do delay(5);
      end;
    get_s_reading:=reading;
  end;

  function convert_s_to_db;
  var reading:real;
  begin
    case s_reading of { map to dB }
 250..255:reading:=0.00;
      249:reading:=0.01;
      248:reading:=0.02;
      247:reading:=0.03;
      246:reading:=0.04;
      245:reading:=0.05;
      244:reading:=0.06;
      243:reading:=0.07;
      242:reading:=0.08;
      241:reading:=0.09;
      240:reading:=0.10;
      239:reading:=0.11;
      238:reading:=0.12;
      237:reading:=0.13;
      236:reading:=0.14;
      235:reading:=0.15;
      234:reading:=0.16;
      233:reading:=0.17;
      232:reading:=0.18;
      231:reading:=0.19;
      230:reading:=0.20;
      229:reading:=0.21;
      228:reading:=0.22;
      227:reading:=0.23;
      226:reading:=0.24;
      225:reading:=0.25;
      224:reading:=0.26;
      223:reading:=0.27;
      222:reading:=0.28;
      221:reading:=0.29;
      220:reading:=0.30;
      219:reading:=0.31;
      218:reading:=0.32;
      217:reading:=0.33;
      216:reading:=0.34;
      215:reading:=0.35;
      214:reading:=0.36;
      213:reading:=0.37;
      212:reading:=0.38;
      211:reading:=0.39;
      210:reading:=0.40;
      209:reading:=0.41;
      208:reading:=0.42;
      207:reading:=0.43;
      206:reading:=0.44;
      205:reading:=0.45;
      204:reading:=0.46;
      203:reading:=0.47;
      202:reading:=0.48;
      201:reading:=0.49;
      200:reading:=0.50;
      199:reading:=0.51;
      198:reading:=0.52;
      197:reading:=0.53;
      196:reading:=0.54;
      195:reading:=0.55;
      194:reading:=0.56;
      193:reading:=0.57;
      192:reading:=0.58;
      191:reading:=0.59;
      190:reading:=0.60;
      189:reading:=0.61;
      188:reading:=0.62;
      187:reading:=0.63;
      186:reading:=0.64;
      185:reading:=0.65;
      184:reading:=0.66;
      183:reading:=0.67;
      182:reading:=0.68;
      181:reading:=0.69;
      180:reading:=0.70;
      179:reading:=0.71;
      178:reading:=0.72;
      177:reading:=0.73;
      176:reading:=0.74;
      175:reading:=0.75;
      174:reading:=0.76;
      173:reading:=0.77;
      172:reading:=0.78;
      171:reading:=0.79;
      170:reading:=0.80;
      169:reading:=0.81;
      168:reading:=0.82;
      167:reading:=0.83;
      166:reading:=0.84;
      165:reading:=0.85;
      164:reading:=0.86;
      163:reading:=0.87;
      162:reading:=0.88;
      161:reading:=0.89;
      160:reading:=0.90;
      159:reading:=0.91;
      158:reading:=0.92;
      157:reading:=0.93;
      156:reading:=0.94;
      155:reading:=0.95;
      154:reading:=0.96;
      153:reading:=0.97;
      152:reading:=0.98;
      151:reading:=0.99;
      150:reading:=1.0;
      149:reading:=1.1;
      148:reading:=1.2;
      147:reading:=1.3;
      146:reading:=1.4;
      145:reading:=1.5;
      144:reading:=1.6;
      143:reading:=1.7;
      142:reading:=1.8;
      141:reading:=1.9;
      140:reading:=2.0;
      139:reading:=2.1;
      138:reading:=2.2;
      137:reading:=2.3;
      136:reading:=2.4;
      135:reading:=2.5;
      134:reading:=2.6;
      133:reading:=2.7;
      132:reading:=2.8;
      131:reading:=2.9;
      130:reading:=3.0;
      129:reading:=3.1;
      128:reading:=3.3;
      127:reading:=3.4;
      126:reading:=3.6;
      125:reading:=3.7;
      124:reading:=3.9;
      123:reading:=4.0;
      122:reading:=4.1;
      121:reading:=4.3;
      120:reading:=4.4;
      119:reading:=4.6;
      118:reading:=4.7;
      117:reading:=4.9;
      116:reading:=5.0;
      115:reading:=5.2;
      114:reading:=5.4;
      113:reading:=5.6;
      112:reading:=5.8;
      111:reading:=6.0;
      110:reading:=6.2;
      109:reading:=6.5;
      108:reading:=6.7;
      107:reading:=7.0;
      106:reading:=7.2;
      105:reading:=7.4;
      104:reading:=7.6;
      103:reading:=7.8;
      102:reading:=8.0;
      101:reading:=8.3;
      100:reading:=8.6;
       99:reading:=9.0;
       98:reading:=9.2;
       97:reading:=9.3;
       96:reading:=9.5;
       95:reading:=9.7;
       94:reading:=9.8;
       93:reading:=9.9;
       92:reading:=10.0;
       91:reading:=11.0;
       90:reading:=13.0;
       89:reading:=15.0;
       88:reading:=17.0;
       87:reading:=20.0;
       86:reading:=22.0;
       85:reading:=23.0;
       84:reading:=25.0;
       83:reading:=26.0;
       82:reading:=27.0;
       81:reading:=28.0;
       80:reading:=30.0;
       79:reading:=32.0;
       78:reading:=35.0;
       77:reading:=36.0;
       76:reading:=38.0;
       75:reading:=40.0;
       74:reading:=42.0;
       73:reading:=45.0;
       72:reading:=48.0;
       71:reading:=52.0;
       70:reading:=55.0;
       69:reading:=57.0;
       68:reading:=58.0;
    0..67:reading:=60.0;
    end;
    convert_s_to_db:=reading;
  end;

  procedure check_s_meter;
  var s:string;
      s_reading, dummy:integer;
      ch:char;
      
  begin
    reading:=1;
    remote_on;
    while async_buffer_check(ch) do delay(5); { discard }
    s_reading:=get_s_reading;
    remote_off(0);
    information_mode_on;
    reading:=convert_s_to_db(s_reading);
  end;

  function squelch_check;
  var s:string;
  begin
    write_com(COM_NRD,'Q'); { request reading }
    comreadln(COM_NRD,s);
    squelch_check:=s = 'Q0';
  end;

  procedure init; { initialize port address, file path and existence of MAP }
  const CONFIG_PATH = 'CONFIG.DAT';
  var buf:file;
      configdat:configtype;
      num:integer;

    procedure read_config(var rslt:integer);
    var ch:char;
        ok,dummy:boolean;
        hour,minute,sec,sec100:word;
        s:string;
        error:boolean;
        cnvrt, i, ascval, err:integer;
    begin
      assign(buf,CONFIG_PATH);
      repeat
        reset(buf,sizeof(configdat));
        rslt:=ioresult;
        if rslt = 0 then { test contents of file }
          begin
            blockread(buf,configdat,1);
            rslt:=ioresult;
          end;
        if rslt <> 0 then
          begin
            repeat
              home;
              writea(LIGHTGREEN, FOREGROUND);
              writeln(output,
  'System could not find Config.dat file.  Please answer these questions');
              writeln(output,
  'about your system and Config.dat will be created for you...');
              entnum(0,10,num,ok,dummy,
                   'your com port (eg 1 for COM1:, 0 for demo):');
              if num in [0..4] then configdat.com_port:=num;
              if not ok then halt;
            until num in [0..4];
            writeln(output);
            writea(LIGHTGREEN, FOREGROUND);
            repeat
              write(output,'Enter receiver type (525 or 535):');
              readln(input,s);
            until (s = '525') or (s = '535');
            if s = '525' then configdat.receiver_type:=525
                         else configdat.receiver_type:=535;
            if configdat.receiver_type = 525 then
              begin
                write(output,'Do you have a KIWA MAP unit? (y=yes):');
                ch:=upcase(fetch);
                configdat.has_map:=ch ='Y';
                writeln(output);
              end
            else configdat.has_map:=FALSE;
            writeln(output,
'Now we will adjust the time offset so time is referenced to GMT.');
            writeln(output,
'Enter the hour only (not minutes) in GMT (eg if GMT = 16:47, enter 16.');
            entnum(0,17,num,ok,dummy,'the');
            if not ok then halt;
            gettime(hour,minute,sec,sec100);
            cnvrt:=hour;
            cnvrt:=num - cnvrt;
            configdat.time_offset:=cnvrt;
            writea(LIGHTGREEN, FOREGROUND);
            repeat
              gotoxy(1,20);
              call_crt(ERASEOS);
              writeln(output,
'Enter ascii character string to enable compressed print on your printer.');
              writeln(output,
'Separate each value with a space.  For example, Epson LQ-500 printers');
              writeln(output,
'use: 27 ',ord('g'),'.  Some Epson printers use: 15');
              writeln(output);
              write(output,
'Enter the appropriate values for your printer:');
              readln(input,s);
              configdat.compressed_print:='';
              error:=FALSE;
              ascval:=0;
              for i:=1 to length(s) do
                begin
                  if s[i] in ['0'..'9'] then
                    begin
                      val(s[i], num, err);
                      ascval:=ascval * 10 + num;
                      if ascval > 255 then
                        begin
                          error:=TRUE;
                          ascval:=0;
                        end;
                    end
                  else if (s[i] = ' ') then
                    begin
                      configdat.compressed_print:=
                          configdat.compressed_print + chr(ascval);
                      ascval:=0;
                    end
                  else error:=TRUE;
                end;
              if ascval > 0 then configdat.compressed_print:=
                configdat.compressed_print + chr(ascval);
              if error then
                begin
                   writeln(output,'Invalid ascii sequence detected');
                   write(output,'Hit any key to try again...');
                   ch:=fetch;
                end;
            until not error;
            rewrite(buf,sizeof(configdat));
            blockwrite(buf,configdat,1);
            close(buf);
            reset(buf,sizeof(configdat));
            rslt:=ioresult;
          end;
        hndlerr(FALSE,ch,rslt);
      until (rslt = 0) or (ch = KEYINFO.ESCKEY);
    end;

    procedure get_config;
    var ch:char;
        rslt:integer;
    begin
      read_config(rslt);
      com_nrd:=configdat.com_port;
      has_map:=configdat.has_map;
      radio_type:=configdat.receiver_type;
      gmt_offset:=configdat.time_offset;
      print_code:=configdat.compressed_print;
      close(buf);
    end;

  begin
    get_config;
  end;

begin
  init;
  old_time_str:='';
  old_time_stamp:=0;
  init_com;
  if has_map then
    begin
      remote_on;  { get receiver status to see if map is on }
      remote_off(0);
      map:=receiverstat.mode = AM;  { assume MAP in use if radio in AM }
    end
  else map:=FALSE;
  get_loglist(loglist);
  get_reccnt;
end.

