{$I-,V-}

unit nrdutil;

interface
  uses crt, dos, graph, screen, nrdio, async;

  type  prompttype    = (PAGE1, PAGE2, PAGE3, PAGE4);

  const
      LINES       = 25;
      CHARPERLINE = 80;
      BACKTAB     = chr(10);
      TAB         = chr(9);
      PAGEUP      = chr(3);
      PAGEDOWN    = chr(4);
      UP          = chr(15);
      DOWN        = chr(11);
      RIGHTARROW  = chr(21);
      LEFTARROW   = chr(7);
      CTRLPAGEUP  = chr(14);
      CTRLPAGEDN  = chr(16);
      HOMEKY      = chr(5);
      ENDKY       = chr(6);
      MAP_OFFSET  = 2.0; { frequency offset from center for sync det. }

  { Receiver window screen limits }
      REC_WIN_X_TOP    = 1;
      REC_WIN_Y_TOP    = 1;
      REC_WIN_X_BOTTOM = 79;
      REC_WIN_Y_BOTTOM = 5;

      REVISION         = '4.5';

  type inc_class_type = (FIXED_OFFSET, NEXT_FREQ, NEXT_MARKED, SCAN_MARKED);
                             { These affect the mode of the "+", "-" commands
                                 FIXED_OFFSET: bump by 5khz
                                 NEXT_FREQ:    bump by 5khz until > threshold
                                                   signal is found
                                 NEXT_MARKED:  bump to next marked freq until
                                                   signal is found
                                 SCAN_MARKED:  same as NEXT_MARKED but resume
                                                   scanning when signal drops
                                                   below threshold
                             }
  var
      { variables used to track display updates }
      mode_cnt:      integer;
      bandwidth_cnt: integer;
      agc_cnt:       integer;
      attenuator_cnt:integer;
      channel_cnt:   integer;
      frequency_cnt: integer;

      x_pos, y_pos:integer;
      prompt_num:prompttype;
      enable_s_meter:boolean;
      meter_reading:integer;
      graphmode:integer;
      display_page:integer;
      update_receiver_display:boolean;
      last_log:integer;  { used for hot keying between active and last log }
      last_log_data:logtype;{ used to copy data from last log to active log }
      i,cnt: integer;
      ch:char;
      s:string;
      oldstat:receivertype;
      logentry:logtype;
      min_mark,max_mark:word;
      mark_pair:boolean;
      mark_array:array[0..MAXREC] of boolean;
      rslt:integer;
      displayed_freq:array[1..LINES] of real;
      displayed_lines:integer;
      inc_class:inc_class_type;
      scan_threshold:integer; { detection threshold for scan function }

  procedure dot;
  procedure sort(var data:sort_array_type; var index:recarraytype;
		       start, points:integer);

  procedure editfield(x,y,fieldlen:integer; number:boolean;
                      var tabkey, backtabkey:boolean; var val:lstring);

  procedure draw_display_titles;
  procedure save_cursor_pos;
  procedure top_window;
  procedure bottom_window;
  procedure init_rec_window;
  procedure threshold_status(highlight:boolean);
  procedure status_window;
  procedure inc_class_status(highlight:boolean);
  function  precess(var rec:integer; cnt:integer):boolean;
  function  bandwidth_to_str(bandwidth:bandwidthtype):short_str;
  function  mode_to_str(mode:modetype):short_str;
  function  agc_to_str(agc:agctype):short_str;
  procedure show_log(rec:integer; refresh_screen,highlight:boolean);
  { refresh_screen = TRUE; paint entire screen with log entries
                   = FALSE then highlight line if indicated }
  procedure write_prompt(s:string);
  procedure cmd_prompt(prompt_num:prompttype);
  procedure timed_s_meter;
  procedure clear_meter;
  function  tick:boolean; { true if a time tick has occurred }
  procedure show_receiver;
  procedure inc_mode;
  procedure dec_mode;
  procedure inc_bandwidth;
  procedure dec_bandwidth;
  function  upcasestr(str:string):string;
  procedure program_radio(log_entry:logtype);
  procedure sync_loglist;
  procedure do_mark_range;
  procedure do_mark_line;
  procedure do_unmark_line;
  procedure do_unmark_all(display:boolean);
  procedure do_move;
  procedure do_overall;
  procedure do_undelete;
  procedure do_sort(auto:boolean);
  procedure update_time_status;
  procedure new_log(lognum:byte; var rslt:integer);
  procedure do_alternate;
  procedure do_search(again:boolean);
  procedure kill_clock(highlight:boolean);
  procedure do_threshold;
  procedure do_inc_class;
  procedure do_page;
  procedure do_tab;
  procedure do_backtab;
  procedure do_edit; { edit field cursor is on }
  procedure do_delete;
  procedure do_log;
  procedure do_tune;
  function  find_rec(rec:integer; freq:real):integer;
  procedure find_freq;
  procedure do_freq;
  procedure do_kiwa; { different mode if KIWA MAP unit in use }
  procedure do_confirm;
  procedure do_write; { as in dudley... }
  procedure do_pageup(cnt:byte);
  procedure do_pagedown(cnt:byte);
  procedure do_up;
  procedure do_down;
  procedure do_right;
  procedure do_left;
  procedure do_home;
  procedure do_end;
  procedure call_do_help;
  procedure goodbye;
  procedure init_crt;

implementation

  var   last_channel: integer;  { last channel written by the move command }

  procedure dot;
  begin
    write(output,'.');
  end;

  procedure sort;
  (* THIS PROCEDURE IMPLEMENTS 'QUICKSORT' BY C.A.R. HOARE. THIS N*LOG(N) *)
  (* ALGORITHM IS A PARTITION EXCHANGE SORT AND IS DOCUMENTED IN THE 7-80 *)
  (* ISSUE OF 'MICRO'.                                                    *)

  TYPE STACKTYPE = RECORD
       UPPER:1..MAXREC; (*STORAGE FOR UPPER SEARCH RANGE*)
       LOWER:1..MAXREC	(*   "     "  LOWER    "     "  *)
     END;

  VAR P,Q (*CURRENT LOWER & UPPER INDEX BOUNDS TO BE SORTED. DATA[P] IS
	      USED AS A COMPARISON KEY IN THE SORTING PROCESS.		   *)
       ,I   (*STARTS AT P & IS INCREMENTED UNTIL DATA[I]>=DATA[P].	   *)
       ,J   (*STARTS AT Q & IS DECREMENTED UNTIL DATA[I]<=DATA[P].	   *)
       ,STACKPTR:INTEGER;
	STACK:PACKED ARRAY[1..MAXREC] OF STACKTYPE; (*SEARCH RANGE STORAGE.*)
	TEMPDATA,VALUE (*DATA[P]*):short_str;
	TEMPINDEX:1..MAXREC;
  BEGIN
    P:=start; Q:=POINTS + start - 1; STACKPTR:=0;
    REPEAT
      WHILE P < Q DO
	  BEGIN
	    VALUE:=DATA[P]; I:=P; J:=Q + 1;
	    REPEAT
	      REPEAT J:=J - 1 UNTIL DATA[J]<=VALUE;
	      REPEAT inc(I)   UNTIL DATA[I]>=VALUE;
	      IF J > I THEN
		BEGIN
		  TEMPDATA:=DATA[I]; TEMPINDEX:=INDEX[I];
		  DATA[I] :=DATA[J]; INDEX[I] :=INDEX[J];
		  DATA[J] :=TEMPDATA;INDEX[J] :=TEMPINDEX
		END;
	    UNTIL J <= I;
	    TEMPINDEX:=INDEX[P];
	    DATA[P]:=DATA[J]; INDEX[P]:=INDEX[J];
	    DATA[J]:=VALUE;   INDEX[J]:=TEMPINDEX;
	    inc(STACKPTR);
	    IF J - P < Q - J
	      THEN
		WITH STACK[STACKPTR] DO
		  BEGIN LOWER:=J + 1; UPPER:=Q; Q:=J - 1
		  END
	      ELSE
		WITH STACK[STACKPTR] DO
		  BEGIN LOWER:=P; UPPER:=J - 1; P:=J + 1
		  END;
	  END; (*WHILE*)
        dot;
	IF STACKPTR > 0 THEN (*GRAB NEW SEARCH RANGE OFF STACK*)
	  WITH STACK[STACKPTR] DO BEGIN Q:=UPPER; P:=LOWER END;
	STACKPTR:=STACKPTR-1
    UNTIL STACKPTR < 0 (*EMPTY STACK*)
  END;

  procedure draw_display_titles;
  begin
    top_window;
    writea(LIGHTGRAY,FOREGROUND);
    writea(BLACK,BACKGROUND);
    gotoxy(1,5);
    clreol;
    case display_page of
      1: begin
           gotoxy(2,5);
           write(output,'Num');
           gotoxy(7,5);
           write(output,'Date');
           gotoxy(13,5);
           write(output,'Strt');
           gotoxy(18,5);
           write(output,'End');
           gotoxy(24,5);
           write(output,'Freq');
           gotoxy(32,5);
           write(output,'Station ID');
           gotoxy(52,5);
           write(output,'Location');
         end;
      2: begin
           gotoxy(3,5);
           write(output,'Freq');
           gotoxy(11,5);
           write(output,'Comment');
         end;
      3: begin
           gotoxy(2,5);
           write(output,'Num');
           gotoxy(7,5);
           write(output,'Date');
           gotoxy(13,5);
           write(output,'Strt');
           gotoxy(18,5);
           write(output,'End');
           gotoxy(24,5);
           write(output,'Freq');
           gotoxy(32,5);
           write(output,'Mode');
           gotoxy(39,5);
           write(output,'BW');
           gotoxy(43,5);
           write(output,'AGC');
           gotoxy(48,5);
           write(output,'Attn');
         end;
    end;
  end;

  procedure editfield;

  { parameters: x,y	 = cursor position
		fieldlen = allowable length for field
		number	 = flag that if true restricts the keys usable
		val	 = return string }

  var ptr,i:integer;
      ch:char;
      errflg,flag,insert_mode:boolean;
      oldval:string[255];

    procedure show_line(x,y,fieldlen:integer; val:lstring; edit:boolean);
    var i:integer;
        lf_bracket,rt_bracket:char;
    begin { show_line }
      gotoxy(x,y + 1);
      if edit then { they are editing this line }
        begin
	  lf_bracket:='[';
          rt_bracket:=']';
	  writea(RED,BACKGROUND);
	  writea(BLACK,FOREGROUND);
        end
      else
        begin
	  writea(CYAN,BACKGROUND);
	  writea(BLACK,FOREGROUND);
          lf_bracket:=' ';
	  rt_bracket:=' ';
        end;
      write(lf_bracket,val);
      gotoxy(x + 1 + fieldlen,y + 1);
      write(rt_bracket);
    end; { show_line }

    procedure blankfill(fieldlen:integer; var val:lstring);
    begin
      while length(val) < fieldlen do val:=concat(val,' ');
    end;


    procedure get_normal;

    { fetch normal character and display it.  Handle field overflow and
      insert_mode }

    var i:integer;
	s:string[1];
    begin
      if ptr < fieldlen
	then
	  begin
	    if ((number) and (ch in ['0'..'9',' ','.']))
		or ((ptr = 0) and (ch = '-')) or not number then
	      begin { all's well }
		if not insert_mode then
		  begin
		    write(ch); { echo character to screen }
		    ptr:=ptr + 1;
		    val[ptr]:=ch
		  end
		else { handle insert mode }
		  begin
		    inc(ptr);
		    for i:=fieldlen downto ptr do val[i]:=val[i - 1];
		    val[ptr]:=ch;
		    for i:=ptr to fieldlen do write(val[i]);
		    gotoxy(x + ptr + 1,y + 1);
		  end
	      end
	  end
    end; { get_normal }

    procedure do_backspace;
    begin
      if ptr > 0 then { it's ok to backspace }
	begin
	  ptr:=ptr - 1;
	  gotoxy(x + ptr + 1,y + 1)
	end
    end;

    procedure do_forwardspace;
    begin
      if ptr < fieldlen then { its ok to forward space }
	begin
	  inc(ptr);
	  gotoxy(x + ptr + 1,y + 1)
	end
    end;

    procedure do_del;
    var i:integer;
    begin
      for i:=ptr + 1 to fieldlen - 1 do val[i]:=val[i + 1];
      val[fieldlen]:=' ';
      for i:=ptr + 1 to fieldlen do write(val[i]);
      gotoxy(x + ptr + 1,y + 1);
    end;

    procedure do_rub;
    var i:integer;
    begin
      if ptr > 0 then ptr:=ptr - 1;
      gotoxy(x + ptr + 1,y + 1);
      do_del;
    end;

    procedure toggle_insert;
    begin
      insert_mode:=not insert_mode;
   end;

   procedure do_home;
   begin
     ptr:=0;
     gotoxy(x + ptr + 1,y + 1)
   end;

   procedure do_end;
   begin
     ptr:=fieldlen;
     while (ptr > 0) and (val[ptr] = ' ') do
       ptr:=ptr - 1;
     gotoxy(x + ptr + 1,y + 1)
   end;

   procedure do_tab;
   begin
     tabkey:=TRUE;
     ch:=chr(13);
   end;

   procedure do_backtab;
   begin
     backtabkey:=TRUE;
     ch:=chr(13);
   end;

  begin { editfield }
    insert_mode:=FALSE;
    tabkey:=FALSE;
    backtabkey:=FALSE;
    if number and (val = '-0.00') then val:='0.00';
    oldval:=val; { save copy in case they abort }
    if length(val) > fieldlen then val:=copy(val,1,fieldlen);
    blankfill(fieldlen,val);
    ptr:=0;
    show_line(x,y,fieldlen,val,TRUE);
    gotoxy(x + 1,y + 1);
    repeat
      ch:=fetch;
      if (ch <> chr(13)) and (ord(ch) >= ord(' ')) then get_normal
      else if ch = keyinfo.bskey   then do_backspace
      else if ch = keyinfo.fskey   then do_forwardspace
      else if ch = keyinfo.delkey  then do_del
      else if ch = keyinfo.rubkey  then do_rub
      else if ch = keyinfo.inskey  then toggle_insert
      else if ch = keyinfo.homekey then do_home
      else if ch = keyinfo.endkey  then do_end
      else if ch = keyinfo.tabkey  then do_tab
      else if ch = keyinfo.backtabkey then do_backtab
    until (ch = chr(13)) or (ch = keyinfo.esckey);

    if number then { strip off trailing blanks }
      begin
	ptr:=length(val);
	flag:=TRUE;
	while (ptr > 0) and flag do
	  begin
	    flag:=val[ptr] = ' ';
	    if flag then delete(val,ptr,1);
	    ptr:=ptr - 1;
	  end;
      end;
    show_line(x,y,fieldlen,val,FALSE);
  end; { editfield }

  procedure save_cursor_pos;
  begin
    x_pos:=wherex; y_pos:=wherey;
  end;

  procedure top_window;
  begin
    window(REC_WIN_X_TOP,REC_WIN_Y_TOP,REC_WIN_X_BOTTOM+1,REC_WIN_Y_BOTTOM);
  end;

  procedure bottom_window;
  begin
    window(1,REC_WIN_Y_BOTTOM + 1, 80,24);
    gotoxy(x_pos,y_pos);
  end;

  procedure init_rec_window;

    procedure draw_box(width,hieght:integer);
    const TL = chr(201);
          TR = chr(187);
          BL = chr(200);
          BR = chr(188);
          HZ = chr(205);
          VT = chr(186);

    var i:integer;

      procedure draw_horiz;
      var i:integer;
      begin
        for i:=1 to width - 2 do write(output,HZ);
      end;

    begin
      { draw top }
      gotoxy(1,2);
      write(output,TL);
      draw_horiz;
      write(output,TR);

      { draw sides }
      for i:=1 to hieght - 2 do
        begin
          gotoxy(1,i + 2);      write(output,VT);
          gotoxy(width, i + 2); write(output,VT);
        end;

      { draw bottom }
      gotoxy(1,hieght + 1);
      write(output,BL);
      draw_horiz;
      write(output,BR);
    end;

  begin
    top_window;
    writea(BLACK,BACKGROUND);
    home;
    cmd_prompt(prompt_num);
    writea(BROWN,FOREGROUND);
    draw_box(REC_WIN_X_BOTTOM - REC_WIN_X_TOP + 1,
             REC_WIN_Y_BOTTOM - REC_WIN_Y_TOP - 1) ;
    gotoxy(30,2);
    writea(LIGHTGRAY,FOREGROUND);
    if radio_type = 525
       then write(output,'NRD 525 Status')
       else write(output,'NRD 535 Status');
    gotoxy(3,3);
    write(output,'Mode:');
    gotoxy(15,3);
    write(output,'BW:');
    gotoxy(29,3);
    write(output,'AGC:');
    if radio_type = 525 then
      begin
        gotoxy(42,3);
        write(output,'Ch:');
      end;
    gotoxy(54,3);
    write(output,'Freq:');
    gotoxy(68,3);
    write(output,' khz');
    display_page:=1;
    writea(LIGHTGRAY,FOREGROUND);
    draw_display_titles;
    writea(LIGHTGRAY,FOREGROUND);
  end;

  procedure inc_class_status;
  var ch:char;
  begin
    if radio_type = 525 then exit;
    gotoxy(46,1);
    if highlight
      then writea(RED,FOREGROUND)
      else writea(CYAN, FOREGROUND);
    case inc_class of
      FIXED_OFFSET:ch:='F';
      NEXT_FREQ:   ch:='N';
      NEXT_MARKED: ch:='M';
      SCAN_MARKED: ch:='S';
    end;
    write(output,ch);
    writea(LIGHTGRAY, FOREGROUND);
  end;

  procedure threshold_status;
  var thresh:real;
  begin
    gotoxy(58,1);
    if highlight
      then writea(RED,FOREGROUND)
      else writea(CYAN,FOREGROUND);
    thresh:=convert_s_to_db(scan_threshold);
    if thresh >= 10.0 then write(output,thresh:4:1)
    else if thresh >= 1.0 then write(output,thresh:3:1,' ')
    else write(output,thresh:4:2);
    writea(LIGHTGRAY, FOREGROUND);
  end;

  procedure status_window;
  begin
    window(1,25,80,25);
    gotoxy(1,1);
    clreol;
    writea(BLACK,BACKGROUND);
    writea(LIGHTGRAY, FOREGROUND);
    write(output,'Active Log:');
    writea(CYAN, FOREGROUND);
    if loglist.currentlog = 0
      then write(output,'<None>')
      else write(output,loglist.log[loglist.currentlog].logname);
    if last_log <> 0 then
      begin
        writea(LIGHTGRAY, FOREGROUND);
        gotoxy(19,1);
        write(output,'Inactive Log:');
        writea(CYAN, FOREGROUND);
        write(output,loglist.log[last_log].logname);
      end;
    writea(LIGHTGRAY, FOREGROUND);
    if radio_type = 535 then { display scan information }
      begin
        gotoxy(39,1);
        write(output,'+-Mode: ');
        inc_class_status(FALSE);
        write(output,' Threshold:');
        threshold_status(FALSE);
      end;
  end;

  function mode_to_str;
  var s:short_str;
  begin
    case mode of
      RTTY:     s:='RTTY';
      CW:       s:=' CW';
      USB:      s:='USB';
      LSB:      s:='LSB';
      AM:       s:=' AM';
      FM:       s:=' FM';
      FAX:      s:='FAX';
      ECSS_USB: s:='ECSSu';
      ECSS_LSB: s:='ECSSl';
    end;
    mode_to_str:=s;
  end;

  function bandwidth_to_str;
  var s:short_str;
  begin
    case bandwidth of
      NARR:  s:=' NARR';
      INTER: s:='INTER';
      WIDE:  s:=' WIDE';
      AUX:   s:=' AUX';
    end;
    bandwidth_to_str:=s;
  end;

  function agc_to_str;
  var s:short_str;
  begin
    case agc of
      FAST: s:='FAST';
      SLOW: s:='SLOW';
      OFF:  s:=' OFF';
    end;
    agc_to_str:=s;
  end;

  procedure show_log_line(logdata:logtype;rec,i:word);

    procedure show_line1;
    begin
      write(output,rec:4);
      with logdata do
        begin
          gotoxy(6,i);
          write(output,date);
          gotoxy(13,i);
          write(output,begin_time);
          gotoxy(18,i);
          write(output,end_time);
          gotoxy(23,i);
          write(output,frequency:8:2);
          gotoxy(32,i);
          write(output,callsign);
          gotoxy(52,i);
          write(output,location);
        end;
    end;

    procedure show_line2;
    begin
      with logdata do
        begin
          gotoxy(2,i);
          write(output,frequency:8:2);
          gotoxy(11,i);
          write(output,comment);
        end;
    end;

    procedure show_line3;
    var s:short_str;
    begin
      write(output,rec:4);
      with logdata do
        begin
          gotoxy(6,i);
          write(output,date);
          gotoxy(13,i);
          write(output,begin_time);
          gotoxy(18,i);
          write(output,end_time);
          gotoxy(23,i);
          write(output,frequency:8:2);
          gotoxy(32,i);
          s:=mode_to_str(mode);
          write(output,s);
          gotoxy(37,i);
          s:=bandwidth_to_str(bandwidth);
          write(output,s);
          gotoxy(43,i);
          s:=agc_to_str(agc);
          write(output,s);
          gotoxy(49,i);
          case attenuator of
            YES: write(output,'ON');
            NO:  write(output,'OFF');
          end;
        end;
    end;

  begin
    gotoxy(1,i); clreol;
    case display_page of
      1: show_line1;
      2: show_line2;
      3: show_line3;
    end;
  end;

  function precess;
  { skip cnt displayed records; return TRUE is not past eof }
  var i:integer;
  begin
    for i:=1 to cnt do
      begin
        rec:=rec + 1;
        while (rec < records)
          and (recdata.recstat[recdata.recptr[rec]] <> SHOW) do
            rec:=rec + 1;
      end;
    if rec > records then rec:=records;
    precess:=recdata.recstat[recdata.recptr[rec]] = SHOW;
  end;

  procedure show_log;
  { refresh_screen = TRUE; paint entire screen with log entries
                   = FALSE then highlight line if indicated }
  var i,j,x_temp,y_temp:integer;
      logdata:logtype;

    procedure welcome; { used when no databases yet exist }
    begin
      gotoxy(1,8);
      writeln(output,
'Welcome to NRD!  You must first create a database to really use this');
      writeln(output,
'program.  To import a sample database,  Hit the following keys: "J" (for');
      writeln(output,
'"Journal"), "I" (for "Import"), "A" for "ASCII".  You will be prompted to');
      writeln(output,
'enter the database name to import.  Enter SAMPLE.  You will next be asked');
      writeln(output,
'to enter the name for the new log.  Enter SAMPLE.  Next select this');
      writeln(output,
'new database with the "S" command.  This will return you to the main');
      writeln(output,
'prompt.  See NRD.DOC for instructions on using NRD.');
    end;

  begin
    writea(CYAN,FOREGROUND); writea(BLACK,BACKGROUND);
    i:=0; j:=rec - 1;
    y_temp:=wherey; { used to highlight cursor line }
    x_temp:=wherex;
    if refresh_screen then home;

    { special case for no logs yet... }
    if loglist.logcount = 0 then welcome;
        
    while (i < LINES - REC_WIN_Y_BOTTOM - 1) and (j < records) do
      begin
        inc(i);
        if precess(j,1) then
          begin
            get_log(logbuf,logdata,recdata.recptr[j]);
            displayed_freq[i]:=logdata.frequency;
            if mark_array[recdata.recptr[j]] then
              begin
                writea(BLACK,FOREGROUND);
                if (i = y_temp) and highlight
                  then
                    begin
                      if graphmode = CGAHI
                        then writea(GREEN,BACKGROUND)
                        else writea(RED,BACKGROUND)
                    end
                  else writea(YELLOW,BACKGROUND);
                show_log_line(logdata,j,i);
                writea(BLACK,BACKGROUND);
                writea(CYAN,FOREGROUND);
              end
            else if (i = y_temp) and highlight then
              begin
                writea(BLACK,FOREGROUND);
                writea(CYAN,BACKGROUND);
                show_log_line(logdata,j,i);
                writea(BLACK,BACKGROUND);
                writea(CYAN,FOREGROUND);
              end
            else if refresh_screen or ((i = y_temp) and not highlight)
              then show_log_line(logdata,j,i);
          end;
      end;
    displayed_lines:=i;
    gotoxy(x_temp,y_temp);
  end;

  procedure write_prompt;

  { write green prompt at top of top window; leave with light gray foreground
    and in top window }

  begin
    top_window;
    writea(GREEN,FOREGROUND);
    writea(BLACK,BACKGROUND);
    gotoxy(1,1);
    clreol;
    write(output,s);
    writea(LIGHTGRAY,FOREGROUND);
  end;

  procedure cmd_prompt;
  const T0 = 'NRD'+REVISION;
        T1 = ': L(og, C(onfirm, S(ort, E(dit, T(une, P(age, H(elp, ';
        T2 = 'Q(uit [/]';
        T3 = 's-meteR, ';
        T4 = ': O(verall, CTL-S(earch, CTL-A(gain';
        T5 = ', CTL-T(hreshold, I(nc mode';
  begin
    case prompt_num of
      PAGE1: if has_map then write_prompt(T0+T1+'K(iwa, '+T2)
             else if radio_type = 525 then write_prompt(T0+T1+T2)
             else write_prompt(T0+T1+T3+T2);
      PAGE2:write_prompt(T0+
': D(elete, uN(delete, F(requency set, J(ournal, A(lternate, W(rite [/]');
      PAGE3:write_prompt(T0+
': M(ark range, CTL-M(ark line, U(nmark, CTL-unmark L(ine, moV(e [/]');
      PAGE4:if radio_type = 535
              then write_prompt(T0+T4+T5+' [/]')
              else write_prompt(T0+T4+' [/]');
    end;
  end;

  procedure clr_display_variables;
  begin
    mode_cnt      :=0;
    bandwidth_cnt :=0;
    agc_cnt       :=0;
    attenuator_cnt:=0;
    channel_cnt   :=0;
    frequency_cnt :=0;
  end;

  procedure init_crt;
  begin
    writea(BLACK,BACKGROUND);
    window(1,1,80,25);
    home;
    clr_display_variables;
    init_rec_window;
    update_receiver_display:=TRUE;
    status_window;
    bottom_window;
  end;

  procedure do_help;
  var ch:char;

    procedure help_commands;
    var ch:char;
    begin
      write_prompt('Help -- Commands  <Hit any key to return>');
      window(1,2,80,25);
      home;
      writeln(output,'Command Summary:');
      writeln(output,
'/: Toggles command menu (commands from all menus are always active)');
      writeln(output,
'L: Log (creates new entry in log, receiver contents automatically included)'
          );
      writeln(output,
'C: Confirm (updates time, date, receiver contents for highlighted entry)');
      writeln(output,
'S: Sort data base with 2 sort keys');
      writeln(output,
'E: Edit field where cursor is located.  Hit ENTER or a TAB key when done');
      writeln(output,
'T: Tune receiver to highlighted entry.  Updates all receiver parameters');
      writeln(output,'F: set Frequency from the keyboard');
      writeln(output,
'P: Page right (the contents for an entry span 3 pages, faster than tabs)');
writeln(output,'D: Deletes a log entry');
      writeln(output,
'N: uNdeletes a log entry (logging new stations reuses deleted space)');
     writeln(output,
'M: Mark range for writing or moving to other datalogs (see Journal)');
      writeln(output,'CTL-M: Mark single line (Hold CTRL and m keys)');
      writeln(output,'CTL-L: unmark single Line');
      writeln(output,'U: Unmark lines');
      writeln(output,'CTL-S: Search for a pattern in a selected data field');
      writeln(output,'CTL-A: search for pattern Again');
      writeln(output,'A: Alternates between Active and Inactive Logs');
      writeln(output,'W: Write entry from Inactive Log to Active Log');
      writeln(output,'V: moVe marked lines to the radio''s channels');
      write(output,  'Q: Quits the program');
      ch:=fetch;
      home;
    end;

    procedure help_more;
    var ch:char;
    begin
      write_prompt('Help -- More Commands  <Hit any key to return>');
      window(1,2,80,25);
      home;
      writeln(output,
'J: Journal: allows you to select other data logs and do things with them.');
      writeln(output,
'         I keep multiple logs -- a music log for stations that play');
      writeln(output,
'         interesting music and target logs for areas I''m trying to get.');
      writeln(output,
'         Target logs allow you to scan what''s there VERY quickly.');
      writeln(output,
'         The Write command allows marked areas to be moved from'
          );
      writeln(output,
'         one database to another; like when you find one of those targets!'
          );
      writeln(output,
'         Move is like a write but deletes the marked entry.  Print writes');
      writeln(output,
'         the selected database to your printer.  Facilities are included');
      writeln(output,
'         to export and import logs in ASCII format as well as to import');
      writeln(output,
'         Tom Sundstrom''s English Language SW Broadcast Schedules.');
      writeln(output,
'         You can order these from Tom (609) 859-2447.'
          );
      if has_map then
        begin
          writeln(output,
'K: KIWA (this only applies if you have a KIWA Map unit)  "K" toggles KIWA');
          writeln(output,
'         mode.  When enabled, the receiver is placed in AM and the radio');
          writeln(output,
'         is detuned a couple of Khz for good fidelity.  Stations logged or'
          );
          writeln(output,
'         confirmed will be rounded to the nearest 5 Khz.  Disabling puts');
          writeln(output,
'         the radio in ECS mode with the appropriate sideband selected based'
          );
          writeln(output,
'         on the offset.  The MAP unit provides synchronous detection for a'
          );
          writeln(output,
'         525 and was described in Guy Atkin''s 9/90 NASWA article p.18.  To'
          );
          writeln(output,
'         tell the program you have a MAP, delete the config.dat file and');
          write(output,
'         rerun program.');
        end;
      if radio_type = 535 then
        begin
          writeln(output,
'R: s-meteR: Toggles mode of periodically updating computer S-Meter display.'
          );
          writeln(output,
'         Unfortunately, reading the S-Meter can cause annoying synthesizer'
          );
          writeln(output,
'         re-locking noise in LSB, USB, CW, and RTTY modes.  Use this mode');
          writeln(output,
'         to disable this when it bothers you');
        end;
      ch:=fetch;
    end;

    procedure help_receiver;
    var ch:char;
    begin
      write_prompt('Help -- Receiver display  <Hit any key to return>');
      window(1,2,80,25);
      home;
      gotoxy(1,3);
      writeln(output,
'The box labled "NRD',radio_type
,' Status" contains the last sampled receiver status.'
          );
      writeln(output,
'Mostly, this is self-explanatory.  BW = Bandwidth, Freq is the receiver'
          );
      writeln(output,
'frequency.  If the attenuator is active, you will see "ATT" at the right'
          );
      writeln(output,
'of the screen.  If you said you had a KIWA Map unit and it is active, you'
          );
      writeln(output,
'will see a "K" at the far right of the status box.  The KIWA features are'
          );
      writeln(output,
'described in the commands section.  If you don''t have one, don''t worry'
          );
      writeln(output,
'about this.  Normally, the status is displayed in CYAN unless something'
          );
      writeln(output,
'has changed since the last sample.  Changes are displayed in RED.  To cause'
          );
      writeln(output,
'the receiver status to be sampled, hit any non-command key (like space).'
          );
      writeln(output,
'This approach to updating the display was chosen deliberately to keep the'
          );
      write(output,
'radio "unlocked" so you can punch up commands on the radio.');
      ch:=fetch;
    end;

    procedure help_other;
    var ch:char;
    begin
      write_prompt('Help -- Other <Hit any key to return>');
      window(1,2,80,25);
      home;
      gotoxy(1,3);
      writeln(output,'Cursor commands:');
      writeln(output);
      writeln(output,
'     HOME = move to top of database. END = move to the bottom.'
          );
      writeln(output,
'     TABS: forward and reverse TABS.  ARROW keys move up/down/right/left.'
          );
      writeln(output,
'     PAGE keys page the database.  Control-PAGE move 10 pages at a time.'
          );
      writeln(output);
      writeln(output,'Radio control commands:');
      writeln(output,
'     The "+" and "-" keys bump the frequency +/-5khz.'
          );
      writeln(output,
'     The "<" & ">" keys (or "," & "." for no shifting) bump the mode.'
          );
      writeln(output,
'     The "[" and "]" keys bump the receiver bandwidth.'
          );
      writeln(output);
      writeln(output,
'Database commands: "*" finds the closest log entry for the current freq.');
      ch:=fetch;
    end;

  begin
    save_cursor_pos;
    repeat
      window(1,2,80,25);
      home;
      gotoxy(1,4);
      writeln(output,
'Type letter for command.  For example, to learn more about the receiver');
      writeln(output,
'display, type "r".  Type "q" to return from the help facility.');
      write_prompt(
'Help: R(eceiver display, C(ommands, M(ore commands, O(ther, Q(uit');
      ch:=upcase(fetch);
      case ch of
        'R': help_receiver;
        'C': help_commands;
        'M': help_more;
        'O': help_other;
      end;
    until ch = 'Q';
    init_crt;
    cmd_prompt(prompt_num);
    bottom_window;
  end;

  procedure timed_s_meter;
  var reading:real;
  begin
    save_cursor_pos;
    if radio_type <> 535 then exit;
    check_s_meter(reading);
    top_window;
    gotoxy(42,3);
    writea(LIGHTGRAY,FOREGROUND);
    write(output,'S-Meter:');
    if (reading > 9.0)
      then writea(RED,FOREGROUND)
      else writea(CYAN,FOREGROUND);
    if reading >= 10.0
      then write(output,reading:2:0,' ')
      else write(output,reading:2:1);
    bottom_window;
  end;

  procedure clear_meter;
  begin
    top_window;
    gotoxy(42,3);
    write(output,'           ');
    bottom_window;
  end;

  function tick;
  begin
    { check for timer tick }
    time_date_stamp(mon_str,day_str,year_str,time_str,FALSE);
    if time_str <> old_time_str then
      begin
        update_time_status;
        old_time_str:=time_str;
        tick:=TRUE;
      end
    else tick:=FALSE;
  end;

  procedure show_receiver;
  var s:string;

    procedure do_out(x,y:integer;unchanged:boolean; s:string;
                     var display_cnt:integer);
    begin
      if not unchanged then display_cnt:=0
                       else if display_cnt < 2 then inc(display_cnt);
      if display_cnt > 1 then exit; { don't write more than necessary }
      gotoxy(x,y);
      if not unchanged then writea(RED,FOREGROUND);
      write(output,s);
      writea(CYAN,FOREGROUND);
    end;

  begin
    save_cursor_pos;
    top_window;
    writea(CYAN,FOREGROUND);
    with receiverstat do
      begin
        case mode of
          RTTY:     s:='RTTY ';
          CW:       s:='CW   ';
          USB:      s:='USB  ';
          LSB:      s:='LSB  ';
          AM:       s:='AM   ';
          FM:       s:='FM   ';
          FAX:      s:='FAX  ';
          ECSS_USB: s:='ECSSu';
          ECSS_LSB: s:='ECSSl';
        end;
        do_out(9,3,receiverstat.mode = oldstat.mode,s,mode_cnt);
        case bandwidth of
          WIDE: s:='WIDE ';
          INTER:s:='INTER';
          NARR: s:='NARR ';
          AUX:  s:='AUX  ';
        end;
        do_out(19,3,receiverstat.bandwidth = oldstat.bandwidth,s
                   ,bandwidth_cnt);
        case agc of
          SLOW: s:='SLOW';
          FAST: s:='FAST';
          OFF:  s:='OFF ';
        end;
        do_out(34,3,receiverstat.agc = oldstat.agc,s,agc_cnt);
        case attenuator of
          YES:  s:='ATT';
          NO:   s:='   ';
        end;
        do_out(74,3,receiverstat.attenuator = oldstat.attenuator,s
                   ,attenuator_cnt);
        if radio_type = 525 then
          begin
            str(channel:3,s);
            do_out(46,3,receiverstat.channel = oldstat.channel,s
                       ,channel_cnt);
          end;
        str(frequency:8:2,s);
        do_out(60,3,receiverstat.frequency = oldstat.frequency,s
                   ,frequency_cnt);
        if radio_type = 525 then
          begin
            gotoxy(78,3);
            if map then write(output,'K') else write(output,' ');
          end;
        writea(LIGHTGRAY,FOREGROUND);
      end;
    bottom_window;
    oldstat:=receiverstat;
    gotoxy(x_pos,y_pos);
  end;

  procedure inc_mode;
  var x_pos,y_pos:integer;
  begin
    x_pos:=wherex; y_pos:=wherey;
    with receiverstat do
      begin
        remote_on;
        if radio_type = 525 then
          begin
            if mode < FAX
              then mode:=succ(mode)
              else mode:=RTTY;
          end
        else { do special case for nrd535 }
          begin
            case mode of
                  RTTY: mode:=CW;
                    CW: mode:=USB;
                   USB: mode:=LSB;
                   LSB: mode:=AM;
                    AM: mode:=ECSS_USB; { change order }
              ECSS_USB: mode:=ECSS_LSB;
              ECSS_LSB: mode:=FM;
                    FM: mode:=FAX;
                   FAX: mode:=RTTY;
            end;
          end;
        set_mode(mode);
        remote_off(REMOTE_DLY + 100);
        toggle_remote;
        show_receiver;
        gotoxy(x_pos,y_pos);
      end;
  end;

  procedure dec_mode;
  var x_pos,y_pos:integer;
  begin
    x_pos:=wherex; y_pos:=wherey;
    with receiverstat do
      begin
        remote_on;
        if radio_type = 525 then
          begin
            if mode > RTTY
              then mode:=pred(mode)
              else mode:=FAX;
          end
        else { do special case for nrd535 }
          begin
            case mode of
                  RTTY: mode:=FAX;
                    CW: mode:=RTTY;
                   USB: mode:=CW;
                   LSB: mode:=USB;
                    AM: mode:=LSB;
              ECSS_USB: mode:=AM;
              ECSS_LSB: mode:=ECSS_USB;
                    FM: mode:=ECSS_LSB;
                   FAX: mode:=FM;
            end;
          end;
        set_mode(mode);
        remote_off(REMOTE_DLY + 100);
        toggle_remote;
        show_receiver;
        gotoxy(x_pos,y_pos);
      end;
  end;

  procedure inc_bandwidth;
  begin
    with receiverstat do
      begin
        remote_on;
        bandwidth:=succ(bandwidth);
        set_bandwidth(bandwidth);
        remote_off(REMOTE_DLY);
        toggle_remote;
        show_receiver;
      end;
  end;

  procedure dec_bandwidth;
  begin
    with receiverstat do
      begin
        remote_on;
        bandwidth:=pred(bandwidth);
        set_bandwidth(bandwidth);
        remote_off(REMOTE_DLY);
        toggle_remote;
        show_receiver;
      end;
  end;

  
  function upcasestr;
  var i:integer;
  begin
    for i:=1 to length(str) do str[i]:=upcase(str[i]);
    upcasestr:=str;
  end;

  procedure program_radio;

  { set receiver to log entry; side effect - zaps channel 199 on 535 }

  begin
    remote_on;
    if radio_type = 535 then
        with log_entry do
          set_all(199,attenuator,bandwidth,mode,frequency,agc)
    else
      begin
        if map then {force to AM}
          begin
            set_freq(log_entry.frequency + MAP_OFFSET);
            set_mode(AM);
            set_bandwidth(WIDE);
            set_agc(FAST);
          end
        else {use log entry}
          begin
            set_freq(log_entry.frequency);
            if (log_entry.mode in [ECSS_USB,ECSS_LSB])
               then log_entry.mode:=AM;
            set_mode(log_entry.mode);
            set_bandwidth(log_entry.bandwidth);
            set_agc(log_entry.agc);
          end;
        set_attenuator(log_entry.attenuator);
      end;
    remote_off(REMOTE_DLY);
    update_receiver_display:=TRUE;
  end;

  procedure sync_loglist;
  var dummy:boolean;
      i,y_pos:integer;
      recnum:integer;
  begin
    i:=loglist.currentlog;
    if i = 0 then exit; { no logs yet }
    y_pos:=wherey;
    loglist.log[i].records:=records;
    recnum:=rec - 1;
    dummy:=precess(recnum,y_pos);
    if recnum = 0 then recnum:=1; { handle special case of empty log }
    loglist.log[i].rec:=recnum;
    put_loglist(loglist);
  end;

  procedure do_mark_range;
  var i, recnum:integer;
      dummy:boolean;

  begin
    save_cursor_pos;
    recnum:=rec - 1;
    dummy:=precess(recnum,y_pos);
    if recnum < min_mark then min_mark:=recnum;
    if recnum > max_mark then max_mark:=recnum;

    { flag to indicate the close of a marked range }
    mark_pair:=not mark_pair;

    { fill in the range between min and max marks }
    for i:=min_mark to max_mark do mark_array[recdata.recptr[i]]:=TRUE;

    { close off range if the end of a pair of marks }
    if not mark_pair then
      begin
        min_mark:=MAXREC + 1;
        max_mark:=0;
      end;
    show_log(rec,TRUE,TRUE);
  end;

  procedure do_mark_line;
  var i, recnum:integer;
      dummy:boolean;

  begin
    save_cursor_pos;
    recnum:=rec - 1;
    dummy:=precess(recnum,y_pos);
    mark_array[recdata.recptr[recnum]]:=TRUE;
    show_log(rec,TRUE,TRUE);
  end;

  procedure do_unmark_line;
  var i, recnum:integer;
      dummy:boolean;

  begin
    save_cursor_pos;
    recnum:=rec - 1;
    dummy:=precess(recnum,y_pos);
    mark_array[recdata.recptr[recnum]]:=FALSE;
    show_log(rec,TRUE,TRUE);
  end;

  procedure do_unmark_all;
  var i:integer;
  begin
    save_cursor_pos;
    max_mark:=0;
    min_mark:=MAXREC + 1;
    mark_pair:=FALSE;
    for i:=1 to MAXREC do mark_array[i]:=FALSE;
    if display then show_log(rec,TRUE,TRUE);
  end;

  procedure do_overall; { control overall radio settings }
  const s1 = 'Overall settings: ';
        s2 = 'I(nc, Q(uit';
        s3 = 'T(une rate, F(ilter, B(fo offset, S(sb, ';
      
  var ch:char;

    procedure do_tune_rate;
    var ch:char;
    begin
      if radio_type = 525 then exit;
      remote_on;
      repeat
        write_prompt('Tune rate: O(ne Hz, T(en Hz, one H(undred Hz, Q(uit');
        ch:=upcase(fetch);
        case ch of
          'O': set_tune_rate(0);
          'T': set_tune_rate(1);
          'H': set_tune_rate(2);
        end;
      until (ch in ['O','T','H','Q']);
      remote_off(0);
    end;

    procedure do_filter;
    var ch:char;
    begin
      if radio_type = 525 then exit;
      remote_on;
      repeat
        write_prompt('Filter: B(ypass, N(ot bypassed, Q(uit');
        ch:=upcase(fetch);
        case ch of
          'B': filter_bypass(TRUE);
          'N': filter_bypass(FALSE);
        end;
      until (ch in ['B','N','Q']);
      remote_off(0);
    end;

    procedure do_bfo_offset;
    var tabkey,backtabkey:boolean;
        s:string;
        i,dummy:integer;
    begin
      if radio_type = 525 then exit;
      write_prompt('Enter BFO offset:');
      str(loglist.bfo,s);
      editfield(18,0,5,TRUE,tabkey,backtabkey,s);
      val(s,i,dummy);
      if (i >= -5000) and (i <= 5000) then
        begin
          remote_on;
          loglist.bfo:=i;
          set_bfo(i);
          remote_off(0);
        end;
    end;

    procedure do_ssb;
    var ch:char;
    begin
      if radio_type = 525 then exit;
      remote_on;
      repeat
        write_prompt('SSB: D(isplay shift, L(ocal shift, Q(uit');
        ch:=upcase(fetch);
        case ch of
          'D': set_ssb(TRUE);
          'L': set_ssb(FALSE);
        end;
      until (ch in ['D','L','Q']);
      remote_off(0);
    end;

    procedure do_inc;
    var tabkey,backtabkey:boolean;
        s:string;
        dummy:integer;
    begin
      write_prompt('Enter frequency offset for +/- commands:');
      str(loglist.inc_amount:3:1,s);
      editfield(42,0,4,TRUE,tabkey,backtabkey,s);
      val(s,loglist.inc_amount,dummy);
    end;

    procedure show_all;
    var s:string;
    begin
      gotoxy(10,5);
      if radio_type = 535 then
        begin
          case loglist.tune_rate of
            0: s:='1';
            1: s:='10';
            2: s:='100';
            else s:='???';
          end;
          write(output,'Tune rate: ',s,'Hz');
          gotoxy(10,7);
          case loglist.filter of
            0: s:='bypassed';
            1: s:='in effect';
            else s:='???';
          end;
          write(output,'Input filter: ',s);
          gotoxy(10,9);
          write(output,'BFO offset: ');
          if (loglist.bfo >= -5000) and (loglist.bfo <=5000)
            then write(output,loglist.bfo,'Hz')
            else write(output,'???Hz');
          gotoxy(10,11);
          case loglist.ssb of
            0: s:='Display shift';
            1: s:='Local shift';
            else s:='??? shift';
          end;
          write(output,'SSB display frequency: ',s);
          gotoxy(1,24);
          write(output,
       'Note: Radio settings cannot be read.  Values reflect last setting');
          gotoxy(10,13);
        end;
      write(output,'Frequency increment for "+" and "-" commands: ',
                              loglist.inc_amount:3:1,'KHz');
    end;

   begin
    save_cursor_pos;
    information_mode_off;
    repeat
      window(1,1,80,25);
      writea(BLACK,BACKGROUND);
      writea(CYAN,FOREGROUND);
      home;
      show_all;
      case radio_type of
        525: write_prompt(s1 + s2);
        535: write_prompt(s1 + s3 + s2);
      end;
      ch:=upcase(fetch);
      case ch of
        'T': do_tune_rate;
        'F': do_filter;
        'B': do_bfo_offset;
        'S': do_ssb;
        'I': do_inc;
      end;
    until ch = 'Q';
    init_crt;
    cmd_prompt(prompt_num);
    bottom_window;
    show_log(rec,TRUE,TRUE);
    information_mode_on;
  end;

  procedure do_move; { load a range of memory channels with marked values }
  var i,channel:integer;
      logdat:logtype;
      ok,nullval:boolean;
      old_stat:receivertype;
      s:string;
  begin
    save_cursor_pos;
    write_prompt
            ('Move: write marked lines to Radio Channels <ESCAPE to exit>');
    bottom_window;
    home;
    i:=0;
    s:='starting channel';
    old_stat:=receiverstat;
    if radio_type = 535 then s:=s + ' (999 clears all)';
    entnum(1,5,channel,ok,nullval,s);
    if nullval then channel:=last_channel;
    if ok and (channel >=0) and (channel < 200) then
      begin
        remote_on;
        write_prompt('Performing move...');
        bottom_window;
        home;
        repeat
          inc(i);
          if mark_array[recdata.recptr[i]] then {found marked area to write}
            begin
              dot;
              get_log(logbuf,logdat,recdata.recptr[i]);
              with logdat do
                 set_all(channel,attenuator,bandwidth,mode,frequency,agc);
              inc(channel);
            end;
        until (i >= records) or (channel > 199);
      end
    else if ok and (channel = 999) and (radio_type = 535) then
               { special case; clear all channels }
      begin
        remote_on;
        write_prompt('Clearing channels...');
        bottom_window;
        home;
        clear_channels;
      end;
    remote_off(0);
    receiverstat:=old_stat;
    { restore radio }
    logdat.attenuator:=receiverstat.attenuator;
    logdat.bandwidth:= receiverstat.bandwidth;
    logdat.mode:=      receiverstat.mode;
    logdat.frequency:= receiverstat.frequency;
    logdat.agc:=       receiverstat.agc;
    program_radio(logdat);
    toggle_remote;
    last_channel:=channel;
    cmd_prompt(prompt_num);
    bottom_window;
    show_log(rec,TRUE,TRUE);
  end;

  procedure do_undelete;
  var t,recnum:integer;
      x_pos,y_pos:integer;
      dummy:boolean;
      i,j:integer;
      ch:char;
  begin
    write_prompt('uNdelete:  Type "y" to continue');
    ch:=upcase(fetch);
    cmd_prompt(prompt_num);
    bottom_window;
    if ch <> 'Y' then exit;
    x_pos:=wherex; y_pos:=wherey;
    recnum:=rec - 1;
    dummy:=precess(recnum,y_pos);
    i:=records;

    while (i > 1) and (recdata.recstat[recdata.recptr[i]] <> DELETED)
      do i:=i - 1;
    if recdata.recstat[recdata.recptr[i]] = DELETED then { found one }
      begin
        t:=recdata.recptr[i];
        recdata.recstat[t]:=SHOW;
        for j:=i downto recnum + 1 do
          recdata.recptr[j]:=recdata.recptr[j - 1];
        recdata.recptr[recnum]:=t;
        put_recdata(loglist.currentlog,recdata);
        show_log(rec,TRUE,TRUE);
        inc(reccnt[loglist.currentlog]);
      end;
  end;

  procedure do_sort;
  var sortdata:sort_array_type;
      subsortdata:sort_array_type;
      primary,secondary:char;

    function get_sort_type(auto:boolean; var primary, secondary:char):boolean;
    const S1=' field: D(ate, T(ime, F(req, I(d, L(oc, C(omment, M(ode, Q(uit';
          S0='Sort - ';
    begin
      if auto then { automatically do Frequency, Time sort }
        begin
          get_sort_type:=TRUE;
          primary:='F';
          secondary:='T';
          exit;
        end;
      repeat
        write_prompt(S0 + 'Primary' + S1);
        primary:=upcase(fetch);
      until primary in ['D','T','F','I','L','C','M','Q'];
      if primary <> 'Q' then
        repeat
          write_prompt(S0 + 'Secondary' + S1);
          secondary:=upcase(fetch);
        until secondary in ['D','T','F','I','L','C','M','Q'];
      cmd_prompt(prompt_num);
      bottom_window;
      get_sort_type:=(primary <> 'Q') and (secondary <> 'Q');
    end;

    procedure get_fields;
    var i:integer;
        logdata:logtype;

      procedure init_array(var sortdata:short_str; cmd:char);
      begin
        case cmd of
          'D':sortdata:=copy(logdata.date,1,SHORTSTRLEN);
          'T':sortdata:=copy(logdata.begin_time,1,SHORTSTRLEN);
          'F':str(logdata.frequency:8:1,sortdata);
          'I':sortdata:=copy(logdata.callsign,1,SHORTSTRLEN);
          'L':sortdata:=copy(logdata.location,1,SHORTSTRLEN);
          'C':sortdata:=copy(logdata.comment,1,SHORTSTRLEN);
          'M':sortdata:=mode_to_str(logdata.mode);
        end;
      end;

    begin
      home;
      for i:=1 to records do
      begin
        if recdata.recstat[i] = DELETED then { dummy sort pos }
          begin
            sortdata[i]:=chr(255);
            subsortdata[i]:=chr(255);
          end
        else
          begin
            get_log(logbuf,logdata,i);
            init_array(sortdata[i],primary);
            init_array(subsortdata[i],secondary);
          end;
        recdata.recptr[i]:=i;
      end;
    end;

    procedure primary_sort; { sort on primary field }
    begin
      write(output,'Primary sort');
      sort(sortdata,recdata.recptr,1,records);
    end;

    procedure secondary_sort; { sort on secondary field }
    var i,top:integer;
        tempstr:string;
    begin
      top:=1; home;
      write(output,'Secondary sort');
      while (top < records) do
        begin
          i:=0;
          tempstr:=sortdata[top];
          while (top + i < records) and (tempstr = sortdata[top + i]) do
            begin
              sortdata[top + i]:=subsortdata[recdata.recptr[top + i]];
              inc(i);
            end;
          sort(sortdata,recdata.recptr,top,i);
          top:=top + i;
        end;
    end;

  begin
    if get_sort_type(auto,primary,secondary) then
      begin
        get_fields;
        primary_sort;
        secondary_sort;
      end;
    show_log(rec,TRUE,TRUE);
    put_recdata(loglist.currentlog,recdata);
  end;

  procedure update_time_status;
  var the_date:string;
      the_time:string;
  begin
    save_cursor_pos;
    the_date:=mon_str + '/' + day_str + '/' + year_str;
    the_time:=copy(time_str,1,2) + ':' + copy(time_str,3,2) + ':' +
              copy(time_str,5,2);
    window(1,25,80,25);
    gotoxy(63,1);
    writea(BLACK,BACKGROUND);
    writea(BROWN,FOREGROUND);
    write(output,the_time,' ', the_date);
    writea(CYAN,FOREGROUND);
    bottom_window;
  end;

  procedure new_log;
  begin
    if lognum = 0 then { no files yet }
      begin
        records:=0;
        rec:=1;
      end
    else
      begin
        open_log(logbuf,lognum, rslt);
        get_recdata(lognum, recdata);
        records:=loglist.log[lognum].records;
        rec:=loglist.log[lognum].rec;
      end;
  end;

  procedure do_alternate;
  var i:integer;
      recnum:integer;
      t_begin, t_end, t_now, dummy:integer;
  begin
    save_cursor_pos;
    recnum:=rec - 1;
    sync_loglist;
    if loglist.currentlog > 0 then { window current rec for "W" command }
      begin
        new_log(loglist.currentlog,rslt);
        if loglist.log[loglist.currentlog].records > 0 then
          begin
            if precess(recnum, y_pos) then
              get_log(logbuf,last_log_data,recdata.recptr[recnum]);
          end
        else clear_log(last_log_data);
        close(logbuf);
      end;
    if last_log > 0 then { there is a log to alternate to }
      begin
        i:=last_log;
        last_log:=loglist.currentlog;
        loglist.currentlog:=i;
        put_loglist(loglist);
        do_unmark_all(FALSE);
      end;
    status_window;
    bottom_window;
    new_log(loglist.currentlog,rslt);
    x_pos:=1; y_pos:=1;
    gotoxy(x_pos,y_pos);
    show_log(rec,TRUE,TRUE);
  end;

  procedure do_search;
  { search for a string in a field.  if again is true, find next occurance }

  const search_field:char = 'C';
        search_string:string = 'Music';
        search_class:char = 'N';

  var ch:char;
      continue:boolean;

    function get_search_class(var class:char):boolean;
    begin
      repeat
        write_prompt('Search: select class:  N(ext, M(ark all, Q(uit');
        class:=upcase(fetch);
      until class in ['N', 'M', 'Q'];
      get_search_class:=class <> 'Q';
    end;

    function get_search_field(var search_field:char):boolean;
    begin
      repeat
        write_prompt('Select search field: D(ate, T(ime, F(req, I(d,'
                     +' L(oc, C(omment, M(ode, Q(uit');
        search_field:=upcase(fetch);
      until search_field in ['D','T','F','I','L','C','M','Q'];
      get_search_field:=(search_field <> 'Q');
    end;

    procedure get_search_string(var search_string:string);
    var tabkey, backtabkey:boolean; { ignored }
        i:integer;
    begin
      write_prompt('Enter string to search for:');
      search_string:='';
      editfield(30,0,8,FALSE,tabkey,backtabkey,search_string);

      {strip off trailing blanks }
      i:=length(search_string);
      while (i > 0) and (search_string[i] = ' ') do
        begin
          delete(search_string,i,1);
          dec(i);
        end;

      search_string:=upcasestr(search_string);
      write_prompt('Searching...');
    end;

    procedure get_field(logdata:logtype; var fielddata:string; cmd:char);
    begin
      case cmd of
        'D':fielddata:=copy(logdata.date,1,DATELEN);
        'T':fielddata:=copy(logdata.begin_time,1,TIMELEN);
        'F':str(logdata.frequency:8:1,fielddata);
        'I':fielddata:=copy(logdata.callsign,1,CALLSIGNLEN);
        'L':fielddata:=copy(logdata.location,1,LOCATIONLEN);
        'C':fielddata:=copy(logdata.comment,1,COMMENTLEN);
        'M':fielddata:=mode_to_str(logdata.mode);
      end;
    end;

    procedure search_next(search_field:char);
    var i:integer;
        logdata:logtype;
        test_string:string;
        found, dummy:boolean;
    begin
      bottom_window;
      home;
      i:=rec - 1;
      dummy:=precess(i,y_pos);
      found:=FALSE;
      while (i < records) and not found do
        begin
          dummy:=precess(i,1);
          if recdata.recstat[recdata.recptr[i]] = SHOW then
            begin
              get_log(logbuf,logdata,recdata.recptr[i]);
              get_field(logdata,test_string,search_field);
              test_string:=upcasestr(test_string);
              found:=pos(search_string,test_string) > 0;
              dot;
            end;
         end;
      rec:=i;
      y_pos:=1;
      gotoxy(x_pos,y_pos);
      show_log(rec,TRUE,TRUE);
    end;

    procedure search_mark(search_field:char);
    var i:integer;
        logdata:logtype;
        test_string:string;
    begin
      bottom_window;
      home;
      for i:=1 to records do
        begin
          if recdata.recstat[recdata.recptr[i]] = SHOW then
            begin
              get_log(logbuf,logdata,recdata.recptr[i]);
              get_field(logdata,test_string,search_field);
              test_string:=upcasestr(test_string);
              if pos(search_string,test_string) > 0
                then mark_array[recdata.recptr[i]]:=TRUE;
              dot;
            end;
         end;
      gotoxy(x_pos,y_pos);
      show_log(rec,TRUE,TRUE);
    end;

  begin
    continue:=TRUE;
    save_cursor_pos;
    if not again then
      begin
        continue:=get_search_class(search_class);
        if continue then continue:=get_search_field(search_field);
        if continue then get_search_string(search_string);
      end;
    if continue then
      begin
        case search_class of
          'N': search_next(search_field);
          'M': search_mark(search_field);
        end;
      end;
    cmd_prompt(prompt_num);
    bottom_window;
  end;

  procedure kill_clock;
  { erase time and date from status window and leave cursor in status window }
  begin
    window(1,25,80,25);
    threshold_status(highlight); { put the cursor in the right place }
    call_crt(ERASEOL); { clear off time and date for now }
  end;

  procedure do_threshold;
  { modify threshold setting used for scanning functions }
  var ch:char;

    procedure bump_threshold(amount:integer);
    begin
      scan_threshold:=scan_threshold + amount;
      threshold_status(TRUE);
    end;

  begin
    if radio_type <> 535 then exit;
    save_cursor_pos;
    write_prompt(
'Scanning threshold adjust: Use UP / DOWN arrow keys to adjust, Q(uit');
    kill_clock(TRUE);
    repeat
      ch:=upcase(fetch);
      case ch of
          UP:bump_threshold(-1);
        DOWN:bump_threshold(1);
      end;
    until not (ch in [UP, DOWN]);
    kill_clock(FALSE); { restore normal color }
    cmd_prompt(prompt_num);
    bottom_window;
    loglist.scan_threshold:=scan_threshold;
  end;

  procedure do_inc_class;
  begin
    if radio_type <> 535 then exit;
    save_cursor_pos;
    write_prompt(
'+ / - Mode select: F(req, N(ext, M(ark, S(can');
    kill_clock(FALSE);
    inc_class_status(TRUE); { put cursor at right point }
    repeat
      ch:=upcase(fetch);
      case ch of
        'F': inc_class:=FIXED_OFFSET;
        'N': inc_class:=NEXT_FREQ;
        'M': inc_class:=NEXT_MARKED;
        'S': inc_class:=SCAN_MARKED;
      end;
      inc_class_status(TRUE);
    until (ch in ['F','N','M','S']);
    inc_class_status(FALSE);
    cmd_prompt(prompt_num);
    bottom_window;
  end;

  procedure do_page;
  begin
    save_cursor_pos;
    display_page:=display_page + 1;
    if display_page > 3 then display_page:=1;
    draw_display_titles;
    bottom_window;
    if rslt = 0 then show_log(rec,TRUE,TRUE);
  end;

  procedure do_tab;
  begin
    save_cursor_pos;
    case display_page of
      1:begin
          if x_pos in [1..5]   then x_pos:=6  else
          if x_pos in [6..12]  then x_pos:=13 else
          if x_pos in [13..17] then x_pos:=18 else
          if x_pos in [18..22] then x_pos:=23 else
          if x_pos in [23..31] then x_pos:=32 else
          if x_pos in [32..51] then x_pos:=52 else
            begin
              x_pos:=11;
              gotoxy(x_pos,y_pos);
              do_page;
            end;
          gotoxy(x_pos,y_pos);
        end;
      2:begin
          x_pos:=32;
          gotoxy(x_pos,y_pos);
          do_page;
        end;
      3:begin
          if x_pos in [1..5]   then x_pos:=6  else
          if x_pos in [6..12]  then x_pos:=13 else
          if x_pos in [13..17] then x_pos:=18 else
          if x_pos in [18..22] then x_pos:=23 else
          if x_pos in [23..31] then x_pos:=32 else
          if x_pos in [32..36] then x_pos:=37 else
          if x_pos in [37..42] then x_pos:=43 else
          if x_pos in [43..48] then x_pos:=49 else
            begin
              x_pos:=6;
              gotoxy(x_pos,y_pos);
              do_page;
            end;
          gotoxy(x_pos,y_pos);
        end;
    end;
  end;

  procedure do_backtab;
  begin
    save_cursor_pos;
    case display_page of
      1:begin
          if x_pos in [7..13]  then x_pos:=6  else
          if x_pos in [14..18] then x_pos:=13 else
          if x_pos in [19..23] then x_pos:=18 else
          if x_pos in [24..32] then x_pos:=23 else
          if x_pos in [33..80] then x_pos:=32 else
            begin
              x_pos:=49;
              gotoxy(x_pos,y_pos);
              display_page:=2;
              do_page;
            end;
          gotoxy(x_pos,y_pos);
        end;
      2:begin
          if x_pos in [3..10]  then x_pos:=2  else
          if x_pos in [12..80] then x_pos:=11 else
            begin
              x_pos:=52;
              gotoxy(x_pos,y_pos);
              display_page:=display_page - 2;
              do_page;
            end;
          gotoxy(x_pos,y_pos);
        end;
      3:begin
          if x_pos in [7..13]  then x_pos:=6  else
          if x_pos in [14..18] then x_pos:=13 else
          if x_pos in [19..23] then x_pos:=18 else
          if x_pos in [24..32] then x_pos:=23 else
          if x_pos in [33..37] then x_pos:=32 else
          if x_pos in [38..43] then x_pos:=37 else
          if x_pos in [44..49] then x_pos:=43 else
          if x_pos in [50..80] then x_pos:=49 else
            begin
              x_pos:=11;
              gotoxy(x_pos,y_pos);
              display_page:=display_page - 2;
              do_page;
            end;
          gotoxy(x_pos,y_pos);
        end;
    end;
  end;

  procedure do_edit; { edit field cursor is on }
  var recnum:integer;
      logdata:logtype;
      s:string;
      i,j,y,dummy:integer;
      tabkey,backtabkey:boolean;

    procedure edit_page1;
    begin
      case x_pos of
         6..12: editfield(5, y,DATELEN,FALSE,tabkey,backtabkey,logdata.date);
        13..17: begin
                  editfield(12,y,TIMELEN,FALSE,tabkey,backtabkey
                                ,logdata.begin_time);
                  while length(logdata.begin_time) < 4 do
                      logdata.begin_time:=concat('0',logdata.begin_time);
                end;
        18..22: begin
                  editfield(17,y,TIMELEN,FALSE,tabkey,backtabkey
                                ,logdata.end_time);
                  while length(logdata.end_time) < 4 do
                      logdata.end_time:=concat('0',logdata.end_time);
                end;
        23..31: begin
                  str(logdata.frequency:8:2,s);
                  editfield(22,y,8,TRUE,tabkey,backtabkey,s);
                  val(s,logdata.frequency,dummy);
                end;
        32..51: editfield(31,y,CALLSIGNLEN,FALSE,tabkey,backtabkey
                              ,logdata.callsign);
        52..80: editfield(51,y,LOCATIONLEN,FALSE,tabkey,backtabkey
                              ,logdata.location);
      end;
    end;

    procedure edit_page2;
    begin
      case x_pos of
         1..10: begin
                  str(logdata.frequency:8:2,s);
                  editfield(1,y,8,TRUE,tabkey,backtabkey,s);
                  val(s,logdata.frequency,dummy);
                end;
        11..80: editfield(10,y,COMMENTLEN-1,FALSE,tabkey,backtabkey
                              ,logdata.comment);
      end;
    end;

    procedure edit_page3;
    begin
      case x_pos of
         6..12: editfield(5, y,DATELEN,FALSE,tabkey,backtabkey,logdata.date);
        13..17: begin
                  editfield(12,y,TIMELEN,FALSE,tabkey,backtabkey
                                ,logdata.begin_time);
                  while length(logdata.begin_time) < 4 do
                      logdata.begin_time:=concat('0',logdata.begin_time);
                end;
        18..22: begin
                  editfield(17,y,TIMELEN,FALSE,tabkey,backtabkey
                                ,logdata.end_time);
                  while length(logdata.end_time) < 4 do
                      logdata.end_time:=concat('0',logdata.end_time);
                end;
        23..31: begin
                  str(logdata.frequency:8:2,s);
                  editfield(22,y,8,TRUE,tabkey,backtabkey,s);
                  val(s,logdata.frequency,dummy);
                end;
        32..36: begin
                  case logdata.mode of
                    RTTY: s:='RTTY';
                    CW:   s:=' CW';
                    USB:  s:='USB';
                    LSB:  s:='LSB';
                    AM:   s:=' AM';
                    FM:   s:=' FM';
                    FAX:  s:='FAX';
                  end;
                  editfield(31,y,4,FALSE,tabkey,backtabkey,s);
                  s:=upcasestr(s);
                  if pos('RTTY',s) > 0 then logdata.mode:=RTTY else
                  if pos('CW',s)   > 0 then logdata.mode:=CW   else
                  if pos('USB',s)  > 0 then logdata.mode:=USB  else
                  if pos('LSB',s)  > 0 then logdata.mode:=LSB  else
                  if pos('AM',s)   > 0 then logdata.mode:=AM   else
                  if pos('FM',s)   > 0 then logdata.mode:=FM   else
                  if pos('FAX',s)  > 0 then logdata.mode:=FAX
                end;
        37..42: begin
                  case logdata.bandwidth of
                    NARR: s:=' NARR';
                    INTER:s:='INTER';
                    WIDE: s:=' WIDE';
                    AUX:  s:=' AUX';
                  end;
                  editfield(36,y,5,FALSE,tabkey,backtabkey,s);
                  s:=upcasestr(s);
                  if pos('INTER',s) > 0 then logdata.bandwidth:=INTER else
                  if pos('NARR',s)  > 0 then logdata.bandwidth:=NARR  else
                  if pos('WIDE',s)  > 0 then logdata.bandwidth:=WIDE  else
                  if pos('AUX' ,s)  > 0 then logdata.bandwidth:=AUX
                end;
        43..48: begin
                  case logdata.agc of
                    FAST: s:='FAST';
                    SLOW: s:='SLOW';
                    OFF:  s:=' OFF';
                  end;
                  editfield(42,y,4,FALSE,tabkey,backtabkey,s);
                  s:=upcasestr(s);
                  if pos('FAST',s) > 0 then logdata.agc:=FAST else
                  if pos('SLOW',s) > 0 then logdata.agc:=SLOW else
                  if pos('OFF',s)  > 0 then logdata.agc:=OFF
                end;
        49..51: begin
                  case logdata.attenuator of
                    YES: s:='ON';
                    NO:  s:='OFF';
                  end;
                  editfield(48,y,3,FALSE,tabkey,backtabkey,s);
                  s:=upcasestr(s);
                  if pos('OFF',s) > 0 then logdata.attenuator:=NO else
                  if pos('ON',s)  > 0 then logdata.attenuator:=YES
                end;
      end;
    end;

  begin { do_edit }
    save_cursor_pos;
    y:=y_pos - 1;
    recnum:=rec - 1;
    if precess(recnum,y_pos) then
      begin
        get_log(logbuf,logdata,recdata.recptr[recnum]);
        case display_page of
          1: edit_page1;
          2: edit_page2;
          3: edit_page3;
        end;
        put_log(logbuf,logdata,recdata.recptr[recnum]);
        if not (tabkey or backtabkey) then
          begin
            gotoxy(x_pos,y_pos);
            show_log(rec,TRUE,TRUE);
          end;
      end;
    gotoxy(x_pos,y_pos);
    if tabkey then
      begin
        case display_page of
           1,3: do_tab;
           2:begin
               if x_pos in [1..10] then
                 begin
                    x_pos:=11;
                    gotoxy(x_pos,y_pos);
                 end
               else
                 begin
                   x_pos:=32;
                   gotoxy(x_pos,y_pos);
                   do_page;
                 end;
             end;
        end;
        do_edit;
      end
    else if backtabkey then
      begin
        do_backtab;
        do_edit;
      end;
  end;

  procedure do_delete;
  var x_pos,y_pos:integer;
      recnum:integer;
      ch:char;
      i,t:integer;
  begin
    x_pos:=wherex; y_pos:=wherey;
    recnum:=rec - 1;
    write_prompt('Delete:  Type "y" to continue');
    ch:=upcase(fetch);
    cmd_prompt(prompt_num);
    bottom_window;
    if ch = 'Y' then if precess(recnum,y_pos) then
      begin
        t:=recdata.recptr[recnum];
        recdata.recstat[t]:=DELETED;
        for i:=recnum to records - 1 do with recdata do
          recptr[i]:=recptr[i + 1];
        recdata.recptr[records]:=t;
        show_log(rec,TRUE,TRUE);
        put_recdata(loglist.currentlog,recdata);
        dec(reccnt[loglist.currentlog]);
      end;
  end;

  procedure do_log;
  var x_pos,y_pos:integer;
      t,recnum:integer;
      ch:char;
      i,j:integer;
      dummy:boolean;
      logdata:logtype;
  begin
    if loglist.currentlog = 0 then exit; { can't log w/o database created }
    x_pos:=32; y_pos:=wherey;
    recnum:=rec - 1;
    dummy:=precess(recnum,y_pos);
    { get receiver status }
    toggle_remote;
    show_receiver;
    remote_off(0);
    i:=1;
    while (i < records) and (recdata.recstat[recdata.recptr[i]] <> DELETED)
       do inc(i);
    if (i >= records) and (recdata.recstat[recdata.recptr[i]]<>DELETED) then
      { insert new entry here }
      begin
        inc(records);
        i:=records;
        recdata.recptr[i]:=i;
        loglist.log[loglist.currentlog].records:=records;
        put_loglist(loglist);
      end;
    t:=recdata.recptr[i];
    if recnum = 0 then recnum:=1; { special case for new arrays }
    for j:=i downto recnum + 1 do with recdata do
      recptr[j]:=recptr[j - 1];
    recdata.recptr[recnum]:=t;
    recdata.recstat[t]:=SHOW;
    clear_log(logdata);
    with receiverstat do
      begin
        if map then {center frequency}
          begin
            logdata.frequency:=trunc(frequency/5.0 + 0.5) * 5;
            logdata.mode:=USB;
            logdata.bandwidth:=INTER;
          end
        else
          begin
            logdata.frequency:=frequency;
            logdata.mode:=mode;
            logdata.bandwidth:=bandwidth;
          end;
        logdata.agc:=agc;
        logdata.attenuator:=attenuator;
      end;
    put_log(logbuf,logdata,recdata.recptr[recnum]);
    put_recdata(loglist.currentlog,recdata);
    inc(reccnt[loglist.currentlog]);
    if display_page <> 1 then
      begin
        display_page:=1;
        draw_display_titles;
        bottom_window;
      end;
    gotoxy(x_pos,y_pos);
    show_log(rec,TRUE,TRUE);
    do_edit;
  end;

  procedure do_tune;

  { assign log entry at cursor location to radio }

  var recnum:integer;
      logdata:logtype;
  begin
   y_pos:=wherey;
   recnum:=rec - 1;
   if precess(recnum, y_pos) then
     begin
       get_log(logbuf,logdata,recdata.recptr[recnum]);
       program_radio(logdata);
       toggle_remote;
     end;
  end;

  function find_rec;
  { find record >= frequency }
  var j:integer;
      logdata:logtype;
      first_try, found:boolean;
  begin
    j:=rec - 20; { skip back enuf records to find start hopefully }
    if j < 0 then j:=0;
    found:=FALSE; first_try:=TRUE;
    while (j < records) and not found do
      begin
        if precess(j,1) then
          begin
            get_log(logbuf,logdata,recdata.recptr[j]);
            if first_try and (logdata.frequency > freq)
              then j:=0
              else found:=logdata.frequency >= freq;
            first_try:=FALSE;
          end;
      end;
    find_rec:=j;
  end;

  procedure find_freq;
  var s:string;
      x_pos,y_pos,i:integer;
      orig_freq:real;
      ch:char;

    procedure display(frequency:real); { find displayed line matching freq }
    var found:boolean;
    begin
      rec:=find_rec(1, frequency);
      if rec < 1 then rec:=1;
      y_pos:=1;
      gotoxy(x_pos,1);
      show_log(rec,TRUE,TRUE);
    end;

  begin {find_freq}
    x_pos:=wherex; y_pos:=wherey;
    show_log(rec,FALSE,FALSE); { unhighlight current cursor line }
    with receiverstat do
      begin
        toggle_remote;
        show_receiver;
        display(receiverstat.frequency);
      end;
  end;

  procedure do_freq;
  var x_pos,y_pos:integer;
      s:string;
      freq:real;
      dummy:integer;
      tabkey,backtabkey:boolean;
  begin
    x_pos:=wherex;
    y_pos:=wherey;
    write_prompt('Enter frequency to set:');
    s:='';
    editfield(25,0,8,TRUE,tabkey,backtabkey,s);
    remote_on;
    val(s,freq,dummy);
    cmd_prompt(prompt_num);
    gotoxy(x_pos,y_pos);
    if freq > 0 then set_freq(freq);
    remote_off(200);
    toggle_remote;
    show_receiver;
    find_freq;
  end;

  procedure do_kiwa; { different mode if KIWA MAP unit in use }
  var freq,offset:real;
      x_pos,y_pos:integer;
  begin
    x_pos:=wherex; y_pos:=wherey;
    map:=not map; {toggle mode}
    with receiverstat do
      begin
        if map then {enable mode}
          begin
            if mode = USB then offset:=MAP_OFFSET else offset:=-MAP_OFFSET;
            remote_on;
            set_mode(AM);
            mode:=AM;
            set_bandwidth(WIDE);
            bandwidth:=WIDE;
            set_agc(FAST);
            agc:=FAST;
            frequency:=trunc(frequency / 5.0 + 0.5) * 5.0 + offset;
            set_freq(frequency);
            remote_off(REMOTE_DLY);
          end
        else
          begin
            remote_on;
            freq:=frequency;
            frequency:=trunc(frequency / 5.0 + 0.5) * 5.0;
            set_freq(frequency);
            if frequency < freq then
              begin
                set_mode(USB);
                mode:=USB;
              end
            else
              begin
                set_mode(LSB);
                mode:=LSB;
              end;
            set_bandwidth(INTER);
            bandwidth:=INTER;
          end;
        remote_off(REMOTE_DLY);
      end;
    show_receiver;
    gotoxy(x_pos,y_pos);
  end;

  procedure do_confirm;

  { refresh database time and date and receiver status }

  var recnum:integer;
      tlog,logdata:logtype;
      t_begin, t_end, t_now, dummy:integer;
      ch:char;
      s:string;
  begin
   save_cursor_pos;
   { get receiver status }
   toggle_remote;
   if async_buffer_check(ch) then check_status(ch,s);
   show_receiver;
   recnum:=rec - 1;
   if precess(recnum, y_pos) then
     begin
      clear_log(tlog);
      get_log(logbuf,logdata,recdata.recptr[recnum]);
      write_prompt('Confirm:  Type "y" to continue');
      ch:=upcase(fetch);
      cmd_prompt(prompt_num);
      if ch = 'Y' then
        begin
          logdata.date:=tlog.date;
          val(tlog.begin_time,t_now,dummy);
          val(logdata.begin_time,t_begin,dummy);
          val(logdata.end_time,t_end,dummy);
          t_begin:=t_begin - t_now;
          if t_begin < 0 then t_begin:=t_begin + 2400;
          t_end:=t_now - t_end;
          if t_end < 0 then t_end:=t_end + 2400;
          if t_begin < t_end
             then if t_begin < 1200 then logdata.begin_time:=tlog.begin_time;
          if t_end < t_begin
             then if t_end < 1200 then logdata.end_time:=tlog.begin_time;
          with receiverstat do
            begin
              if not map then {don't update receiver params if using map}
                begin
                  logdata.frequency:=frequency;
                  logdata.mode:=mode;
                  logdata.agc:=agc;
                  logdata.attenuator:=attenuator;
                  logdata.bandwidth:=bandwidth;
                end;
            end;
          put_log(logbuf,logdata,recdata.recptr[recnum]);
        end;
      end;
   bottom_window;
   show_log(rec,TRUE,TRUE);
  end;

  procedure do_write; { as in dudley... }
  { copy record at cursor in inactive log to current log }
  var x_pos,y_pos:integer;
      t,recnum:integer;
      ch:char;
      i,j:integer;
      dummy:boolean;
  begin
    if last_log = 0 then exit;
    x_pos:=wherex; y_pos:=wherey;
    recnum:=rec - 1;
    dummy:=precess(recnum,y_pos);
    i:=1;
    while (i < records) and (recdata.recstat[recdata.recptr[i]] <> DELETED)
       do inc(i);
    if (i >= records) and (recdata.recstat[recdata.recptr[i]]<>DELETED) then
      { insert new entry here }
      begin
        inc(records);
        i:=records;
        recdata.recptr[i]:=i;
        loglist.log[loglist.currentlog].records:=records;
        put_loglist(loglist);
      end;
    t:=recdata.recptr[i];
    if recnum = 0 then recnum:=1; { special case for new arrays }
    for j:=i downto recnum + 1 do with recdata do
      recptr[j]:=recptr[j - 1];
    recdata.recptr[recnum]:=t;
    recdata.recstat[t]:=SHOW;
    put_log(logbuf,last_log_data,recdata.recptr[recnum]);
    put_recdata(loglist.currentlog,recdata);
    inc(reccnt[loglist.currentlog]);
    if display_page <> 1 then
      begin
        display_page:=1;
        draw_display_titles;
        bottom_window;
      end;
    gotoxy(x_pos,y_pos);
    show_log(rec,TRUE,TRUE);
  end;

  procedure do_pageup;
  var i,j:integer;
      dummy:boolean;
      last:boolean;
  begin
    x_pos:=wherex;
    for j:=1 to cnt do
      begin
        for i:=1 to LINES - REC_WIN_Y_BOTTOM + 1 do
          begin
            if rec > 1 then dec(rec);
            while (rec > 1) and (recdata.recstat[recdata.recptr[rec]]
                    <> SHOW) do dec(rec);
          end;
      end;

    { place cursor at bottom (or last record) }
    y_pos:=0;
    j:=rec - 1;
    last:=FALSE;
    for i:=2 to LINES - REC_WIN_Y_BOTTOM  do
      begin
        dummy:=precess(j,1);
        if (recdata.recstat[recdata.recptr[j]] = SHOW) and (not last)
           then inc(y_pos);
        last:=j = records; { funky ending condition; j won't exceed records}
      end;
    gotoxy(x_pos,y_pos);
    if rslt = 0 then show_log(rec,TRUE,TRUE);
  end;

  procedure do_pagedown;
  var i:integer;
      dummy:boolean;
  begin
    x_pos:=wherex; y_pos:=1;
    gotoxy(x_pos,y_pos);
    for i:=1 to cnt do
      begin
        dummy:= precess(rec, LINES - REC_WIN_Y_BOTTOM - 1);
        if rec >= records then
          begin
             rec:=records;
             if rec < 1 then rec:=1; { special case for empty file }
             while (recdata.recstat[recdata.recptr[rec]] <> SHOW) do
               dec(rec);
          end;
      end;
    if rslt = 0 then show_log(rec,TRUE,TRUE);
  end;

  procedure do_up;
  begin
    save_cursor_pos;
    y_pos:=y_pos - 1;
    if y_pos < 1 then
      begin
        y_pos:=1;
        if rec > 1 then rec:=rec - 1;
        show_log(rec,TRUE,TRUE);
      end
    else
      begin
        show_log(rec,FALSE,FALSE);
        gotoxy(x_pos,y_pos);
        show_log(rec,FALSE,TRUE);
      end;
  end;

  procedure do_down;
  var dummy:boolean;
      i,y,j:integer;
  begin
    save_cursor_pos;
    y:=y_pos;
    inc(y);

    { keep cursor from moving past bottom }
    j:=rec - 1;
    for i:=1 to y do
      begin
        inc(j);
        if (j > records) or (recdata.recstat[recdata.recptr[j]]<>SHOW) then
          begin
            y_pos:=wherey;
            exit;
          end;
      end;
    y_pos:=y;
    if y_pos > LINES - REC_WIN_Y_BOTTOM - 1 then
     begin
       y_pos:=LINES - REC_WIN_Y_BOTTOM - 1;
       dummy:=precess(rec,1);
       if rec > records then rec:=records;
       if rslt = 0 then show_log(rec,TRUE,TRUE);
     end
    else
      begin
        show_log(rec,FALSE,FALSE);
        gotoxy(x_pos,y_pos);
        show_log(rec,FALSE,TRUE);
      end;
  end;

  procedure do_right;
  begin
    save_cursor_pos;
    inc(x_pos);
    if x_pos > CHARPERLINE then
      begin
        x_pos:=1;
        inc(display_page);
        if display_page > 3 then display_page:=1;
        draw_display_titles;
        bottom_window;
        if rslt = 0 then show_log(rec,TRUE,TRUE);
      end;
    gotoxy(x_pos,y_pos);
  end;

  procedure do_left;
  begin
    save_cursor_pos;
    dec(x_pos);
    if x_pos < 1 then
      begin
        x_pos:=CHARPERLINE;
        display_page:=display_page - 1;
        if display_page < 1 then display_page:=3;
        draw_display_titles;
        bottom_window;
        if rslt = 0 then show_log(rec,TRUE,TRUE);
      end;
    gotoxy(x_pos,y_pos);
  end;

  procedure do_home;
  begin
    rec:=1; x_pos:=1; y_pos:=1;
    gotoxy(x_pos,y_pos);
    show_log(rec,TRUE,TRUE);
  end;

  procedure do_end;
  begin
    rec:=records; x_pos:=1; y_pos:=1;
    do_pagedown(1);
  end;

  procedure call_do_help;
  begin
    do_help;
    show_log(rec,TRUE,TRUE);
    status_window;
    bottom_window;
  end;

  procedure graph_init;
  var graphdriver:integer;
      errorcode:integer;
  begin
    if radio_type = 525 then exit;
    graphdriver:=detect;
    case graphdriver of
      CGA:     graphmode:=CGAHI;
      MCGA:    graphmode:=MCGAHI;
      EGA:     graphmode:=EGAHI;
      EGA64:   graphmode:=EGA64HI;
      EGAMONO: graphmode:=EGAMONOHI;
      IBM8514: graphmode:=IBM8514HI;
      HERCMONO:graphmode:=HERCMONOHI;
      ATT400:  graphmode:=ATT400HI;
      VGA:     graphmode:=VGAHI;
      PC3270:  graphmode:=PC3270HI;
    end;
    initgraph(graphdriver,graphmode,'');
    errorcode:=graphresult;
    if errorcode <> grok then exit;
    restorecrtmode;
    home;
  end;

  procedure goodbye; { clean up and leave }
  begin
    information_mode_off;
    sync_loglist;
    close(logbuf);
    put_reccnt;
    window(1,1,80,25);
    home;
    gotoxy(1,8);
    writeln(output,'Send comments and suggestions to:');
    writeln(output);
    writeln(output,'    Tom Whiteside (512) 258-5924');
    writeln(output,'    11505 Oak View');
    writeln(output,'    Austin, TX 78759');
  end;

begin
  inc_class:=FIXED_OFFSET;
  scan_threshold:=loglist.scan_threshold;
  last_log:=0;
  last_channel:= 2000; { set outside of normal range to start }
  enable_s_meter:=FALSE;
  prompt_num:=PAGE1;
  new_log(loglist.currentlog,rslt);
  graph_init;
  x_pos:=1; y_pos:=1;
  init_crt;
  gotoxy(x_pos,y_pos);
  do_unmark_all(TRUE);

  { init old receiver status to current radio settings }
  oldstat:=receiverstat;
end.