/* From G0/K8KA */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
int one_val(char *, int);
/* Removes repeated DASH packets from UOSAT-5 CCD images */
/* This happens on early images where the DASH resyncs and accepts
   duplicate packets. Run this program on the data after you have
	 removed the PACSAT File Header from the file, but before any
	 other processing.
*/
void main(int argc, char * argv[]){
	char * thing, * oldthing;
  int length=254;
	int got;
	int offset=0;
	int skip=0;
	FILE * fpin, *fpout, *trash;
 
	if (argc < 3){
		printf("usage: pac-out <input> <output> [<packet length> [<skip>]]\n");
		printf("Removes duplicate packets from early UO-5 CCD images.\n");
		printf("You should not need any arguments other than <input> and <output>,\n");
		printf("the two file names.\n");
		printf("G0/K8KA\n");
	}
	if (argc > 3)
		length=atoi(argv[3]);
	if (argc > 4)
		skip=atoi(argv[4]);
 
	printf("Checking for duplicate packets of length %d\n", length);
	thing = (char *) malloc(length);
	oldthing = (char *) calloc(length, 1);
	
	if (thing){
		fpin = fopen(argv[1], "rb");
		if (skip){
			printf("Skipping %d.\n", skip);
			fseek(fpin, skip, SEEK_SET);
		}
		else
			printf("Starting from beginning of file.\n");
 
		if (fpin){
			fpout=fopen(argv[2], "wb");
			if (fpout){
				while (!feof(fpin)){
					/* Read one frame */
					got = fread(thing, 1, length, fpin);
					/*printf("Read %d : ", got);*/
					if (got){
						if (memcmp(thing, oldthing, got))
							/* Not a duplicate of the previous frame */
							fwrite(thing, 1, got, fpout);
						else if ( one_val(thing, got) ){
							/* Is a duplicate, but contains all the same pixel value */
							/* So we keep it.                                        */
							fwrite(thing, 1, got, fpout);
							printf("Duplicate kept at %d.\n", offset);
						}
						else{
							/* Is a duplicate with differing pixels, so we throw it out. */
							printf("Duplicate rejected at %d.\n", offset);
						}
					}
					memcpy(oldthing, thing, got);
					/*printf("Packet: %d\r", offset);*/
					++offset;
				}
				fclose(fpout);
			}
			fclose(fpin);
		}
	  free(thing);
		free(oldthing);
	}
}
 
/* There may be some legitimate duplicate packets in the file. They
   will have all one value, e.g. all 0s. This function detects them
	 so that we can put them into the output file.
*/
int one_val(char *buf, int length){
	int i;
	for(i=1; i<length; i++){
		if (buf[i]!=buf[0])
			return 0;
	}
	return 1;
}
