
/*
  Fred Peachman KB7YW
  7186 Northview Drive
  Brookfield, OH 44403
  Thu  03-26-1992


  This not part of the NOS program. It can be compiled with

    tcc -mt -lt permssns

    or

    bcc -mt -lt permssns.c

  and produces a com file.

  There are three ways to use it (all from the MS-DOS command prompt):

  Method 1) example: "permssns 59"
  The permssns.com program is given an ftp permission number and
  produces the permissions granted as per mailbox.h AND files.h in
  PA0GRI NOS 2.0f_ss.

  Method 2) no arguments given, i.e. "permssns"
  If no permission number is entered at the command line, the program
  will prompt the user for the various permissions desired and
  compute the number needed to produce the result.

  Method 3) enter a user's login name, example: "permssns kb7yw"
  Enter an ftpuser login name like 'kb7yw'. If there is an /ftpusers
  file in the root of the  current working directory and it contains an
  entry for the user, his current /ftp permissions will be displayed.

 */
 #include <stdlib.h>
 #include <stdio.h>
 #include <conio.h>
 #include <bios.h>
 #include <ctype.h>
 #include <string.h>

 /* How many different types of privilege does parogi-nos define? */
 #define PRIV_TYPES 14
 /* keyboard key defs returned by Turbo C's bioskey(0) */
 #define SPACE  0x3920
 #define TAB    0x0f09
 #define ENTER  0x1c0d
 #define UP     0x4800
 #define DOWN   0x5000
 #define LEFT   0x4B00
 #define RIGHT  0x4D00

 /* from files.h in the NOS source code: */
 #define FTP_READ  1 /* Read files */
 #define FTP_CREATE  2 /* Create new files */
 #define FTP_WRITE 4 /* Overwrite or delete existing files */
 #define AX25_CMD  8 /* AX.25 gateway operation allowed */
 #define TELNET_CMD  16  /* Telnet gateway operation allowed */
 #define NETROM_CMD  32  /* NET/ROM gateway operation allowed */
 #define SYSOP_CMD 64  /* Remote sysop access allowed */
 #define EXCLUDED_CMD  128 /* This user is banned from the BBS */
 #define PPP_ACCESS_PRIV 0x0100  /* Priv bit for PPP connection */
 #define PPP_PWD_LOOKUP  0x0200  /* Priv bit for peerID/pass lookup */

 /* from mailbox.h in the NOS source code */
 #define NO_SENDCMD  1024  /* Disallow sending of mail */
 #define NO_READCMD  2048  /* Disallow reading of mail */
 #define NO_3PARTY 4096  /* Disallow 3th party mail to be send */
 #define IS_BBS    8192  /* This is a known BBS */

 struct privs {
  char *label;
  int spec;
  char *description;
 };

 struct privs priv[PRIV_TYPES] = {
  { "FTP_READ",            1,  "Read files"  },
  { "FTP_CREATE",          2,  "Create new files"  },
  { "FTP_WRITE",           4,  "Overwrite or delete existing files"  },
  { "AX25_CMD",            8,  "AX.25 gateway operation allowed" },
  { "TELNET_CMD",         16,  "Telnet gateway operation allowed"  },
  { "NETROM_CMD",         32,  "NET/ROM gateway operation allowed" },
  { "SYSOP_CMD",          64,  "Remote sysop access allowed" },
  { "EXCLUDED_CMD",      128,  "This user is banned from the BBS"  },
  { "PPP_ACCESS_PRIV",0x0100,  "Priv bit for PPP connection" },
  { "PPP_PWD_LOOKUP", 0x0200,  "Priv bit for peerID/pass lookup" },
  { "NO_SENDCMD",       1024,  "Disallow sending of mail"  },
  { "NO_READCMD",       2048,  "Disallow reading of mail"  },
  { "NO_3PARTY",        4096,  "Disallow 3th party mail to be send"  },
  { "IS_BBS",           8192,  "This is a known BBS" }
 };

 void report_perm(int, char *);
 void compute_permssns(void);
 int find_perm(char *);
 void press_any(void);

void close_down(int exitval)
{
  printf("\n\nPress any key to exit to MS-DOS....");
  bioskey(0);
  exit(exitval);
}

 main(int argc, char **argv)
 {
 unsigned int t, level;
 int permssn;
 char str[132];

  if (argc == 1)  {
    compute_permssns();
    exit(0);
  }

  for (t = 1; t < argc; t++)  {
    permssn = atoi(argv[t]);
    clrscr();
    if (permssn == 0) permssn = find_perm(argv[t]);
    else printf("\t\tpermission value %d:\n\n", permssn);
    if (permssn == -1) {
      press_any();
      continue;
    }
  printf("\n\t\t%-15s %-5s  %-34s","Label","Value","Description");
  printf("\n\t\t%-15s %-5s  %-34s\n","-----","-----","-----------");
    for (level = 0; level < PRIV_TYPES; level++)  {
      if (permssn & 1)  {
        printf("\t\t");
        report_perm(level,str);
        printf("%s\n", str);
      }
      permssn >>= 1;  /* Bring down (right-shift) the next bit in permssn */
    }
    if (t < argc - 1) press_any();
  }
  close_down(0);
 }

 void press_any(void)
 {
      printf("\n\n\t\tPress any key to continue..\n");
      bioskey(0);
 }

 void
 report_perm(level, str)
 int level;
 char *str;
 {
  struct privs *p;
  p = &priv[level];
  sprintf(str, "%-15s % 5d  %-34s", p->label,p->spec, p->description);
  return;
 }

 void compute_permssns(void)
 {
 int level;
 char str[132];
 int x, y[PRIV_TYPES], yend;
 int key;
 int perm = 0, bit;

  clrscr();
  printf("\n\
          /ftpusers permissions calculator for PA0GRI NOS ver. 2.0f");
  printf("\n\t");
  x = wherex();
  printf("Use SPACE BAR to turn on/off; TAB for next privilege; "
             "ENTER to quit\n\n");
  printf("\n\t\t%-15s %-5s  %-34s","Label","Value","Description");
  printf("\n\t-\t%-15s %-5s  %-34s","-----","-----","-----------");
  for (level = 0; level < PRIV_TYPES; level++)  {
    printf("\n\t\t");
    y[level] = wherey();
    report_perm(level, str);
    printf(str);
  }
  printf("\n\n\t");
  yend = wherey();

  gotoxy(x, y[0]);
  level = 0;

  do {
    key = bioskey(0);

    switch (key)  {
      case  SPACE:
        bit = 1 << level; /* priv bit we are changing */
        if (perm & bit) {
          perm &= (~bit);
          printf(" ");
        }
        else  {
          perm = perm | bit;
          printf("%c", '\xfe'); /*   */
        }
        gotoxy(x, yend);
        printf("/ftpusers permission level = % 6d\n", perm);
        gotoxy(x, y[level]);
        break;

      case  TAB:
      case DOWN:
      case RIGHT:
        level++;
        if (level == PRIV_TYPES) level = 0;
        gotoxy(x,y[level]);
        break;

      case  UP:
      case  LEFT:
        if (level == 0) level = PRIV_TYPES;
        level--;
        gotoxy(x,y[level]);
        break;

      case  ENTER:
        break;
    }
  } while (key != ENTER);
  gotoxy(x, yend);  /* Let the MS-DOS command prompt show up at the bottom  */
  return;
}

int find_perm(char *user)
{
FILE *ftpuser;
char line[132];
char *q, *p;
int perm = -1;

  if ((ftpuser = fopen("\\ftpusers", "r")) == NULL) {
    printf("No /ftpusers file present: cannot look up %s\n", user);
    return -1;
  }

  while (!(feof(ftpuser)))  {
    if (fgets(line, 131, ftpuser) == NULL) break;
    q = line;
    while (isspace(*q)) q++;
    if (*q == '#')  continue; /* remark */

    if ((p = strstr(q, user)) == NULL) continue;  /* should == q  */
    if (stricmp(p,q) != 0) continue; /* must be an exact match */

    /* Vee haf a vinner: get to pass  */
    while (!isspace(*q))  q++;
    while (isspace(*q))   q++;

    /* get to permitted subdir list */
    while (!isspace(*q))  q++;
    while (isspace(*q))   q++;

    /* get to permission value  */
    while (!isspace(*q))  q++;
    while (isspace(*q))   q++;
    perm = atoi(q);
    while ((*q != '\0') && (*q != '\n'))  q++;
    *q = '\0';  /* strip off the \n from the end of the line  */

    printf("\n\t\tIn file /ftpusers, the entry:\n");
    printf("\n\t\t\"%s\"\n\n", line);
    printf("\t\tgives user \"%s\" permission value %d\n", user, perm);
    printf("\t\twhich allows the following operations:\n");
    printf("\t\t-------------------------------------\n");
    break;
  }
  fclose(ftpuser);
  if (perm == -1)
    printf("\n\t\tIn file /ftpusers, user \"%s\" is NOT found\n", user);
  return perm;
}
