Following is a list of modifications for the Kenwood R-5000 general coverage
shortwave receiver, which I recently submitted to the MODS Database.
						-- Eric Roskos

Kenwood R5000 Modification Notes
By E. Roskos

The following describes some option jumpers and other features I have
found in the Kenwood R-5000 receiver. 

The R5000 is fairly delicate internally, so you should not undertake
these modifications if you are not fairly confident of your ability to
work with delicate electronics (or to repair it if necessary).  In
particular, the receiver's numerous circuit boards are connected
together with wire harnesses made of relatively fine and delicate wires. 
Also, some of the boards (particularly the IF board) have small
"daughter boards" containing small surface mount parts vertically
attached to the main board.  These are also fairly fragile, and caution
is required in handling them. 

EXPANSION FEATURES

The R5000 has six "Expansion Feature" options which are not documented
in the user manuals.  These are controlled by jumpers (actually diodes)
on the CPU board, which is attached to the back of the receiver's front
control panel, underneath a metal RF shield.  Unfortunately, you must
completely remove the receiver's covers, and unfasten the front panel,
in order to access these. 

Gaining Access to the Jumpers

To access the jumpers, remove the top and bottom covers of the receiver
by removing the eight silver screws which hold each cover in place.  Use
caution at this point, since the radio will be sitting only on its
internal chassis, and delicate parts will be exposed. 

Next, remove the four flat silver screws which were *under* the covers
(NOT the black screws that are visible with the covers on) which hold
the front panel onto the main chassis.  Be sure the receiver is sitting
on a solid table so that the front panel will not fall off when you do
this, since there are a large number of wire harnesses connecting to the
CPU board.  Very carefully pull the front panel forward and rotate it so
that you can get access to the back of the front panel. 

Loosen the five small metal screws (two at the top, three at the bottom)
which hold the RF shield in place over the CPU board.  The holes in the
shield the screws go into are slots, which allow you to slide the shield
off without removing the screws completely.  This is fortunate since the
screws are fairly small.  Remove the RF shield. 

Changing The Options

Locate the row of approximately eight small, vertically-mounted option
diodes on the CPU board.  They are labelled D65 through D72, and are
located near and perpendicular to six-pin inline connectors 54 and 55,
and parallel (and directly adjacent to) eight-pin connector 53.  Behind
connector 53, you may see another row of option jumpers; in my R5000,
most of the diodes in this row were not installed.  They are numbered
D73-D79.  Note that D65-D72, D73-D79, and connector 53 (which goes to
the front panel keyboard matrix) are all read by IC53, an 82C55 PIO,
which strobes the cathodes of one of the three rows of diodes and then
reads the anode side of each diode in the row in parallel; it is pulled
up by resistor pack R850 unless the diode is connected, in which it is
pulled down by the (negative-going) strobe. 

You enable one of the options by cutting the top loop of the wire coming
out of the anode end of the diode (remember that these are vertically
mounted diodes, so the end which is sticking up forms a loop).  It is
suggested that you just cut through the wire carefully, and bend it
slightly to one side; then if you want to disable the option in the
future, you can carefully re-solder the connection without having to
solder a new diode onto the board. 

The jumpers we are concerned with are D65-D72.  Note that the other row
of diodes (D73-D79) are also option jumpers, but they are not documented
in the R5000's technical manual; the manual's parts list simply says
that D73 is installed in Australia, and D74-D76 are installed in Europe. 
Only one of these (I think D79) was installed in my receiver; and the
parts list doesn't say what it is for.  I'm interested in hearing from
anyone who may know what these undocumented jumpers do. 

The meaning of each jumper is as follows:

D65: Selects whether the display will show in 10 Hz increments (diode
present) or in 100 Hz increments (diode absent).  As shipped, the diode
is present, which is probably what you'll want since it gives a higher
resolution to the display. I don't know if it affects tuning; the radio
does tune in 10 Hz increments with the diode installed.

D66: Controls whether the mode buttons will generate a morse-code letter
through the speaker when depressed (diode present) or will simply beep
when depressed (diode absent).  As shipped, the diode is present, and
morse-code letters are generated through the speaker. 

D67: Controls whether FM mode will step in 2.5 kHz increments (present)
or 500 Hz increments (absent), when you have the front-panel step switch
"on" (STEP displayed on the display).  This applies to HF-band FM;
VHF-band FM (with the VC20 installed) already stepped in 500 Hz
increments.  As shipped, the diode is present, and steps are in 2.5 KHz
increments. 

D68: Controls the "BUSY STOP" feature.  With the diode present, scanning
will stop when the BUSY light comes on only in AM and FM modes.  With
the diode absent, scanning will stop when the BUSY light comes on in all
modes.  Note that this is the modification some radio stores sell as an
extra feature, by connecting the "dimmer" switch to the option diode so
that you can turn this on or off from the front panel. 

D69: "Memory search" feature.  This is (in my opinion) the best of the
extended options.  The diode is shipped present; as shipped, the memory
search feature is disabled.  If you remove the diode, you enable the
feature.  With the feature enabled, when you press the orange M.IN key
the first time (to allow you to select which memory you want to store a
frequency in), the silver "1MHz Down/Up" buttons on the front panel
allow you to step through the set of *unused* memory positions.  This
lets you select a memory to store in without having to scan through all
the ones you've already stored into.  The front panel knob still scans
the whole set of memories, so you don't lose any functionality when you
enable this feature.  Also, when you are in the memory mode (i.e., the
mode in which M.CH is displayed on the display) and not storing into
memory (normal operating mode), the Down/Up buttons step through the set
of *used* memories, skipping any unused ones. 

D70-71: These jumpers are currently unused (according to the manual).

D72: I have not tried this option.  According to the manual, if you
remove the jumper, you are prevented from storing into a memory channel
which has a frequency stored into it.  I don't know whether this
completely disables all M.IN operations, or just the attempts to modify
used channels; or whether there is a way to get around this (e.g.,
whether the CLEAR button still works). 

That's all the options that are documented.  I've tried all except D65
and D72; I didn't want to take the radio apart multiple times to try
those, since they didn't sound like particularly desirable features.  If
anybody tries them (particularly D72) and finds details of how they
work, let me know and I'll update the description. 

OTHER NOTES

Note that the Serial Port option, which is fairly expensive, actually
consists simply of two readily-available ICs: an 8251 (the manual lists
a uPD8251AFC), and a 4040.  The 8251 UART plugs into the IC54 socket on
the CPU board (which you access as described above); the 4040 plugs in
to the IC55 socket next to it.  Another IC to know about in connection
with this option is IC56, a 7404 inverter, which is what drives the
serial port socket in back of the radio.  According to the manual, the
pin assignment on this socket is as follows:

	3 - RXD (in)
	4 - TXD (out)
	5 - CTS (in)
	6 - RTS (out)

I haven't actually tried these to verify them.  Also, there is a CMOS
82C51 available, and an NMOS 8251.  According to the manual, the NMOS
8251 is used; although most of the other parts on the board are CMOS.  I
don't know whether the IC kits they provide actually supply the CMOS or
NMOS version. 

In addition to these IC kits, to connect to an RS232 interface on a
computer you have to have an external converter to convert from the TTL
voltage levels coming out of (and going into) the socket, to the RS232
levels.  That is the other part of the serial port kit (the "IF232"). 
Don't connect an RS232 port directly to the serial port socket on the
back of the radio! You will (at least) damage IC56. 

---

Note that there are two potentiometers (VR2 and VR3) mounted on top of a
small PC board attached to the top of the control panel (accessible when
you remove the top cover).  VR3 controls how long it takes for scan to
resume after the BUSY light goes off, and VR2 controls the scan speed. 
From the design, it looks like maybe the person who designed the
receiver intended for these to be external controls, and then they put
them inside when they packaged the receiver. 

What's interesting about these controls is that they, and the IF shift
control (and another control used in aligning the receiver) are read
through an A/D converter.  They actually serve as digital inputs to the
microprocessor.  It's unfortunate that they didn't use one of the
channels on the A/D converter to read the signal strength meter (so you
could find the signal strength (AGC level) through the serial port)
instead of using them for these relatively minor functions.  EEB in
Vienna sells a modification for the R5000 which allows their CRIS
computer interface to read the AGC level, but I don't know how it works. 

---

One other thing to note involves installing the optional filters. 
First, I found that the AM filter which came with the radio introduced a
whistle into most AM signals.  I replaced it with the optional AM
filter, and it eliminated the whistle (and gave a "fuller" sound to the
AM). 

Also, note that if you install other filters, you have to install them
"in order" -- in other words, the narrowest has to go in the N position,
the next narrowest in the M1 position.  Note that the M2 position is
already occupied (as shipped) by a high-quality SSB filter.  The reason
they have to be in order is that as you select narrower and narrower
filters, the wider filters remain in the circuit, so if you put a
narrower filter in M1 than you have in N, it will stay enabled even when
you select the wider filter, causing the wider filter to have no effect. 

It's a little unfortunate that it works that way, since the M1 filter is
only enabled by the switch (not in AUTO mode), which would be ideal for
the very-narrow YK88CN filter.  I have identified a modification to make
M1 be selected only when the switch is in the M1 position; it involves
disconnecting 1/2 of a dual diode and soldering a diode between two
points in the filter selection circuit, but I haven't tested it since
I don't have that many filters in my radio.

--

Note that the YK88CN filter is very useful if you are an RTTY
enthusiast; it is a worthwhile option.  You can use it to receive VFT on
a PK232 (so you don't have to pay $1000 for an M7000), and it helps in
tuning AMTOR when there is a lot of adjacent channel activity.  The IF
shift shifts the frequency of the FSK tone produced in this mode, which
is necessary to get it tuned properly for the PK232. 

-- 
Eric Roskos, IDA (roskos@CS.IDA.ORG or Roskos@DOCKMASTER.ARPA)

