'program REBUILD.BAS

'
'*****************************************************************
'*                                                               *
'*  This program is used with SWEEP.BAS to rebuild the binary    *
'*  tree file and the "configuration" file.  I made the          *
'*  assumption that if the binary tree file got "trashed" there  *
'*  was a good likelihood that the "configuration" file would    *
'*  also be trashed.  The configuration file contains the start- *
'*  information for the contest user as well as the total number *
'*  of sections worked and an array to identify which of those   *
'*  sections were worked and which were not.  ALB 7/29/90        *
'*                                                               *
'*****************************************************************
'
DEFINT A-Z

TYPE RecordType
   Band AS STRING * 3
   Date AS STRING * 8
   TimeOnOff AS STRING * 4
   Time AS STRING * 4
   SentNum AS STRING * 5
   RCVDNum AS STRING * 4
   Precedence AS STRING * 1
   RCVDCall AS STRING * 11
   CHECK AS STRING * 2
   Section AS STRING * 3
   CarReturn AS STRING * 1
   LineFeed AS STRING * 1
END TYPE

TYPE TreeType
   LeftPointer AS INTEGER
   RightPointer AS INTEGER
   CallSign AS STRING * 11
END TYPE

DECLARE SUB AnyKey ()
DECLARE SUB CleanLine (Row, Col)
DECLARE SUB Filter (Row, Col, IN$, CL)
DECLARE SUB LoadTree (SS AS RecordType, Counter, C$, SSTree AS TreeType)

DIM SS AS RecordType
DIM SSTree AS TreeType

CLS
LOCATE 10, 20

PRINT "File Prefix: ";
Col = POS(0)                                                      'save current column
Row = CSRLIN                                                      'save current row
CL = 25                                                           'allow 25 characters
IN$ = ""                                                          'input string for callsign
DO WHILE IN$ = ""
   CALL Filter(Row, Col, IN$, CL)

   FileName$ = IN$ + ".SWS"                                       'main data file
   FileName1$ = IN$ + ".DAT"                                      'binary tree file
   FileName2$ = IN$ + ".CFG"                                      'configuration file

LOOP

CLS
LOCATE 12, 20

PRINT "REBUILDING THE BINARY TREE FILE"

OPEN FileName$ FOR RANDOM AS #1 LEN = LEN(SS)                     'sequential contact file
OPEN FileName1$ FOR RANDOM AS #2 LEN = LEN(SSTree)                'binary tree file

QSOCounter = (LOF(1) \ LEN(SS))

FOR N = 1 TO QSOCounter
   GET #1, N, SS                                                  'get a record from original file
   C$ = SS.RCVDCall                                               'set C$ equal to the callsign
   CALL LoadTree(SS, N, C$, SSTree)                               'see where it fits in the tree file
NEXT N

READ NumberOfSections                                             'currently 77 sections
DIM CallAreas(10)                                                 '10 call areas + Canada
DIM Sections$(NumberOfSections)                                   'physical data
DIM SectionCounter(NumberOfSections)                              '"0" means not yet worked
                                                                  '"1" used to show a qso
FOR J = 0 TO 10
   READ CallAreas(J)                                              'how many sections in each area?
NEXT J

FOR N = 1 TO NumberOfSections
   READ Sections$(N)                                              'read sections into array
   SectionCounter(N) = 0                                          'initialize the tracking data
NEXT N

SectionsWorked = 0                                                'Number of sections worked
'************************************************
'*           SECTION LIST                       *
'************************************************
'
'the first entry in the first data line below
' identifies the total number of sections
'
'the numbers following show how many sections are in each call
' area beginning with call area zero and ending with the VEs
'
DATA 77,8,7,5,4,11,8,10,10,3,3,8
DATA "CO ","IA ","KS ","MN ","MO ","NE ","ND ","SD "
DATA "CT ","EMA","ME ","NH ","RI ","VT ","WMA"                          : REM 1's
DATA "ENY","NLI","NNJ","SNJ","WNY"                                      : REM 2's
DATA "EPA","DE ","MDC","WPA"                                            : REM 3's
DATA "AL ","GA ","KY ","NC ","NFL","PR ","SC ","SFL","TN ","VA ","VI "  : REM 4's
DATA "AR ","LA ","MS ","NM ","NTX","OK ","STX","WTX"                    : REM 5's
DATA "EB ","LAX","ORG","SB ","SCV","SDG","SF ","SJV","SV ","PAC"        : REM 6's
DATA "AK ","AZ ","EWA","ID ","MT ","NV ","OR ","UT ","WWA","WY "        : REM 7's
DATA "MI ","OH ","WV "                                                  : REM 8's
DATA "IL ","IN ","WI "                                                  : REM 9's
DATA "MAR","PQ ","ON ","MB ","SK ","AB ","BC ","VE8"                    : REM VE's

CLS

DO
   LOCATE 7, 20

   SPREC$ = ""                                                    'get user Precedence
   DO                                                             ' A = low power
      LOCATE 10, 20                                               ' B = high power
      PRINT "Precedence? ";                                       ' Q = QRP
      Row = CSRLIN
      Col = POS(0)
      IN$ = ""
      CL = 1
      CALL Filter(Row, Col, IN$, CL)
      IF IN$ <> "" THEN SPREC$ = IN$
      valid = INSTR("ABQ", SPREC$)
      IF valid = 0 THEN
         LOCATE 23, 20                                            'invalid entry
         BEEP
         PRINT "Precedence must be A, B, or Q      Any key to continue...";
         CALL AnyKey
         Row = 23
         Col = 20
         CALL CleanLine(Row, Col)                                 'erase error message
         LOCATE 10, 32
         PRINT SPACE$(3)                                          'erase original entry
         SPREC$ = ""
      END IF
   LOOP UNTIL SPREC$ <> ""                                        'do this until the user enters
                                                                  ' a legal entry
   ID$ = ""
   DO                                                             'get the sending callsign
      LOCATE , 20
      PRINT "Call ?      ";
      Row = CSRLIN
      Col = POS(0)
      IN$ = ""
      CL = 11
      CALL Filter(Row, Col, IN$, CL)
      IF IN$ <> "" THEN ID$ = IN$
      IF ID$ = "" THEN                                            'no callsign entered
         BEEP
         LOCATE 11, 20
      END IF
   LOOP UNTIL ID$ <> ""

   CK$ = ""                                                       'get the check number - the two
   DO                                                             ' digits that represent the year
      LOCATE , 20                                                 ' the user was first licensed
      PRINT "Check ?     ";
      Row = CSRLIN
      Col = POS(0)
      IN$ = ""
      CL = 2
      CALL Filter(Row, Col, IN$, CL)
      IF IN$ <> "" THEN CK$ = IN$
      IF CK$ = "" THEN
         BEEP
         LOCATE 12, 20
      END IF
   LOOP UNTIL CK$ <> ""

   SEC$ = ""                                                      'get the ARRL Section code
   DO
      LOCATE , 20
      PRINT "Section ?   ";
      Row = CSRLIN
      Col = POS(0)
      IN$ = ""
      CL = 3
      CALL Filter(Row, Col, IN$, CL)
      IF IN$ <> "" THEN SEC$ = IN$

      IF LEN(SEC$) < 3 THEN                                       'force section to three characters
         SEC$ = SEC$ + " "
      END IF

      FOR N = 1 TO NumberOfSections                               'check all the sections
         IF SEC$ = Sections$(N) THEN                              ' and make sure this entry
            EXIT DO                                               ' is valid
         END IF
      NEXT N
      BEEP
      LOCATE 23, 20
      PRINT "INVALID SECTION!!   Any key to continue..."
      CALL AnyKey
      Row = 23
      Col = 20
      CALL CleanLine(Row, Col)
      SEC$ = ""
      LOCATE 13, 32
      PRINT SPACE$(3);                                            'erase the incorrect entry
      LOCATE 13, 20
   LOOP UNTIL SEC$ <> ""

   MO$ = ""                                                       'get the operating mode
   DO                                                             ' either CW or SSB
      LOCATE , 20
      PRINT "Mode ?      ";
      Row = CSRLIN
      Col = POS(0)
      IN$ = ""
      CL = 3
      CALL Filter(Row, Col, IN$, CL)
      IF IN$ <> "" THEN MO$ = IN$
      IF MO$ <> "CW" THEN
         IF MO$ <> "SSB" THEN
            MO$ = ""
            LOCATE 23, 20
            BEEP
            PRINT "Only CW or SSB are valid modes.   Any key to continue...";
            CALL AnyKey
            Row = 23
            Col = 20
            CALL CleanLine(Row, Col)
            LOCATE 14, 32
            PRINT SPACE$(4);                                      'erase invalid entry
         END IF
      END IF
   LOOP UNTIL MO$ <> ""

   PRINT
   LOCATE , 20
   PRINT "All OK? (Y/N) ";
   COLOR 23: PRINT CHR$(254); : COLOR 7                           'flashing cursor
   RE$ = ""
   WHILE RE$ = ""
      RE$ = UCASE$(INKEY$)
      IF RE$ = "Y" THEN EXIT DO
   WEND
   CLS
LOOP

CLS
LOCATE 12, 20
PRINT "REBUILDING CONFIGURATION FILE"

FOR N = 1 TO QSOCounter
   GET #1, N, SS                                                  'get a record from original file
   C$ = SS.Section                                                'set C$ equal to the section
   K = 1                                                          'keep track of the sections
   DO
      FOR I = 0 TO 10                                             'work through the call areas
         FOR J = 1 TO CallAreas(I)
            IF C$ = Sections$(K) THEN                             'valid section?
               IF SectionCounter(K) = 0 THEN                      'worked yet?
                  SectionCounter(K) = 1                           ' if not - show it as worked
                  SectionsWorked = SectionsWorked + 1             ' and increment sections worked
               END IF
               EXIT DO
            END IF
            K = K + 1                                             'haven't found it yet
            IF K > NumberOfSections THEN                          'run out of sections?
               EXIT DO                                            '    skip town
            END IF
         NEXT J                                                   'keep looking
      NEXT I
   LOOP
NEXT N

OPEN FileName2$ FOR OUTPUT AS #3

WRITE #3, SPREC$, ID$, CK$, SEC$, MO$, SectionsWorked
FOR N = 1 TO NumberOfSections
   WRITE #3, SectionCounter(N)
NEXT N

CLOSE

'************************************************
'*                                              *
'*  SUB AnyKey                                  *
'*                                              *
'*  This sub procedure waits for any key to     *
'*  be pressed to continue processing.          *
'*                                              *
'************************************************
'
SUB AnyKey

DO
LOOP UNTIL LEN(INKEY$)

END SUB

'*******************************************
'*                                         *
'*  SUB CleanLine                          *
'*                                         *
'*  This sub procedure is used to "erase"  *
'*  old data from the input area at the    *
'*  bottom of the data entry screen.       *
'*                                         *
'*******************************************
'
SUB CleanLine (Row, Col)

LOCATE Row, Col
PRINT STRING$(79 - Col, 32)        'print a bunch of spaces

END SUB

'*************************************************
'*                                               *
'*  SUB Filter                                   *
'*                                               *
'*  This sub procedure "filters" the various     *
'*  inputs.  Parameters are passed from the      *
'*  calling area.  This technique is a much      *
'*  cleaner method than a plain old input!       *
'*                                               *
'*************************************************
'
SUB Filter (Row, Col, IN$, CL)

BUFF$ = SPACE$(1)                                                 'helps "erase" the input when backspace is used
CRSR$ = CHR$(254)                                                 'prints a fake cursor
Limit = CL                                                        'maximum size of input string
L = 1                                                             'extended/normal key identifier

DO
   LOCATE Row, Col
   TempChar$ = UCASE$(INKEY$)                                     'change to upper case
   CL = LEN(IN$)                                                  'length of string

   IF TempChar$ = CHR$(8) THEN                                    'check backspace
      CL = CL - 1                                                 'decrease string
      TempChar$ = ""
   ELSEIF TempChar$ = CHR$(13) THEN                               'return is terminator
      EXIT DO
   END IF

   IF TempChar$ <> "" THEN                                        'filter callsign
      IF Limit = 11 THEN
         valid = INSTR("ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890@^/", TempChar$)
         IF valid = 0 THEN
            BEEP
            TempChar$ = ""
         END IF
      ELSEIF Limit = 1 THEN                                       'filter VDISK or
         valid = INSTR("ABCDEFQ", TempChar$)                      'precedence (A,B, or Q)
         IF valid = 0 THEN
            BEEP
            TempChar$ = ""
         END IF
      ELSEIF Limit = 2 THEN                                       'filter check number or
         IF Row = 14 THEN                                         'mode
            valid = INSTR("BCSW", TempChar$)
            IF valid = 0 THEN
               BEEP
               TempChar$ = ""
            END IF
         ELSE
            valid = INSTR("1234567890", TempChar$)
            IF valid = 0 THEN
               BEEP
               TempChar$ = ""
            END IF
         END IF
      ELSEIF Limit = 3 THEN                                       'filter section or band
         IF Row = 23 THEN
            valid = INSTR("1245680", TempChar$)                   'filter band
            IF valid = 0 THEN
               BEEP
               TempChar$ = ""
            END IF
         ELSE                                                     'filter section
            valid = INSTR("ABCDEFGHIJKLMNOPQRSTUVWXYZ8", TempChar$)
            IF valid = 0 THEN
               BEEP
               TempChar$ = ""
            END IF
         END IF
      ELSEIF Limit = 25 THEN                                      'filter filenames
         valid = INSTR("ABCDEFGHIJKLMNOPQRSTUVWXYZ/\0123456789:", TempChar$)
         IF valid = 0 THEN
            BEEP
            TempChar$ = ""
         END IF
      END IF
   END IF

   IF CL > Limit THEN                                             'string too long?
      CL = Limit
   ELSEIF CL < 0 THEN                                             'string too short?
      CL = 0
   END IF

   IN$ = LEFT$(IN$, CL) + TempChar$                               'build string
   CL = CL + 1                                                    'increase string
   PRINT IN$;                                                     'echo it
   LOCATE Row, Col + CL - 1
   COLOR 23
   PRINT CRSR$; BUFF$                                             'print some kind of cursor
   COLOR 7
LOOP
LOCATE Row, Col + CL                                              'the BUFF$ is a blank at the end
PRINT BUFF$                                                       'of the temporary string - if the
                                                                  'user backs up, all extra junk gets "erased"
END SUB

SUB LoadTree (SS AS RecordType, Counter, C$, SSTree AS TreeType)

'*******************************************************************
'*                                                                 *
'*  NOTE:  This routine was originally written to be used as both  *
'*         a dupe search routine and to locate where a callsign    *
'*         should be placed in the binary tree.  In this context   *
'*         where it is being used to rebuild a trashed tree file,  *
'*         there won't be any dupes - they've already been ex-     *
'*         cluded during the original pass.  It shortens up the    *
'*         routine just a hair.                                    *
'*                                                                 *
'*******************************************************************
'
IF Counter = 1 THEN                                               'first record
   SSTree.CallSign = C$                                           'set tree callsign equal to C$
   SSTree.LeftPointer = 0                                         'leftpointer set to "NULL"
   SSTree.RightPointer = 0                                        'rightpointer set to "NULL"
   PUT #2, 1, SSTree                                              'write it into the tree file
   EXIT SUB                                                       'go get more records
END IF

firstempty = LOF(2) \ (LEN(SSTree)) + 1                           'the next "empty" location is equal
treecounter = 1                                                   '  to total file size divided by
pointer = 1                                                       '  the size of each record
DO
   treecounter = pointer
   GET #2, treecounter, SSTree                                    'get a record from the tree file
   IF C$ < RTRIM$(SSTree.CallSign) THEN                           'is the callsign less than the one in the record?
      pointer = SSTree.LeftPointer                                'Yes - set leftpointer and keep looking
      direction = 1
   ELSE
      pointer = SSTree.RightPointer                               'No - set rightpointer and keep looking
      direction = 2
   END IF
LOOP UNTIL pointer = 0                                            'once we reach a "NULL" pointer we have
                                                                  '  found the last record on that side
IF direction = 1 THEN                                             '  of the tree (or node)
   SSTree.LeftPointer = firstempty                                'update the pointers in the
ELSE                                                              '  existing tree record to indicate
   SSTree.RightPointer = firstempty                               '  whether the next record is to the
END IF                                                            '  left or to the right
PUT #2, treecounter, SSTree                                       're-write the existing record back into the tree file
SSTree.CallSign = C$                                              'new record callsign equals C$
SSTree.LeftPointer = 0                                            'since it's a new record set "NULL" leftpointer
SSTree.RightPointer = 0                                           'since it's a new record set "NULL" rightpointer
PUT #2, firstempty, SSTree                                        'write it into the tree file

END SUB

